(*
  Una cosa es un objeto que puede guardarse.
  Una TListaDeCosas, es una lista de cosas
*)

// Este define, permite que al leer se escriban en consola la clase y el nombre
// de las cosas leídas si son del tipo TCosaConNombre. Sirve a los efectos de
// debuggin cuando se tranca durante la lectura de la sala de juegos.
{xDEFINE WRLN_COSAS_CON_NOMBRE_LEIDAS}

unit ucosa;

interface

uses
  SysUtils, Classes, Dialogs, xmatdefs, ufechas, typinfo,
  {$IFDEF CASOS_MACROVARS} uparametros, {$ENDIF}
  contnrs, RegExpr, uCosaHashList, uConversions,
  uauxiliares,
  uevaluador,
  uversion_architexto,
  {$IFDEF MISION_IMPOSIBLE}
  unitLecturaArchivo,
  {$ENDIF}
  uconstantesSimSEE;

resourcestring
  rs_NoSeEncuentraArchivo = 'No se encuentra el archivo: ';
  rs_ElArchivo = 'El archivo';
  rs_VersionPosterior =
    'esta escrito con una versión posterior a la instalada. Debe actualizar el software.';
  rs_NoInstanciasDe = 'No hay instancias de cosas de la clase';


const
  formatoReales = ffFixed;

type

  {dfusco@20150507}
  TBlackHole=class
  public
    TheHole:integer;
  end;

  TArchiTexto = class; // definido más adelante.

  TSIUD = (TSELECT, TINSERT, TUPDATE, TDELETE);

  TValidation=class;
  TListValidation=class;

  DataTypeOfCosa = class of TDataTypeOfCosa;

  TDataTypeOfCosa = class;
  TDataColumnListOfCosa = class;
  TDataColumnOfCosa = class;

  TDataCellOfCosa = class;
  //TListDataCellOfCosa=class;

  //TListDataRowOfCosa=class;
  TDataRowOfCosa = class;

  //TListDataTableOfCosa=class;
  TDataTableOfCosa = class;
  TDataTableOfListOfCosa = class;

  TDataVTableOfCosa = class;

  TDataStore = class;

  // Define la referenica a un archivo externo.
  // se utiliza para centralizar las dependencias.
  TArchiRef = class
  private
    archiRefStr: string;
    function get_archi: string;
    procedure set_archi(const archi_str: string);

  public
    constructor Create(archiRef_: string);
    procedure Free; virtual;

    // permite pasar carpeta en cuyo caso si no encuentra el archivo
    // con la archiRefStr original y sino busca en la lista_caminos
    function testear: boolean;
    property archi: string read get_archi write set_archi;
  end;


  // Clase de todas las cosas que usamos en una sala.

  { TCosa }

  TCosa=class;
  TClaseDeCosa = class of TCosa;

  TCosa = class
  public
  var
 {$IFDEF CASOS_MACROVARS}
    par_lst: TListaDeParametros;
 {$ENDIF}

    //dfusco@20150422
    id:integer;

    capa: integer;

    constructor Create; overload; virtual;
    constructor Create(capa: integer); overload; virtual;
    constructor Create_ReadFromText(f: TArchiTexto); virtual;

    procedure WriteToText(f: TArchiTexto); virtual;

    // crea el archivo y lo sobreescribe si existe y llama a WriteToText
    procedure WriteToArchi(archi: string);


    // retorna el texto como se guarda en el archivo de texto de  la sala.
    function asMemo: TStrings;
    class function Create_FromMemo(const memo: TStrings): TCosa;
    function Create_Clone: TCosa; virtual;

    // obtiene el string infoAd pero lo recorta a 20 caracteres.
    function InfoAd_20: string;

    function InfoAd_: string; virtual;
    class function DescClase: string; virtual;
    procedure Free; virtual;

    function Apodo: string; virtual;
    // en las cosas con nombre retorna el nombre, en las sin nombre retonra '-'

    procedure ActivarCapas(const capasActivas: TDAOfNInt); virtual;

    class function CreateDataConversionList: TListDataConversionOfCosa; virtual;

    //Al comienzo del programa se crea la BD con las tablas en la última versión (-2).
    //Luego cuando se lee el archivo se crea una tabla en la versión de éste (DataTableAux) y se convierte a la última versión.
    class function CreateDataColumnList(xClaseDeCosa: TClaseDeCosa; xVersion: integer = -2): TDataColumnListOfCosa; virtual; overload;

    // Procedimiento que se debera ejecuar luego de creada la Instancia
    // desde un TDataStore.
    procedure AfterInstantiation(); virtual;

    function Validate():boolean; virtual;

  end;



  TCualquierClase = class of TObject;

  TDAOfTCosa = array of TCosa;


  { TListaDeCosas }

  TListaDeCosas = class(TCosa)
  protected
    lst: TList;
    function getItem(i: integer): TCosa;
    procedure setItem(i: integer; cosa: TCosa);
    function getCapacity: integer;
    procedure setCapacity(newCapacity: integer);
  public

    {dfusco@20141029
     Se agrega atributo que representa el largo de la lista.}
    n: integer;

    //class var itemsType: ClassType;

    idCarpeta: string; // identificador para salvar en carpeta
    constructor Create(capa: integer; idCarpeta: string);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    function Add(cosa: TCosa): integer; virtual;

    //Si freeElementos = true llama a ClearFreeElementos, sino a Clear,
    //luego hace add de todas las cosas en listaDeCosas
    procedure ponerIgualA(listaDeCosas: TListaDeCosas; freeElementos: boolean);
    procedure ActivarCapas(const CapasActivas: TDAOfNInt);
    procedure Free; override;
    //Libera la lista sin destruir los objetos que contiene
    procedure FreeSinElemenentos;
    function Count: integer;
    procedure insert(indice: integer; cosa: TCosa);
    function Remove(cosa: TCosa): integer;
    procedure Delete(indice: integer);
    procedure Pack;
    procedure Exchange(indice1, indice2: integer);
    function IndexOf(cosa: TCosa): integer;
    function replace(cosaARemplazar, cosaNueva: TCosa): integer;
    procedure Clear;
    procedure ClearFreeElementos;
    procedure Sort(Compare: TListSortCompare);
    function getList: TList;
    function toArray: TDAOfTCosa;
    property Capacity: integer read getCapacity write setCapacity;
    property items[i: integer]: TCosa read getItem write setItem; default;

    class function CreateDataColumnList(xClaseDeCosa: TClaseDeCosa=nil; xVersion: integer = -2): TDataColumnListOfCosa;
      override;

  end;

(* La función ProcesoLineaRes puede retornar laguno de los siguientes
valores. No se prevé un código de error pues la función lanza
excepciones con cualquier error que detecte. *)
  TResultadoProcesoLinea = (
    CTPLR_CONTINUAR,   // continuar procesando
    CTPLR_FIN_OBJETO,  // se llegó al fin del objeto (cerrar la lista)
    CTPLR_ABRIR_NUEVO_OBJETO); // Se encontró un objeto abrir otra lista

  TTipoCampo = (
    CTC_NReal,
    CTC_String,
    CTC_ArchRef,
    CTC_NInt,
    CTC_NCardinal,
    CTC_Boolean,
    CTC_DAOfNReal,
    CTC_DAOfNInt,
    CTC_DAOfBoolean,
    CTC_DAOfString,
    CTC_StringList,
    CTC_Fecha,
    CTC_TDataRowOfCosa,
    CTC_Cosa,
    CTC_Referencia);


  TProcFijarValorPorDefecto = procedure(const nombre: string;
    pVal: Pointer; tipo: TTipoCampo; const klinea: integer);

  TCampoParaLeer = class
  public
    nombre: string;
    tipo: TTipoCampo;
    pVal: pointer;
    proc_vpd: TProcFijarValorPorDefecto;
    referente: TCosa;
    flgError: integer; // 0 es showmessage, 1 no muestra nada
    constructor Create(const Nombre: string; pVal: Pointer; Tipo: TTipoCampo;
      proc_vpd: TProcFijarValorPorDefecto); overload;
    constructor Create(const Nombre: string; pVal: Pointer; Tipo: TTipoCampo;
      proc_vpd: TProcFijarValorPorDefecto; flgError: integer); overload;
    constructor Create(const Nombre: string; pVal: Pointer; Tipo: TTipoCampo;
      proc_vpd: TProcFijarValorPorDefecto; referente: TCosa); overload;
    procedure Ejecutar_vpd(const klinea: integer);
  end;

  ExcepcionNoSePidioElCampo = class(Exception)
  end;

  ExcepcionResolverCampoValor = class(Exception)
  end;

  ExcepcionClaseNoRegistrada = class(Exception)
  end;

  { TListCamposParaLeer }

  TListCamposParaLeer = class(TList)
  public

    DataRow: TDataRowOfCosa;
    cosa: TCosa;

    lecturaRetrasada, leiFinDeObjeto: boolean;
    constructor Create(IdentificadorDeLaClaseDelObjeto: string);
    procedure Esperar_NReal(const Nombre: string; var Variable: NReal;
      proc_vpd: TProcFijarValorPorDefecto);
    procedure Esperar_String(const Nombre: string; var Variable: string;
      proc_vpd: TProcFijarValorPorDefecto);
    procedure Esperar_ArchRef(const Nombre: string; var Variable: TArchiRef;
      proc_vpd: TProcFijarValorPorDefecto);
    procedure Esperar_NInt(const Nombre: string; var Variable: integer;
      proc_vpd: TProcFijarValorPorDefecto);
    procedure Esperar_NCardinal(const Nombre: string; var Variable: cardinal;
      proc_vpd: TProcFijarValorPorDefecto);
    procedure Esperar_Boolean(const Nombre: string; var Variable: boolean;
      proc_vpd: TProcFijarValorPorDefecto);
    procedure Esperar_DAOfNReal(const Nombre: string; var Variable: TDAOfNReal;
      proc_vpd: TProcFijarValorPorDefecto);
    procedure Esperar_DAOfNInt(const Nombre: string; var Variable: TDAOfNInt;
      proc_vpd: TProcFijarValorPorDefecto);
    procedure Esperar_DAOfBoolean(const Nombre: string; var Variable: TDAOfBoolean;
      proc_vpd: TProcFijarValorPorDefecto);
    procedure Esperar_DAOfString(const Nombre: string; var Variable: TDAOfString;
      proc_vpd: TProcFijarValorPorDefecto);
    procedure Esperar_StringList(const Nombre: string; var Variable: TStringList;
      proc_vpd: TProcFijarValorPorDefecto);
    procedure Esperar_Fecha(const Nombre: string; var Variable: TFecha;
      proc_vpd: TProcFijarValorPorDefecto); overload;
    procedure Esperar_Fecha(const Nombre: string; var Variable: TFecha;
      proc_vpd: TProcFijarValorPorDefecto; flgError: integer); overload;
    procedure Esperar_TDataRowOfCosa(const nombre: string;
      var variable: TDataRowOfCosa; proc_vpd: TProcFijarValorPorDefecto);
    procedure Esperar_Cosa(const Nombre: string; var Variable: TCosa;
      proc_vpd: TProcFijarValorPorDefecto);
    procedure Esperar_Referencia(const Nombre: string; var Variable: TCosa;
      proc_vpd: TProcFijarValorPorDefecto; referente: TCosa);

    function ProcesarLinea(klinea: integer; linea: string; var extres: string;
      var ColumnName: string = ''): TResultadoProcesoLinea;
    procedure Free;
  private
    IdentificadorDeLaClaseDelObjeto: string; // identificador de la clase de objeto
    lineaAbierta: boolean;
    xNombre, xval: string; // valores bajo procesamiento
    procedure ResolverCampoValor(const xNombre, xVal: string);
    procedure Ejecutar_vpds(klinea: integer);
    procedure Ejecutar_vpd(klinea: integer; nombreCampo: string);
{$IFDEF DBG}
    //Para debug, muestra los campos que la lista tiene por leer
    function NombresCamposPorLeer: TDAofString;
{$ENDIF}
  end;

  TListaDeObjetosPorLeer = class(TList)
  public
    constructor Create;
    procedure Free;
    function ListaActiva: TListCamposParaLeer;
    procedure FreeListaPorLeer; // libera la última lista(cerró objeto)
    procedure InicioObjeto(const NombreClase: string);
    procedure FinalObjeto;
  end;

  TArchiTexto = class
  public
    klinea: integer;
    Version: integer;
    nombreArchivo: string;
    aux_idCarpeta: string;
    // auxiliar para pasaje de parámetros de TListaDeCosas CreateReadFromText

    unArchivoPorCosa: boolean;
    padre: TArchiTexto;
    ramas: TList;

    constructor CreateForRead(const nombreArchivo: string; abortarEnError: boolean);
    constructor CreateForWrite(const nombreArchivo: string;
      crearBackupSiExiste: boolean; maxNBackups: integer);

    procedure Free;
    function NextLine: string;
    procedure writeline(const s: string);

    procedure wr(const nombre: string; const valor: NReal; precision: integer;
      decimales: integer); overload;
    procedure wr(const nombre: string; const valor: NReal); overload;
    procedure wr(const nombre: string; const valor: string); overload;
    procedure wrArchRef(const nombre: string; const valor: TArchiRef);
    procedure wr(const nombre: string; const valor: NInt); overload;
    procedure wr(const nombre: string; const valor: cardinal); overload;
    procedure wr(const nombre: string; const valor: boolean); overload;
    procedure wr(const nombre: string; const valor: TDAOfNReal;
      precision: integer; decimales: integer); overload;
    procedure wr(const nombre: string; const valor: TDAOfNReal); overload;
    procedure wr(const nombre: string; const valor: TDAOfNInt); overload;
    procedure wr(const nombre: string; const valor: TDAOfBoolean); overload;
    procedure wr(const nombre: string; const valor: TDAOfString); overload;
    procedure wr(const nombre: string; const valor: TStringList); overload;
    procedure wr(const nombre: string; const valor: TFecha); overload;
    procedure wr(const nombre: string; const valor: TCosa); overload;
    procedure wrReferencia(const nombre: string; const valor: TCosa);

    procedure rd(const nombre: string; var valor: NReal); overload;
    procedure rd(const nombre: string; var valor: string); overload;
    procedure rdArchRef(const nombre: string; var valor: TArchiRef);
    procedure rd(const nombre: string; var valor: NInt); overload;
    procedure rd(const nombre: string; var valor: cardinal); overload;
    procedure rd(const nombre: string; var valor: boolean); overload;
    procedure rd(const nombre: string; var valor: TDAOfNReal); overload;
    procedure rd(const nombre: string; var valor: TDAOfNInt); overload;
    procedure rd(const nombre: string; var valor: TDAOfString); overload;
    procedure rd(const nombre: string; var valor: TDAOfBoolean); overload;
    procedure rd(const nombre: string; var valor: TStringList); overload;
    procedure rd(const nombre: string; var valor: TFecha); overload;
    // flgError= 0 lo mismo que sin flg. flgError= 1 no avisa del error
    procedure rd(const nombre: string; var valor: TFecha; flgError: integer); overload;
    procedure rd(const nombre: string; var valor: TDataRowOfCosa); overload;
    procedure rd(const nombre: string; var valor: TCosa); overload;
    procedure rdReferencia(const nombre: string; var valor: TCosa; referente: TCosa);


    procedure IniciarLecturaRetrasada;
    procedure EjecutarLectura;

    function CreateRamaForWrite(const Carpeta, NombreClase, NombreCosa: string):
      TArchiTexto;
    function CreateRamaForRead(const Carpeta, NombreClase, NombreCosa: string):
      TArchiTexto;
  private
    f: textfile;
    f_abierto: boolean;

    listaObjetosPorLeer: TListaDeObjetosPorLeer; //TListCamposParaLeer;
    abortarEnError: boolean;
    oldFileMode: byte;

    indentstr: string;
    function EOF: boolean;
    function quedanCosasDelObjetoActualPorLeer(claseObjeto: string): boolean;
    function ListaPorLeer: TListCamposParaLeer;

    //dfusco@20141001
    {Este procedimiento crea una instancia de TDataRowOfCosa a partir de la
    clase del campo que se leyó del archivo de texto (Ej: <+TGlobs>) y se hace
    la lectura de cada columna definida (DataColumnList) y el valor se guarda en
    la celda correspondiete de la fila creada.
    Sustituye al procedimiento ReadCosaFromText
    }
    procedure ReadDataRowFromText(var DataRow: TDataRowOfCosa;
      ColumnName: string; cosaStrId: string);
    procedure ReadCosaFromText(var cosa: TCosa; cosaStrId: string);

  end;


  //////////////////////////////////////////////////////////////////////////////
  //TSalaDeJuegoConnection
  //////////////////////////////////////////////////////////////////////////////
  //dfusco@20141001
  {Esta clase representa una conexión a la base de datos SALA DE JUEGO.}
  TSalaDeJuegoConnection = class
  private
    procedure writeline(const s: string);
    procedure wr(DataTableName: string; DataRowId: integer; tabulation: string);
  public
    FileName:WideString;
    FileHandler: TArchiTexto;
    f:TextFile;
    constructor Create(xFileName: WideString); overload;
    constructor Create(xFileHandler: TArchiTexto); overload;

    procedure ReadDataStoreFromTextFile();
    function WriteDataStoreToTextFile(filename:string=nil):boolean;

    procedure ProcessFile(SourceList:TStringList; var i:integer; var FPHashList:TCosaHashList);


  end;

  TGreekAlphabet=(alfa,beta,gama,delta,epsilon,zeta);

  TDAOfDataStore = array[TGreekAlphabet] of TDataStore;

  //////////////////////////////////////////////////////////////////////////////
  //TDataStore
  //////////////////////////////////////////////////////////////////////////////
  //dfusco@20141001
  {Esta clase representa un almacén de datos.}
  TDataStore = class
  private
    _DataTableList: TList;

  protected
    function GetDataTableByIndex(Index: integer): TDataTableOfCosa;
    function GetDataTableListCount: integer;

  public

    SetName:string;
    DataSetParent: TDataStore;

    procedure LoadFromCosaHashList(cosaHashList: TCosaHashList);

    constructor Create(xDataSetParent: TDataStore);

    function ReadDataCellTextValue(TableName: string; RowId: integer;
      ColumnName: string; var sval:string; climb:boolean=false): boolean;
    function ReadDataCellTextValues(TableName, ColumnNameToQuery, CellSValToQuery,
      ColumnNameToGetSVal: string; climb: boolean=false): TDAofString;

    function ReadRowIdsTextValues(TableName, ColumnNameToQuery,
      CellSValToQuery: string): TDAofNInt;

    function WriteDataCellTextValue(TableName: string; RowId: Integer;
      AttributeName: string; Value: string): boolean;

    procedure CreateClassInstance(TableName: string; RowId: integer; var cosa: TCosa);
    function CreateDataRowFromInstance(cosa: TCosa; DataTableName: string;
      RowName: string; climb:boolean=false): integer;

    function AddDataTable(xDataTable: TDataTableOfCosa): integer;
    property DataTableList[Index: Integer]: TDataTableOfCosa read GetDataTableByIndex;
    function GetDataTableByName(DataTableName: string; climb: boolean=false): TDataTableOfCosa;
    function GetIndexOfDataTableByName(Name: string): integer;
    function GetDataTableList: TList;
    property DataTableListCount: integer read GetDataTableListCount;

    function Query(Select: string; FromName: string;
      ColumnNameToEval, Value: string; var ds: TDataStore): boolean; overload;
    function Query(Select: string; FromName: string; var ds: TDataStore): boolean;
      overload;
    function Query (Select: String; FromName: string; RowId: integer; var ds: TDataStore
      ): boolean;overload;

    function GetEmptyTable(TableName: string; CloneIdCounter: Boolean):TDataTableOfCosa;

    procedure DeleteDataRows (FromName: string; ColumnNameToEval,
      Value: string);

    function PrepareInsert(Into, ColumnName: string; RowId: integer;
      Value: string; isItemOfList: boolean = True): TDataStore;

    function PrepareUpdate(From, ColumnName: string; RowId: integer;
      Value: string; lst: boolean): TDataStore;

    function PrepareDelete(Parent_DataTableName:string; Parent_DataRowId:integer; Child_DataTableName:string; Child_DataRowId:integer):TDataStore;

    function Clone(CloneRowIdCounter: boolean = True; CloneRowList: boolean = True): TDataStore;

    //fbarreto@20150306 Devuelve una vista del DataSet
    //Ej: 'Capa, Nombre, etc'
//    function View (ColNames: string): TDataTableOfCosa;overload;
    //fbarreto@20150310 Devuelve una vista de los elementos de la tabla virtual indicada
    function View (VirtualTable:TDataVTableOfCosa): TDataTableOfCosa;overload;

    //dfusco@20150415 Método para vaciar las tablas de la BD.
    procedure Clear();

    procedure Merge(ds: TDataStore);

    //procedure wr (DataTableName: String; RowId: Integer);

    procedure Free;

    function Equals(ADataStore: TDataStore): boolean;

    function CompareTo(Beta_DataStore: TDataStore; DataTableName: string;
      DataRowId: integer; var Gamma_DataStore: TDataStore; BetaDataRowId:integer=-1): boolean;


    //dfusco-fbarreto@20150312
    // Intentara completar el dataSet para tener la información necesaria para
    // crear la instancia del objeto representado por la fila de la tabla indicada
    function Fill(xDataTableName: string; xDataRowId: integer;
      var ADataSet: TDataStore):Boolean;

    //fbarreto@20150323
    // Devuelve una tabla en la ultima version
    procedure GetDataTableOnLastVersion (DataTableName:String; var DataTable:TDataTableOfCosa);

  end;

  //////////////////////////////////////////////////////////////////////////////
  //TDataTableOfCosa
  //////////////////////////////////////////////////////////////////////////////
  //dfusco@20141001
  {Esta clase representa una tabla de una almacén de datos.}
  TDataTableOfCosa = class
  private
    _DATAROWID_COUNTER: integer;
    _DataRowList: TList;
    _DataColumnList: TDataColumnListOfCosa;
    _DataConversionList: TListDataConversionOfCosa;
    function GetDataRowIdCounter: integer;
    procedure SetDataRowIdCounter(AValue: integer);
  protected
    function GetDataColumnByIndex(Index: integer): TDataColumnOfCosa;
    function GetDataColumnListCount():integer;

    function GetDataRowByIndex(Index: integer): TDataRowOfCosa; virtual;
    function GetDataRowListCount():integer; virtual;

    function GetDataConversionByIndex(Index: integer): TDataConversionOfCosa;
    function GetDataConversionListCount():integer;
  public

    Visible: Boolean;
    TableName: string;
    DataStoreOwner:TDataStore;

    function Clone(CloneRowIdCounter:boolean=true; CloneRowList:boolean=true):TDataTableOfCosa; virtual;

    property DataRowIdCounter: integer read GetDataRowIdCounter write SetDataRowIdCounter;

    constructor Create(xTableName:string; claseDeCosa:TClaseDeCosa; xVersion: Integer = -2); overload;
    constructor Create(xTableName: string; xDataColumnList: TDataColumnListOfCosa); overload;
    constructor Create(xTableName: string;
      xDataColumnList: TDataColumnListOfCosa; xDataRowList:TList); overload;
    constructor Create(xTableName: string); overload;

    function ReadDataCellTextValue(ColumnName:string; var sval:string):boolean; overload;
    function ReadDataCellTextValue(RowId:integer; ColumnName:string; var sval:string):boolean; overload;
    function ReadDataCellTextValueByRowIndex (RowIndex:integer; ColumnName:string; var sval:string):boolean;

    //procedure WriteDataCellTextValue(RowName,ColumnName,value:string);

    //Devuelve el rowId
    function AddDataRow(xDataRow: TDataRowOfCosa; SetRowId: boolean = True): integer;
    property DataRowList[Index: integer]: TDataRowOfCosa read GetDataRowByIndex;
    function GetDataRowList():TList; virtual;
    procedure SetDataRowList(drl: TList);
    property DataRowListCount: integer read GetDataRowListCount;
    function GetDataRowById(Id:integer):TDataRowOfCosa;
    function GetIndexOfDataRowById(Id:integer):integer;

    function AddDataColumn(xDataColumn: TDataColumnOfCosa): integer;
    property DataColumnList[Index: integer]: TDataColumnOfCosa read GetDataColumnByIndex;

    function GetDataColumnByName(Name: string): TDataColumnOfCosa;

    function GetIndexOfDataColumnByName(Name: string): integer;
    function GetDataColumnList():TDataColumnListOfCosa;
    procedure SetDataColumnList(dcl: TDataColumnListOfCosa);
    property DataColumnListCount: integer read GetDataColumnListCount;

    property DataConversionList[Index:integer]:TDataConversionOfCosa read GetDataConversionByIndex;
    function GetDataConversionList():TListDataConversionOfCosa;

    function ConvertDataRowToNewestVersion(var DataRow: TDataRowOfCosa; DataTableAux: TDataTableOfCosa):Boolean;

    function GetDataRowWithDefaulValues():TDataRowOfCosa;

    //No compara las filas, de momento no es necesario
    function Equals (dt: TDataTableOfCosa): boolean;

    procedure Merge(Beta_DataTable: TDataTableOfCosa);

    procedure Free(); virtual;
    procedure Clear(); virtual;


    //fbarreto@20150305 Devuelve los valores de la columna indicada
    function GetColValues(kCol: Integer):TStringList;overload;
    function GetColValues(ColName: string):TStringList;overload;

    //fbarreto@20150305 Si las tablas tiene la misma definicion agrega las filas de tabla pasada como parametro
    // a la tabla. Si reAssignIds=FALSE y hay Ids que coinciden levanta una exception
    procedure AppendDataTable(dt: TDataTableOfCosa; reAssignIds: Boolean );

    //fbarreto@20150305 Realiza busqueda en la tabla
    // Devuelve una tabla con la columnas "select" y las filas que cumplan la condicion
    function Query( Select: string; ColumnNameToEval: string='';
      Value: string='' ):TDataTableOfCosa;overload;

    //fbarreto@20150305 Realiza busqueda en la tabla
    // Devuelve una tabla con la columnas "select" y la fila rowId
    function Query( Select: string; rowId: Integer ):TDataTableOfCosa;overload;

    //fbarreto@20150306 Devuelve una tabla con columnas ColNames, las columnas
    //que no pertenecen a las tabla se completan con "-"
    function View (ColNames: string):TDataTableOfCosa;



  end;

  //////////////////////////////////////////////////////////////////////////////
  //TDataVTableOfCosa
  //////////////////////////////////////////////////////////////////////////////
  //dfusco@20141001
  {Esta clase representa una tabla virtual de una almacén de datos para agrupar
  tablas semánticamente.}
  TDataVTableOfCosa = class(TDataTableOfCosa)
  private
    _ChildClassNameList: TStringList;
  protected
    function GetChildClassNameList():TStringList;
  public
    constructor Create(xDataTableName: string);overload;
    constructor Create(xDataTableName: string; xDataColumnList: TDataColumnListOfCosa
      );overload;
    constructor Create(xDataTableName: string; xClassNameList:TStringList; xDataColumnList: TDataColumnListOfCosa
      );overload;

    function AddChildClassName(AClassName: string): integer;
    function GetDataRowList(ds:TDataStore):TList;
    property ChildClassNameList: TStringList read GetChildClassNameList;

  end;

  { TDataTableOfListOfCosa }

  TDataTableOfListOfCosa = class(TDataTableOfCosa)
  private
    _LastIdParent: integer;
  public
    constructor Create();
    function GetNextIdParent():integer;
    function Clone(CloneRowIdCounter: boolean=true; CloneRowList: boolean=true
      ): TDataTableOfListOfCosa; override;

    procedure Free(); override;
    procedure Clear(); override;
  end;

  //////////////////////////////////////////////////////////////////////////////
  //TDataRowOfCosa
  //////////////////////////////////////////////////////////////////////////////
  //dfusco@20141001
  {Esta clase representa una fila de una tabla de una almacén de datos.}
  TDataRowOfCosa = class
  private
    _DataCellList: TList;
    RowId: integer;
    function GetRowId: integer;
    procedure SetRowId(AValue: integer);
  protected
    function GetDataCellByIndex(Index: integer): TDataCellOfCosa;
    function GetDataCellListCount():integer;
    function GetLastDataCell():TDataCellOfCosa;
  public
    SIUD: TSIUD;
    RowName: string;
    DataTableOwner: TDataTableOfCosa;
    constructor Create(xRowName: string = '-'); overload;
    constructor Create(xRowId:integer); overload;
    constructor Create(xDataTableOwner:TDataTableOfCosa; xRowName: string = '-'); overload;

    property _RowId: integer read GetRowId write SetRowId;

    function AddDataCell(xDataCell: TDataCellOfCosa; SetOwner: boolean=true): Integer;
    property DataCellList[Index: Integer]: TDataCellOfCosa read GetDataCellByIndex;
    property DataCellListCount: integer read GetDataCellListCount;
    property LastDataCell: TDataCellOfCosa read GetLastDataCell;

    procedure InsertDataCell(ADataCell:TDataCellOfCosa; AIndex:integer);

    function Clone(CloneRowId: boolean = True): TDataRowOfCosa;

    //Clona el DataRow si DataRow[i]=Value, sino devuelve nil
    function ConditionalClone(i: integer; Value: string): TDataRowOfCosa;

    //Devuelve el DataRow[kCell] si DataRow[kCellToEval]=Value, sino nil
    function ConditionalGetDataCell (kCell, kCellToEval: Integer; Value:String):TDataCellOfCosa;

    function Equals(dr: TDataRowOfCosa): boolean;
    procedure Merge (ADataRow: TDataRowOfCosa);

    function CompareTo(Beta_DataRow:TDataRowOfCosa;Gamma_DataTable:TDataTableOfCosa):boolean;

    procedure Free; virtual;

    function ToString():TStrings;

  end;

  { TDataVRowOfCosa }

  TDataVRowOfCosa=class(TDataRowOfCosa)
  public
    OriginalDataTableName:string;
    OriginalDataRowId:integer;
    procedure Free(); override;
  end;

  //////////////////////////////////////////////////////////////////////////////
  //TDataCellOfCosa
  //////////////////////////////////////////////////////////////////////////////
  //dfusco@20141001
  {Esta clase representa una celda de una tabla de una almacén de datos.}
  TDataCellOfCosa = class
  private
    function getVal: string;
    procedure setVal(AValue: string);
  public
    _sval: string;
    DataRowOwner: TDataRowOfCosa;
    pval: pointer;
    SIUD: TSIUD;
    Changed:boolean;
    constructor Create(xsval: string = '');
    function Clone():TDataCellOfCosa;
    function Equals(dcll: TDataCellOfCosa): boolean;
    property sval:string read getVal write setVal;
    procedure Free;

  end;

  { TDataColumnListOfCosa }

  TDataColumnListOfCosa = class(TList)
  private
    _ClassOwner: TClaseDeCosa;
    _Version: Integer;
    _ItemListClaseDeCosa: TClaseDeCosa;
    procedure SetClassOwner(AValue: TClaseDeCosa);
    procedure SetItemListClaseDeCosa(AValue: TClaseDeCosa);
  protected
    function GetItem(Index: Integer): TDataColumnOfCosa;
    function GetClassOwner():TClaseDeCosa;
    function GetItemListClaseDeCosa():TClaseDeCosa;
    function GetVersion():Integer;
  public
    DataTableOwner:TDataTableOfCosa;

    //Lista de operaciones para validar
    ValidationList:TListValidation;

    constructor Create(); overload;
    constructor Create(xClassOwner: TClaseDeCosa; xVersion: integer = -2); overload;
    constructor Create(xClassOwner:TClaseDeCosa; xItemListClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2); overload;

    function AddDataColumn(xDataColumn: TDataColumnOfCosa;
                           SetOwner:    boolean=true):integer;overload;

    function AddDataColumn(xColumnName: string;
                           xDataType:   DataTypeOfCosa): integer; overload;

    function AddDataColumn(xColumnName:  string;
                           xDataType:    DataTypeOfCosa;
                           xOptions:     array of string;
                           xVersionAlta: integer;
                           xVersionBaja: integer;
                           xClaseDeCosa: TClaseDeCosa=nil): integer; overload;

    function AddDataColumn(xColumnName:  string;
                           xDataType:    DataTypeOfCosa;
                           xVersionAlta: integer;
                           xVersionBaja: integer;
                           xClaseDeCosa: TClaseDeCosa=nil): integer; overload;

    function AddDataColumn(xColumnName:  string;
                           var xField;
                           xDataType:    DataTypeOfCosa;
                           xOptions:     array of string;
                           xVersionAlta: integer;
                           xVersionBaja: integer;
                           xClaseDeCosa: TClaseDeCosa=nil): integer; overload;

    function AddDataColumn(xColumnName:  string;
                           var xField;
                           xDataType:    DataTypeOfCosa;
                           xVersionAlta: integer;
                           xVersionBaja: integer;
                           xClaseDeCosa: TClaseDeCosa=nil): integer; overload;

    property Items[Index: Integer]: TDataColumnOfCosa read GetItem;default;
    property ClassOwner: TClaseDeCosa read GetClassOwner write SetClassOwner;
    property ItemListClaseDeCosa: TClaseDeCosa read GetItemListClaseDeCosa write SetItemListClaseDeCosa;
    property Version: Integer read GetVersion;
    function GetIndexOfDataColumnByName(AName: string): integer;

    procedure Move(CurIndex, NewIndex: Integer);

    function Clone():TDataColumnListOfCosa;
    
    function Equals(dcl: TDataColumnListOfCosa): boolean;

    // Lee el campo iCampo de aCosa y lo retorna como String
    function GetValStr( aCosa: TCosa; iCampo: integer ): string;

    // Fija el valor del campo iCampo de aCosa de acuerdo al NuevoValor
    // El resultado es TRUE si logró setear el campo (no falló la conversión de tipo)
    function SetValStr( aCosa: TCosa; iCampo: integer; NuevoValor: string ): boolean;

    // Destruye el objeto
    procedure Free;
  end;

  //////////////////////////////////////////////////////////////////////////////
  //TDataColumnOfCosa
  //////////////////////////////////////////////////////////////////////////////
  //dfusco@20141001
  {Esta clase representa una columna de una tabla de una almacén de datos.}
  TDataColumnOfCosa = class
    Index:integer;
    ColumnName: string;
    LabelCaption: string;
    DataType: TDataTypeOfCosa;
    Offset: longint;
    ValorPorDefecto: string;
    VersionAlta: integer;
    VersionBaja: integer;
    DataColumnListOwner:TDataColumnListOfCosa;

    constructor Create(xColumnName:  string;
                       xDataType:    DataTypeOfCosa;
                       xOffset:      longint;
                       xVersionAlta: integer;
                       xVersionBaja: integer;
                       xClaseDeCosa: TClaseDeCosa=nil); overload;

    constructor Create(xColumnName:   string;
                       xDataType:     DataTypeOfCosa;
                       xOffset:       longint;
                       xOptions:      array of string;
                       xVersionAlta:  integer;
                       xVersionBaja:  integer;
                       xClaseDeCosa:  TClaseDeCosa=nil); overload;

    constructor Create(xColumnName:  string;
                       xDataType:    DataTypeOfCosa;
                       xClaseDeCosa: TClaseDeCosa=nil); overload;

    function Clone():TDataColumnOfCosa;

    function Equals(dc: TDataColumnOfCosa): boolean;

    procedure Free;
  end;

  TOperator=(

     //Relational Operators
     EQUAL,
     DISTINCT,
     LESS,
     LESS_OR_EQUAL,
     GREATER,
     GREATER_OR_EQUAL,

     //Regular Expresion
     REGEX

   );

  //////////////////////////////////////////////////////////////////////////////
  //TDataTypeOfCosa
  //////////////////////////////////////////////////////////////////////////////
  //dfusco@20141001
  {Esta clase representa un tipo de dato de una columna de una tabla de una
  almacén de datos.}
  TDataTypeOfCosa = class
    DataColumnOwner: TDataColumnOfCosa;
    ClaseDeCosa: TClaseDeCosa;
    CosaStrId:string;

    constructor Create(DataColumnOwner: TDataColumnOfCosa);

    function Validate(sval:string; var ErrorMessage:string):boolean;virtual;
    function CompareTo(TheOperator:TOperator; asval:string; bsval:string):boolean;virtual;
    function Eval(sval: string; pval: pointer): boolean; virtual; abstract;
    function ToString(pval: pointer): string; virtual; abstract;
    function IsAMacroVariable(sval: string): boolean;
    function Clone():TDataTypeOfCosa; virtual;
    function Equals(dtc: TDataTypeOfCosa): boolean;

    function Format(sval:string):string;virtual;

    procedure Free;
  end;

  { TDataTypeDataRowOfCosa }

  TDataTypeDataRowOfCosa = class(TDataTypeOfCosa)
    DataTableName: string;
    PK: string;
    function Eval(sval: string; pval: pointer): boolean; override;
    function ToString(pval: pointer): string; override;
  end;

  { TDataTypeListOfCosa }

  TDataTypeListOfCosa = class(TDataTypeOfCosa)
    function Eval(sval: string; pval: pointer): boolean; override;
    function ToString(pval: pointer): string; override;
  end;

  { TDataTypeIntegerOfCosa }

  TDataTypeIntegerOfCosa = class(TDataTypeOfCosa)
    function Validate(sval:string; var ErrorMessage:string):boolean;override;
    function CompareTo(TheOperator:TOperator; asval:string; bsval:string):boolean;override;
    function Eval(sval: string; pval: pointer): boolean; override;
    function ToString(pval: pointer): string; override;
  end;

  { TDataTypeStringOfCosa }

  TDataTypeStringOfCosa = class(TDataTypeOfCosa)
    function Eval(sval: string; pval: pointer): boolean; override;
    function ToString(pval: pointer): string; override;
  end;

  { TDataTypeDateOfCosa }

  TDataTypeDateOfCosa = class(TDataTypeOfCosa)
    function Validate(sval:string; var ErrorMessage:string):boolean;override;
    function CompareTo(TheOperator:TOperator; asval:string; bsval:string):boolean;override;
    function Eval(sval: string; pval: pointer): boolean; override;
    function ToString(pval: pointer): string; override;
  end;

  { TDataTypeDoubleOfCosa }

  TDataTypeDoubleOfCosa = class(TDataTypeOfCosa)
    function Validate(sval:string; var ErrorMessage:string):boolean;override;
    function Eval(sval: string; pval: pointer): boolean; override;
    function ToString(pval: pointer): string; override;
  end;

  { TDataTypeBooleanOfCosa }

  TDataTypeBooleanOfCosa = class(TDataTypeOfCosa)
    function Eval(sval: string; pval: pointer): boolean; override;
    function ToString(pval: pointer): string; override;
  end;

  { TDataTypeDAOfDoubleOfCosa }

  TDataTypeDAOfDoubleOfCosa = class(TDataTypeOfCosa)
    function Eval(sval: string; pval: pointer): boolean; override;
    function ToString(pval: pointer): string; override;
    function Format(sval:string):string; override;
  end;

  { TDataTypeDAOfStringOfCosa }

  TDataTypeDAOfStringOfCosa = class(TDataTypeOfCosa)
    function Eval(sval: string; pval: pointer): boolean; override;
    function ToString(pval: pointer): string; override;
    function Format(sval:string):string; override;
  end;

  { TDataTypeStringListOfCosa }

  TDataTypeStringListOfCosa = class(TDataTypeOfCosa)
    function Eval(sval: string; pval: pointer): boolean; override;
    function ToString(pval: pointer): string; override;
    function Format(sval:string):string; override;
  end;

  { TDataTypeDAOfIntegerOfCosa }

  TDataTypeDAOfIntegerOfCosa = class(TDataTypeOfCosa)
    function Eval(sval: string; pval: pointer): boolean; override;
    function ToString(pval: pointer): string; override;
    function Format(sval:string):string; override;
  end;

  { TDataTypeDAOfBooleanOfCosa }

  TDataTypeDAOfBooleanOfCosa = class(TDataTypeOfCosa)
    function Eval(sval: string; pval: pointer): boolean; override;
    function ToString(pval: pointer): string; override;
    function Format(sval:string):string; override;
  end;

  { TDataTypeArchiRefOfCosa }

  TDataTypeArchiRefOfCosa = class(TDataTypeOfCosa)
    function Eval(sval: string; pval: pointer): boolean; override;
    function ToString(pval: pointer): string; override;
  end;

  { TDataTypeCardinalOfCosa }

  TDataTypeCardinalOfCosa = class(TDataTypeOfCosa)
    function Eval(sval: string; pval: pointer): boolean; override;
    function ToString(pval: pointer): string; override;
  end;

  { TDataTypeReferenciaOfCosa }

  TDataTypeReferenciaOfCosa = class(TDataTypeOfCosa)
    function Eval(sval: string; pval: pointer): boolean; override;
    function ToString(pval: pointer): string; override;
  end;


  { TValidation }

  TValidation=class
  public

    ValidationListOwner:TListValidation;

    TheOperator:TOperator;
    A:string;
    B:string;
    ErrorMessage:string;
    constructor Create(xTheOperator:TOperator;xA:string;xB:string;xErrorMessage:string);
    function Validate(xDataType:TDataTypeOfCosa; asval:string):boolean; overload;
    function Validate(xDataType:TDataTypeOfCosa; asval:string;bsval:string):boolean; overload;
  end;

  { TListValidation }

    TListValidation=class(TList)
    private
      function GetItem(Index: Integer): TValidation;
    public
      constructor Create;
      function Add(xValidation:TValidation):integer;
      property Items[Index: Integer]: TValidation read GetItem;
      function Clone():TListValidation;
      function GetValidationToDoOverField(xA: string): TListValidation;
    end;

   TProcMsg = procedure(msg: string);

    procedure parseRowReference (sval: string; var DataTableName: string;
      var RowId: Integer);
    function parseListReference(sval: string; var DataTableName: string;
          var DataColumnName: string; var IdParent: string): Boolean;




procedure ValorPorDefectoNulo(const nombre: string; pVal: Pointer;
  tipo: TTipoCampo; const klinea: integer);
procedure ValorPorDefecto_ERROR(const nombre: string; pVal: Pointer;
  tipo: TTipoCampo; const klinea: integer);

var
  registro_de_ClasesDeCosas_: TListaDeCosas;
  procMsgValorPorDefecto: TProcMsg;
  procMsgErrorLectura: TProcMsg;
  procMsgAdvertenciaLectura: TProcMsg;

{$IFDEF CNT_COSAS}
type
  TCntCosasClase = class
  private
    clase: TClass;
    cnt: integer;
  public
    constructor Create(xclase: TClass);
  end;

  TCntCosas = class
  private
    CntCosasClases: TList {of TCntCosasClase};
  public
    cnt_Cosas: integer;

    constructor Create;
    function incCntCosasClase(clase: TClass): integer;
    function decCntCosasClase(clase: TClass): integer;
    function CntCosasClase(clase: TClass): integer;
    procedure Free;
  end;

var
  CantCosas: TCntCosas;

{$ENDIF}

function registrarClaseDeCosa(const nombre: string; Clase: TClaseDeCosa): integer;
function getClaseOf(const nombre: string): TCualquierClase;
// si la clase no está registrada retorna '<CLASE_NO_REGISTRADA>'
function getIdStrOf(clase: TClaseDeCosa): string;

//Retorna TRUE si la clase child está en el árbol de herencias de la clase
// Parent y false en caso contrario.
function IsChild(Parent, Child: TClass): boolean;

procedure LimpiarRegistroDeClasesDeCosas;

// Retorna la cadena de padres separados por '->'
function ParentsStrList( clase: TClass ): string;

procedure AlInicio;
procedure AlFinal;

procedure FreeAndNil(var obj: TCosa);
function referenciaACosa(cosa: TCosa): string;

type
  ExceptionFileNotFound = class(Exception)
  public
    arch: string;
    constructor Create(arch, comentarios: string);
  end;

  { TFichaClase }

  TFichaClase = class(TCosa)
  public
    strClaseId: string;
    clase: TClaseDeCosa;
    //dfusco@20141023 se agrega la lista de definición de campos.
    DataColumnList: TList;
    constructor Create(strClaseID: string; clase: TClaseDeCosa);
    procedure Free; override;
  end;

var
  // Lista de referencias a archivos externos.
  ReferenciasAArchivosExternos: TList;

  // Lista de caminos de búsquedas de los archivos externos.
  // La aplicación debe agregar aquí las rutas de búqueda que
  // se deben usar en la función testear de las referencias a
  // archivos.
  lista_caminos: TStringList;

  //dfusco-fbarreto@20141023 El DataStore pasa a ser global para crear las
  //estructuras de las tablas AlInicio.
  DATA_STORE: TDataStore;
  //Lista de Cosas con nombre en el DATA:STORE
  cosasConNombre: TListaDeCosas;

// retorna la constante de versión de formato de sala almacenada en la unidad
// uversion_architexto
function version_ArchiTexto: integer;

procedure PrintDataSet(DataSet:TDataStore);
procedure PrintDataSetToFile(DataSet:TDataStore);


implementation

uses
//  uVirtualTablesDefs,
  uCosaConNombre, usalasdejuego, uFuenteConstante;

var
  global_FlgError: integer;


function version_ArchiTexto: integer;
begin
  Result := uversion_architexto.version_ArchiTexto;
end;

procedure PrintDataSet(DataSet:TDataStore);
var
  z: Integer;
  j: Integer;
  i: Integer;
  a: Integer;
begin
  WriteLn();
  WriteLn('---');
  WriteLn('SetName = ' + DataSet.SetName);
  for z:=0 to DataSet.DataTableListCount-1 do
  begin
    WriteLn('   ' + 'TableName = ' + DataSet.DataTableList[z].TableName);
    if DataSet.DataTableList[z].TableName='dt_TFichasLPD' then
      a:=1;
    for j:=0 to DataSet.DataTableList[z].DataRowListCount-1 do
    begin
      WriteLn('      ' + 'RowId = ' + IntToStr(DataSet.DataTableList[z].DataRowList[j]._RowId) + ' | SIUD = ' + GetEnumName(TypeInfo(TSIUD), ord(DataSet.DataTableList[z].DataRowList[j].SIUD)));
      for i:=0 to DataSet.DataTableList[z].DataColumnListCount-1 do
      begin
        WriteLn('         ' + DataSet.DataTableList[z].DataColumnList[i].ColumnName+' = '+DataSet.DataTableList[z].DataRowList[j].DataCellList[i].sval);
      end;
    end;
  end;
  WriteLn('---');
  WriteLn();
end;

procedure PrintDataSetToFile(DataSet: TDataStore);
var
  z: Integer;
  j: Integer;
  i: Integer;
  a: Integer;
  f: TextFile;
  xFileName: String;
  oldFileMode: Byte;
begin

  xFileName:='pepe.txt';
  assignFile(f, xFileName);
  oldFileMode := FileMode;
  filemode := fmOpenWrite;
  Rewrite(f);
  if ioresult <> 0 then
    raise Exception.Create('TArchiTexto.CreateForWrite(' + xFileName + ')');

  WriteLn(f);
  WriteLn(f, '---');
  WriteLn(f, 'SetName = ' + DataSet.SetName);
  for z:=0 to DataSet.DataTableListCount-1 do
  begin
    WriteLn(f, '   ' + 'TableName = ' + DataSet.DataTableList[z].TableName);
    if DataSet.DataTableList[z].TableName='dt_TFichasLPD' then
      a:=1;
    for j:=0 to DataSet.DataTableList[z].DataRowListCount-1 do
    begin
      WriteLn(f, '      ' + 'RowId = ' + IntToStr(DataSet.DataTableList[z].DataRowList[j]._RowId) + ' | SIUD = ' + GetEnumName(TypeInfo(TSIUD), ord(DataSet.DataTableList[z].DataRowList[j].SIUD)));
      for i:=0 to DataSet.DataTableList[z].DataColumnListCount-1 do
      begin
        WriteLn(f, '         ' + DataSet.DataTableList[z].DataColumnList[i].ColumnName+' = '+DataSet.DataTableList[z].DataRowList[j].DataCellList[i].sval);
      end;
    end;
  end;
  WriteLn(f, '---');
  WriteLn(f);

  CloseFile(f);
  filemode:=oldFileMode;
end;

{ TListValidation }

function TListValidation.GetItem(Index: Integer): TValidation;
begin
  Result:=TValidation(inherited Items[Index]);
end;

constructor TListValidation.Create;
begin
  inherited Create;
end;

function TListValidation.Add(xValidation: TValidation): integer;
begin
  Result:=inherited Add(xValidation);
  xValidation.ValidationListOwner:=self;
end;

function TListValidation.Clone: TListValidation;
var
  Alfa_validation: TValidation;
  Beta_validation: TValidation;
  i: Integer;
begin
  Result:=TListValidation.Create();
  for i:=0 to Count-1 do
  begin
    Alfa_validation:=self.Items[i];
    Beta_validation:=TValidation.Create(Alfa_validation.TheOperator,Alfa_validation.A,Alfa_validation.B,Alfa_validation.ErrorMessage);
    Result.Add(Beta_validation);
  end;
end;

function TListValidation.GetValidationToDoOverField(xA: string): TListValidation;
var
  i: Integer;
begin
  Result:=TListValidation.Create();
  for i:=0 to self.Count-1 do
    if LowerCase(self.Items[i].A)=LowerCase(xA) then
    begin
      Result.Add(self.Items[i]);
      exit;
    end;
end;

{ TValidation }

constructor TValidation.Create(xTheOperator: TOperator; xA: string; xB: string;
  xErrorMessage: string);
begin
  self.TheOperator:=xTheOperator;
  self.A:=LowerCase(xA);
  self.B:=LowerCase(xB);
  self.ErrorMessage:=xErrorMessage;
end;

function TValidation.Validate(xDataType: TDataTypeOfCosa; asval: string
  ): boolean;
begin
  Result:=True;
end;

function TValidation.Validate(xDataType: TDataTypeOfCosa; asval: string;
  bsval: string): boolean;
var
  RegexObj: TRegExpr;
begin

  Case self.TheOperator of

    TOperator.REGEX:
    begin
      RegexObj:=TRegExpr.Create();
      RegexObj.Expression:=bsval;
      Result:=RegexObj.Exec(asval);
      RegexObj.Free;
    end
    else
      Result:=xDataType.CompareTo(self.TheOperator, asval, bsval);
  end;
end;

{ TDataVRowOfCosa }

procedure TDataVRowOfCosa.Free;
begin
  inherited Free;
end;




//---------------------------------
// Métodos de TArchiRef
//=================================


constructor TArchiRef.Create(archiRef_: string);
begin
  archiRefStr := archiRef_;
  ReferenciasAArchivosExternos.add(self);
  //writeln( 'TArchiRef.Crete: '+archiRef_ );
end;

procedure TArchiRef.Free;
var
  i: integer;
begin
  i := ReferenciasAArchivosExternos.IndexOf(Self);
  if i >= 0 then
    ReferenciasAArchivosExternos.Delete(i);
  inherited Free;
end;

function TArchiRef.get_archi: string;
begin
  Result := archiRefStr;
end;

procedure TArchiRef.set_archi(const archi_str: string);
begin
  archiRefStr := archi_str;
end;

function TArchiRef.testear: boolean;
var
  nombre: string;
  camino: string;
  carpeta: string;
  buscando: boolean;
  k: integer;

begin
  if FileExists(archiRefStr) then
    Result := True
  else
  begin
    buscando := lista_caminos.Count > 0;
    k := 0;
    while buscando and (k < lista_caminos.Count) do
    begin
      carpeta := lista_caminos[k];
      if carpeta <> '' then
      begin
        nombre := ExtractFileName(archiRefStr);
        camino := ExtractFilePath(archiRefStr);
        if length(camino) > length(nombre) then
        begin
          Delete(nombre, 1, length(camino));
        end;
        nombre := carpeta + barraDir + nombre;
        if FileExists(nombre) then
        begin
          archiRefStr := nombre;
          buscando := False;
        end
        else
          Inc(k);
      end;
    end;
    Result := not buscando;
  end;
end;



//---------------------------------
// Métodos de ExceptionFileNotFound
//=================================

constructor ExceptionFileNotFound.Create(arch, comentarios: string);
begin
  inherited Create(rs_NoSeEncuentraArchivo + ': ' + arch + '. ' + comentarios);
  self.arch := arch;
end;




//-----------------
// Métodos de TCosa
//=================


constructor TCosa.Create;
begin
  inherited Create;
  self.id:=-1;
end;

constructor TCosa.Create(capa: integer);
begin
  inherited Create;
  self.capa := capa; // CapaPorDefecto
end;



function TCosa.Apodo: string;
begin
  Result := '-';
end;


procedure TCosa.ActivarCapas(const capasActivas: TDAOfNInt);
begin
  // este no hace nada.
end;

procedure TCosa.WriteToArchi(archi: string);
var
  f: TArchiTexto;
begin
  f := TArchiTexto.CreateForWrite(archi, False, 0);
  try
    f.wr(apodo, Self);
  finally
    f.Free;
  end;
end;

function TCosa.Create_Clone: TCosa;
var
  f: TArchiTexto;
  a: TCosa;
  dirTemporales: string;
begin
  {dfusco@20150423
   Si se está en la nueva versión del programa (PERSISTENCIA TEXTO) no voy a
   clonar el objeto, me voy a quedar con el mismo porque necesito el cosa.id.
   Si clono como está acá el id lo pierdo porque no persiste.
   Como el objeto es temporal no hay problema en hacer esto...
   Al abrir y cerrar la ventana se va a destruir porque se guarda siempre en
   la base de datos. }
  {$IfDef MISION_IMPOSIBLE}
  Result:=self;
  {$Else}
  a:=nil;
  dirTemporales := getDir_Tmp;

  f := TArchiTexto.CreateForWrite(dirTemporales + 'aux_clonar.tmp', False, 0);
  try
    f.wr('_', self);
  finally
    f.Free;
  end;

  f := TArchiTexto.CreateForRead(dirTemporales + 'aux_clonar.tmp', True);
  try
    f.rd('_', a);
    Result := a;
  finally
    f.Free;
  end;
  {$EndIf}
end;

function TCosa.asMemo: TStrings;
var
  f: TArchiTexto;
  ff: textfile;
  a: TCosa;
  dirTemporales: string;
  r: string;
  res: TStringList;
begin
  dirTemporales := getDir_Tmp;
  f := TArchiTexto.CreateForWrite(dirTemporales + 'aux_clonar.tmp', False, 0);
  try
    f.wr('_', self);
  finally
    f.Free;
  end;

  res := TStringList.Create;
  assignfile(ff, dirTemporales + 'aux_clonar.tmp');
  reset(ff);
  while not EOF(ff) do
  begin
    readln(ff, r);
    res.add(r);
  end;
  closefile(ff);
  Result := res;
end;

class function TCosa.Create_FromMemo(const memo: TStrings): TCosa;
var
  f: TArchiTexto;
  ff: textfile;
  a: TCosa;
  dirTemporales: string;
  k: integer;
begin
  a:=nil;
  dirTemporales := getDir_Tmp;
  assignfile(ff, dirTemporales + 'aux_clonar.tmp');
  rewrite(ff);
  for k := 0 to memo.Count - 1 do
    writeln(ff, memo[k]);
  closefile(ff);

  f := TArchiTexto.CreateForRead(dirTemporales + 'aux_clonar.tmp', True);
  try
    f.rd('_', a);
    Result := a;
  finally
    f.Free;
  end;
end;

constructor TCosa.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create;
  if f.version < 91 then
    capa := 0
  else
  begin
    f.IniciarLecturaRetrasada;
    f.rd('capa', capa);
    f.EjecutarLectura;
  end;
end;

procedure TCosa.WriteToText(f: TArchiTexto);
begin
  f.wr('capa', capa);
end;

procedure TCosa.Free;
var
  k: integer;
begin
  ucosaConNombre.eliminar_referencias_del(self);
  inherited Free;
end;


function TCosa.InfoAd_: string;
begin
  Result := IntToStr(capa) + ', ';
end;

function TCosa.InfoAd_20: string;
var
  res: string;
begin
  res := infoAd_;
  if length(res) > 20 then
  begin
    Delete(res, 21, length(res) - 20);
    res := res + '...';
  end;
  Result := res;
end;


class function TCosa.DescClase: string;
begin
  Result := 'Cosa';
end;

class function TCosa.CreateDataConversionList: TListDataConversionOfCosa;
begin
  Result := TListDataConversionOfCosa.Create;
end;

class function TCosa.CreateDataColumnList(xClaseDeCosa: TClaseDeCosa;
  xVersion: integer): TDataColumnListOfCosa;
begin

  Result := TDataColumnListOfCosa.Create(xClaseDeCosa, xVersion);



  Result.AddDataColumn('capa', TCosa(nil).capa, TDataTypeIntegerOfCosa, ['Capa', '0'], 91, -1);

end;

procedure TCosa.AfterInstantiation;
begin

end;

function TCosa.Validate: boolean;
begin
  Result:=True;
end;

{$IFDEF CNT_COSAS}
constructor TCntCosasClase.Create(xclase: TClass);
begin
  clase := xclase;
  cnt := 1;
end;

constructor TCntCosas.Create;
begin
  cnt_Cosas := 0;
  CntCosasClases := TList.Create;
end;

function TCntCosas.incCntCosasClase(clase: TClass): integer;
var
  i: integer;
  resultado: integer;
  aux: TCntCosasClase;
begin
  resultado := 0;
  for i := 0 to CntCosasClases.Count - 1 do
    if TCntCosasClase(CntCosasClases[i]).clase = clase then
    begin
      TCntCosasClase(CntCosasClases[i]).cnt := TCntCosasClase(CntCosasClases[i]).cnt + 1;
      resultado := TCntCosasClase(CntCosasClases[i]).cnt;
      break;
    end;
  if resultado = 0 then
  begin
    aux := TCntCosasClase.Create(clase);
    resultado := aux.cnt;
    CntCosasClases.Add(aux);
  end;
  cnt_Cosas := cnt_Cosas + 1;
  Result := resultado;
end;

function TCntCosas.decCntCosasClase(clase: TClass): integer;
var
  i, ipos: integer;
begin
  ipos := -1;
  for i := 0 to CntCosasClases.Count - 1 do
    if TCntCosasClase(CntCosasClases[i]).clase = clase then
    begin
      TCntCosasClase(CntCosasClases[i]).cnt := TCntCosasClase(CntCosasClases[i]).cnt - 1;
      ipos := i;
      cnt_Cosas := cnt_Cosas - 1;
      break;
    end;
  if ipos <> -1 then
    Result := TCntCosasClase(CntCosasClases[ipos]).cnt
  else
    raise Exception.Create(rs_NoInstanciasDe + ': ' + clase.ClassName);
end;

function TCntCosas.CntCosasClase(clase: TClass): integer;
var
  i, ipos: integer;
begin
  ipos := -1;
  for i := 0 to CntCosasClases.Count - 1 do
    if TCntCosasClase(CntCosasClases[i]).clase = clase then
    begin
      ipos := i;
      break;
    end;
  if ipos <> -1 then
    Result := TCntCosasClase(CntCosasClases[i]).cnt
  else
    Result := 0;
end;

procedure TCntCosas.Free;
var
  i: integer;
begin
  for i := 0 to CntCosasClases.Count - 1 do
    TCntCosasClase(CntCosasClases[i]).Free;
  CntCosasClases.Free;
  inherited Free;
end;

{$ENDIF}

// métodos de TFichaClase
constructor TFichaClase.Create(strClaseID: string; clase: TClaseDeCosa);
begin
  inherited Create;
  Self.strClaseId := strClaseId;
  self.clase := clase;
end;

procedure TFichaClase.Free;
begin
  inherited Free;
end;


procedure TArchiTexto.ReadDataRowFromText(var DataRow: TDataRowOfCosa;
  ColumnName: string; cosaStrId: string);
var
  clase: TClaseDeCosa;
  j, jj, a: integer;
  Name, aaa: string;
  count: Integer;
  DataColumn: TDataColumnOfCosa;

  DataTypeDataRow: TDataTypeDataRowOfCosa;
  DataTableName: string;
  DataTable: TDataTableOfCosa;
  h: integer;

  DataRowChild: TDataRowOfCosa;

  n: integer;
  esPeriodica: boolean;
  nombre: string;

  cl: boolean; //bandera que me indica si TListaDeCosasConNombre tendrá elementos (n>0)
  z: integer;
  DataColumn2: TDataColumnOfCosa;
  DataRow2: TDataRowOfCosa;
  DataTable2: TDataTableOfCosa;

  index: integer;

  sreferencia: string;
  claseReferido: TClaseDeCosa;
  nombreClaseReferido, nombreAtributoReferido: string;

  dataTableAux: TDataTableOfCosa;
  dataRowAux: TDataRowOfCosa;

  DataTableOfList: TDataTableOfListOfCosa;
  DataRowOfList: TDataRowOfCosa;
  LastIdParent: integer;
  strValor: string;
  adt: TDateTime;
  ad: NReal;
  b: String;

begin

  cl := False;

  clase := TClaseDeCosa(getClaseOf(CosaStrId));
  if clase = nil then
    raise ExcepcionClaseNoRegistrada.Create(
      'TArchiTexto.ReadDataRowFromText: Error, ' + CosaStrId +
      ' No es una clase registrada. Procesando Línea: ' + IntToStr(klinea));

  if clase.ClassName='TListGenerador' then
    a:=1;

  //Tabla auxiliar que corresponde a la version del archiText

  DataTableName := 'dt_' + clase.ClassName;
  DataTableAux := TDataTableOfCosa.Create(DataTableName, clase, self.Version);
  DataRowAux := TDataRowOfCosa.Create(DataTableAux, 'dr_' + ColumnName);
  DataRowAux.SIUD:=TSIUD.TSELECT;
  DataTableAux.AddDataRow(DataRowAux);
  Count := dataTableAux.DataColumnListCount;
  self.IniciarLecturaRetrasada;
  j := 0;
  while j < count do
  begin
    DataRowAux.AddDataCell(TDataCellOfCosa.Create);
    DataColumn := DataTableAux.DataColumnList[j];

    if LowerCase(DataColumn.ColumnName)='flg_controlcotaobjetivoinferior' then
      a:=1;

    if (Datacolumn.ColumnName[1] <> ':') then
    begin
      if (DataColumn.DataType is TDataTypeDataRowOfCosa) then
      begin
          DataRowChild:=nil;
          self.rd(DataColumn.ColumnName, DataRowChild);
          self.EjecutarLectura; //FUNDAMENTAL PARA QUE TODO FUNCIONE BIEN!
          if (DataRowChild <> nil) then
            DataRowAux.LastDataCell.sval:=TDataTableOfCosa(DataRowChild.DataTableOwner).TableName + '.' + IntToStr(DataRowChild._RowId);
      end
      else if (DataColumn.DataType is TDataTypeListOfCosa) then
      begin
        if (cl = True) then
        begin
          DataTableOfList:=TDataTableOfListOfCosa(DATA_STORE.GetDataTableByName('dt_TList'));
          if Assigned(DataTableOfList) then
          begin
            LastIdParent := DataTableOfList.GetNextIdParent;
            DataRowAux.LastDataCell.sval:=DataTableOfList.TableName + '.IdParent.' + IntToStr(LastIdParent);
          end;
        end;
      end
      {else if (DataColumn.DataType is TDataTypeReferenciaOfCosa) then
      begin
        TDataTypeReferenciaOfCosa(DataColumn.DataType).referente:=DataRow;
        self.rd(DataColumn.ColumnName, DataRowAux.LastDataCell.sval);
      end}
      else
      begin
        //TRATAMIENTO ESPECIAL CUANDO HAY INSTANCIAS DE LISTAS (el campo 'n' me indica esto)
        if (LowerCase(DataColumn.ColumnName) = 'n') then
        begin
          n:=0;
          self.rd(DataColumn.ColumnName, n);
          self.EjecutarLectura;
          dataRowAux.LastDataCell.sval := IntToStr(n);
          if (n > 0) then
            cl := True; //VIENE UNA LISTA!
        end
        else
        begin
          self.rd(DataColumn.ColumnName, dataRowAux.LastDataCell._sval);

          if DataColumn.DataType is TDataTypeDoubleOfCosa then
          begin
            self.EjecutarLectura();

            //SE CONVIERTE EL REAL AL FORMATO LOCAL
            b:=dataRowAux.LastDataCell._sval;
            if b='0.87' then
              a:=1;

            ad:=StrToFloat(dataRowAux.LastDataCell._sval);
            uauxiliares.setSeparadoresLocales();
            dataRowAux.LastDataCell._sval:=FloatToStr(ad);
            b:=dataRowAux.LastDataCell._sval;
            uauxiliares.setSeparadoresGlobales();
          end;
          if DataColumn.DataType is TDataTypeDateOfCosa then
          begin
            self.EjecutarLectura();

            //SE CONVIERTE LA FECHA AL FORMATO LOCAL
            adt:=StrToDate(dataRowAux.LastDataCell._sval);
            uauxiliares.setSeparadoresLocales();
            dataRowAux.LastDataCell._sval:=DateToStr(adt);
            uauxiliares.setSeparadoresGlobales();
          end;
        end;
      end;
    end;
    if (j = Count - 1) then
      //Si se leyó último campo de los definidos en TListaDeCosasConNombre
      if (cl = True) then // Lo que viene ahora son las instancias de la lista
      begin
        //self.EjecutarLectura; //Fuerzo lectura del resto de los campos hasta las instancias de la lista.
        for z := 0 to n - 1 do
        begin
          DataRow2:=nil;
          self.rd(':', DataRow2);

          //ReadDataRowFromText(var DataRow: TDataRowOfCosa; ColumnName: string; cosaStrId: string)

          self.EjecutarLectura;
          DataTable2 := TDataTableOfCosa(DataRow2.DataTableOwner);
          DataRow2.RowName := 'dr_:' + IntToStr(DataTable2.DataRowListCount);

          //CREO EL REGISTRO EN LA TABLA DE LISTAS:
          DataRowOfList:=TDataRowOfCosa.Create(DataTableOfList, 'dr_'+IntToStr(DataTableOfList.DataRowListCount));
          DataRowOfList.SIUD:=TSIUD.TSELECT;
          DataRowOfList.AddDataCell(TDataCellOfCosa.Create(IntToStr(LastIdParent)));
          DataRowOfList.AddDataCell(TDataCellOfCosa.Create(DataTable2.TableName+'.'+IntToStr(DataRow2._RowId)));
          DataTableOfList.AddDataRow(DataRowOfList);

        end;
        cl := False;
      end;
    j := j + 1;
  end;
  self.EjecutarLectura;


  { SE OBTIENE UNA FILA CONVERTIDA A LA ÚLTIMA VERSIÓN
    --------------------------------------------------
    DataTableAux: es una tabla con la versión del archivo.
    DataTable: es una tabla con la última versión posible (VERSION=-2). }


  (********************************
   VER EL TEMA DE LA VERSION ENTRE DataTable y DataTableAux, así controlamos y
   no entra todo el tiempo. ***************************************************)
  if clase.ClassName='TListGenerador' then
    a:=1;

  DataTable:=DATA_STORE.GetDataTableByName(DataTableName);
  DataRow:=TDataRowOfCosa.Create(DataTable, DataRowAux.RowName);
  DataRow.SIUD:=DataRowAux.SIUD;
  DataTable.AddDataRow(DataRow);
  DataTable.ConvertDataRowToNewestVersion(DataRow, DataTableAux);
end;

procedure TArchiTexto.ReadCosaFromText(var cosa: TCosa; cosaStrId: string);
var
  clase: TClaseDeCosa;
begin
  clase := TClaseDeCosa(getClaseOf(CosaStrId));
  if clase = nil then
    raise ExcepcionClaseNoRegistrada.Create(
      'TArchiTexto.ReadCosaFromText: Error, ' + CosaStrId +
      ' No es una clase registrada. Procesando Línea: ' + IntToStr(klinea));
  cosa := clase.Create_ReadFromText(Self);

{$IFDEF WRLN_COSAS_CON_NOMBRE_LEIDAS}
  if cosa is TCosaConNombre then
    writeln(IntToStr(klinea) + ', Se leyó: ' + cosaStrId + '...' + cosa.apodo);
{$ENDIF}
end;


//-------------------------
// Métodos de TListaDeCosas
//=========================

constructor TListaDeCosas.Create(capa: integer; idCarpeta: string);
begin
  inherited Create(capa);
  self.idCarpeta := idCarpeta;
  lst := TList.Create;
end;

constructor TListaDeCosas.Create_ReadFromText(f: TArchiTexto);
var
  k: integer;
  cosas: array of TCosa;
begin
  inherited Create_ReadFromText(f);
  self.idCarpeta := f.aux_idCarpeta;
  f.IniciarLecturaRetrasada;
  f.rd('n', n);
  f.EjecutarLectura;
  lst := TList.Create;
  SetLength(cosas, n);
  f.IniciarLecturaRetrasada;
  for k := 0 to n - 1 do
    f.rd(':', TCosa(cosas[k]));
  f.EjecutarLectura;

  //Esto es porque puede estar mal grabado el archivo y no se carguen las n cosas
  for k := 0 to n - 1 do
    if cosas[k] <> nil then
      lst.Add(cosas[k]);
  SetLength(cosas, 0);
end;

procedure TListaDeCosas.WriteToText(f: TArchiTexto);
var
  n, k: integer;
  cosa: TCosa;
begin
  inherited WriteToText(f);
  n := lst.Count;
  f.wr('n', n);
  for k := 0 to n - 2 do
  begin
    cosa := lst.items[k];
    f.wr(':', cosa);
    f.writeline('');
  end;
  if n > 0 then
  begin
    cosa := lst.items[n - 1];
    f.wr(':', cosa);
  end;
end;

function TListaDeCosas.Add(cosa: TCosa): integer;
begin
  Result := self.lst.Add(cosa);
end;

procedure TListaDeCosas.ponerIgualA(listaDeCosas: TListaDeCosas; freeElementos: boolean);
var
  i: integer;
begin
  if freeElementos then
    ClearFreeElementos
  else
    Clear;

  lst.Capacity := listaDeCosas.lst.Count;
  for i := 0 to listaDeCosas.lst.Count - 1 do
    lst.Add(listaDeCosas.lst[i]);
end;


procedure TListaDeCosas.ActivarCapas(const CapasActivas: TDAOfNInt);
var
  k: integer;
  ac: TCosa;
begin
  k := 0;
  while (k < Count) do
  begin
    ac := items[k];
    if kInArray(CapasActivas, ac.capa) >= 0 then
    begin
      ac.ActivarCapas(CapasActivas);
      Inc(k);
    end
    else
    begin
      ac.Free;
      Delete(k);
    end;
  end;
end;

procedure TListaDeCosas.Free;
var
  k: integer;
begin
  for k := 0 to lst.Count - 1 do
    if lst.Items[k] <> nil then
      TCosa(lst.items[k]).Free;
  lst.Free;
  inherited Free;
end;

procedure TListaDeCosas.FreeSinElemenentos;
begin
  lst.Free;
  inherited Free;
end;

function TListaDeCosas.Count: integer;
begin
  Result := lst.Count;
end;

procedure TListaDeCosas.insert(indice: integer; cosa: TCosa);
begin
  lst.Insert(indice, cosa);
end;

function TListaDeCosas.Remove(cosa: TCosa): integer;
begin
  Result := lst.Remove(cosa);
end;

procedure TListaDeCosas.Delete(indice: integer);
begin
  lst.Delete(indice);
end;

procedure TListaDeCosas.Pack;
begin
  lst.Pack;
end;

procedure TListaDeCosas.Exchange(indice1, indice2: integer);
begin
  lst.Exchange(indice1, indice2);
end;

function TListaDeCosas.IndexOf(cosa: TCosa): integer;
begin
  Result := lst.IndexOf(cosa);
end;

function TListaDeCosas.replace(cosaARemplazar, cosaNueva: TCosa): integer;
var
  i: integer;
begin
  i := 0;
  while (i < Count) and (lst.Items[i] <> cosaARemplazar) do
    i := i + 1;
  if i < Count then
  begin
    lst.Items[i] := cosaNueva;
    Result := i;
  end
  else
    Result := -1;
end;

procedure TListaDeCosas.Clear;
begin
  lst.Clear;
end;

procedure TListaDeCosas.ClearFreeElementos;
var
  i: integer;
begin
  for i := 0 to lst.Count - 1 do
    TCosa(items[i]).Free;
  lst.Clear;
end;

procedure TListaDeCosas.Sort(Compare: TListSortCompare);
begin
  lst.Sort(Compare);
end;

function TListaDeCosas.getList: TList;
begin
  Result := lst;
end;

function TListaDeCosas.toArray: TDAOfTCosa;
var
  i: integer;
  res: TDAOfTCosa;
begin
  SetLength(res, lst.Count);
  for i := 0 to lst.Count - 1 do
    res[i] := lst[i];
  Result := res;
end;

function TListaDeCosas.getItem(i: integer): TCosa;
begin
  Result := lst.items[i];
end;

procedure TListaDeCosas.setItem(i: integer; cosa: TCosa);
begin
  lst.items[i] := cosa;
end;

function TListaDeCosas.getCapacity: integer;
begin
  Result := lst.Capacity;
end;

procedure TListaDeCosas.setCapacity(newCapacity: integer);
begin
  lst.Capacity := newCapacity;
end;



class function TListaDeCosas.CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: integer):
TDataColumnListOfCosa;
begin
  Result:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);

  Result.AddDataColumn('n', TListaDeCosas(nil).n, TDataTypeIntegerOfCosa, 113, -1);
  Result.AddDataColumn('lst', TListaDeCosas(nil).lst, TDataTypeListOfCosa, 113, -1);

end;


/////////////////////////
//Procedimientos Globales
/////////////////////////

//dfusco@20141023
//Se modifica la función para pasar al registro la lista de definicion de campos
//de la clase
function registrarClaseDeCosa(const nombre: string; Clase: TClaseDeCosa
  ): integer;
var
  ficha: TFichaClase;
  DataTable: TDataTableOfCosa;
  a: Integer;
begin
  if Assigned(DATA_STORE.GetDataTableByName('dt_' + Clase.ClassName)) then
    raise Exception.Create('La clase '+Clase.ClassName+' ya fue registrada, verificar procedimientos AlInicio');

  DataTable := TDataTableOfCosa.Create('dt_' + Clase.ClassName, TClaseDeCosa(Clase));
  DATA_STORE.AddDataTable(DataTable);

  ficha := TFichaClase.Create(nombre, clase);
  Result := registro_de_ClasesDeCosas_.lst.Add(ficha);
end;

function getClaseOf(const nombre: string): TCualquierClase;
var
  ipos: integer;
  res: TCualquierClase;
  slow: string;
begin
  res := nil;
  slow:= LowerCase( nombre );
  for ipos := 0 to registro_de_ClasesDeCosas_.lst.Count - 1 do
  begin
    //dfusco@20141017
    //Para comparar los string se convierte todo a LowerCase
    if LowerCase(TFichaClase(registro_de_ClasesDeCosas_.lst[ipos]).strClaseId) =
      slow then
    begin
      res := TFichaClase(registro_de_ClasesDeCosas_.lst.items[ipos]).clase;
      break;
    end;
  end;

  if res = nil then
    raise Exception.Create('ucosa.getClaseOf: Clase de Actor desconocida ' + nombre)
  else
    Result := res;
end;

// si la clase no está registrada retorna '<CLASE_NO_REGISTRADA>'
function getIdStrOf(clase: TClaseDeCosa): string;
var
  ipos: integer;
  buscando: boolean;
  b: String;
begin
  buscando := True;
  b:=clase.ClassName;
  for ipos := 0 to registro_de_ClasesDeCosas_.lst.Count - 1 do
    if TFichaClase(registro_de_ClasesDeCosas_.lst.items[ipos]).Clase = Clase then
    begin
      buscando := False;
      break;
    end;
  if buscando then
    Result := '<CLASE_NO_REGISTRADA>'
  else
    Result := TFichaClase(registro_de_ClasesDeCosas_.lst.items[ipos]).strClaseId;
end;


// Retorna la cadena de padres separados por #9
function ParentsStrList( clase: TClass ): string;
var
  res: string;
  padre: TClass;
begin
  res:= clase.ClassName;
  padre:= clase;
  while assigned( padre.ClassParent ) do
  begin
    padre:= padre.ClassParent;
    res:= padre.ClassName+'->'+res;
  end;
  result:= res;

end;

function IsChild(Parent, Child: TClass): boolean;
begin
  if not Assigned(Child) then
  begin
    Result:=False;
    exit;
  end;
  //  writeln(Parent.ClassName,' = ',Child.ClassName);
  if (Parent.ClassName = Child.ClassParent.ClassName) then
    Result := True
  else if Child.ClassParent.ClassName = TObject.ClassName then
    result := False
  else
    Result := IsChild(Parent, Child.ClassParent);
end;

procedure LimpiarRegistroDeClasesDeCosas;
var
  k: integer;
begin
  for k := 0 to registro_de_ClasesDeCosas_.lst.Count - 1 do
    TFichaClase(registro_de_ClasesDeCosas_.lst.items[k]).Free;
  registro_de_ClasesDeCosas_.lst.Clear;
end;

procedure AlInicio;
begin
  ReferenciasAArchivosExternos := TList.Create;
  lista_caminos := TStringList.Create;

  DATA_STORE := TDataStore.Create(nil);

{$IFDEF CNT_COSAS}
  CantCosas := TCntCosas.Create;
{$ENDIF}
  registro_de_ClasesDeCosas_ := TListaDeCosas.Create(0, 'RegistroClasesDeActor');
  //  registrarClaseDeCosa( TCosa.ClassName, TCosa );
  registrarClaseDeCosa(TListaDeCosas.ClassName, TListaDeCosas);

  DATA_STORE.AddDataTable(TDataTableOfListOfCosa.Create);

end;

procedure AlFinal;
begin
//  LimpiarRegistroDeClasesDeCosas;
  registro_de_ClasesDeCosas_.Free;
{$IFDEF CNT_COSAS}
  CantCosas.Free;
{$ENDIF}
  ReferenciasAArchivosExternos.Free;
  lista_caminos.Free;
end;

procedure FreeAndNil(var obj: TCosa);
var
  aux: TCosa;
begin
  aux := obj;
  obj := nil;
  aux.Free;
end;

function referenciaACosa(cosa: TCosa): string;
begin
  if assigned(cosa) and (cosa is TCosaConNombre) then
    Result := '<' + cosa.ClassName + '.' + TCosaConNombre(cosa).nombre + '>'
  else
    Result := '<?.?>';
end;

constructor TListaDeObjetosPorLeer.Create;
begin
  inherited Create;
end;

procedure TListaDeObjetosPorLeer.Free;
var
  k: integer;
begin
  for k := 0 to Count - 1 do
    TListCamposParaLeer(items[k]).Free;
  self.Clear;
  inherited Free;
end;

function TListaDeObjetosPorLeer.ListaActiva: TListCamposParaLeer;
begin
  if Count > 0 then
    Result := items[Count - 1]
  else
  begin
    raise Exception.Create('Error, no hay un Objeto activo para lectura ');
    Result := nil;
  end;
end;

procedure TListaDeObjetosPorLeer.FreeListaPorLeer;
begin
  if Count > 0 then
  begin
    TListCamposParaLeer(items[Count - 1]).Free;
    Delete(Count - 1);
  end;
end;

procedure TListaDeObjetosPorLeer.InicioObjeto(const NombreClase: string);
var
  a: TListCamposParaLeer;
begin
  a := TListCamposParaLeer.Create(NombreClase);
  add(a);
end;

procedure TListaDeObjetosPorLeer.FinalObjeto;
var
  xDataRow: TDataRowOfCosa;
  xCosa: TCosa;
  nombreDelCampoObjeto: string;
  lpl: TListCamposParaLeer;
  i: integer;
  res: boolean;
begin
  // procesar hasta el final del objeto
  lpl := ListaActiva;
  xDataRow := lpl.DataRow;
  xCosa := lpl.cosa;
  FreeListaPorLeer;
  lpl := ListaActiva;
  nombreDelCampoObjeto := lowercase(lpl.xNombre);
  res := False;
  for i := 0 to lpl.Count - 1 do
    if lowercase(TCampoParaLeer(lpl[i]).nombre) = nombreDelCampoObjeto then
    begin

      if xDataRow <> nil then
        TDataRowOfCosa(TCampoParaLeer(lpl.items[i]).pVal^) := xDataRow;
      if xCosa <> nil then
        TCosa(TCampoParaLeer(lpl.items[i]).pVal^) := xCosa;
      if ((xDataRow = nil) and (xCosa = nil)) then
        raise Exception.Create(
          'TListaDeObjetosPorLeer.FinalObjeto: xDataRow=xCosa=nil');

      TCampoParaLeer(lpl.items[i]).Free;
      lpl.Delete(i);
      res := True;
      break;
    end;
  if not res then
    raise Exception.Create('Aparentemente el objeto: ' +
      lpl.IdentificadorDeLaClaseDelObjeto + ' no tienen definido el campo: ' +
      nombreDelCampoObjeto);
end;


constructor TArchiTexto.CreateForRead(const nombreArchivo: string;
  abortarEnError: boolean);
var
  s: string;
  iposver: integer;
begin
  setSeparadoresGlobales;
  if FileExists(nombreArchivo) then
  begin
    inherited Create;

    f_abierto := False;
    unArchivoPorCosa := False;
    padre := nil;
    ramas := nil;


    Self.NombreArchivo := nombreArchivo;
    oldFileMode := FileMode;
    filemode := fmOpenRead;

    assignfile(f, NombreARchivo);
    {$I-}
    reset(f);
    {$I+}
    if ioresult = 0 then
      f_abierto := True
    else
      raise Exception.Create('TArchiTexto.CreateForRead: No pude abrir archivo: ' +
        NombreArchivo);

    self.abortarEnError := abortarEnError;

    readln(f, s);
    s := trim(s);

    klinea := 0;

    iposver := pos('VERSION=', s);
    if iposver >= 1 then
    begin
      Delete(s, 1, iposver - 1 + length('VERSION='));
      s := trim(s);
      version := StrToInt(s);
      Inc(klinea);
    end
    else
      version := 0;

    // parche chancho para poder leer archivos de la gente de la regasificadora
    if (version < 10000) and (version > VERSION_ArchiTexto) then
      raise Exception.Create(rs_ElArchivo + ' ' + nombreArchivo +
        ' ' + rs_VersionPosterior);


    if version <= 45 then
      setSeparadoresGlobales_FechaUruguaya_V45_;

    listaObjetosPorLeer := TListaDeObjetosPorLeer.Create;
    indentstr := '';
    listaObjetosPorLeer.InicioObjeto('TODO');
  end
  else
    raise Exception.Create('TArchiTexto.CreateForRead: ' +
      rs_NoSeEncuentraArchivo + nombreArchivo);
end;



constructor TArchiTexto.CreateForWrite(const nombreArchivo: string;
  crearBackupSiExiste: boolean; maxNBackups: integer);
begin
  inherited Create;
  setSeparadoresGlobales;

  unArchivoPorCosa := False;
  padre := nil;
  ramas := nil;

  f_abierto := False;
  Self.NombreArchivo := NombreArchivo;

  if crearBackupSiExiste and FileExists(nombreArchivo)
  { *Converted from FileExists*  } then
    uconstantesSimSEE.backupearArchivoAntesDeSalvar(nombreArchivo, maxNBackups);
  assignFile(f, NombreArchivo);
  oldFileMode := FileMode;
  filemode := fmOpenWrite;
   {$I-}
  Rewrite(f);
   {$I+}
  if ioresult = 0 then
    f_abierto := True
  else
    raise Exception.Create('TArchiTexto.CreateForWrite(' + nombreArchivo + ')');

  writeln(f, 'VERSION=' + IntToStr(VERSION_ArchiTexto));
  klinea := 1;
  listaObjetosPorLeer := TListaDeObjetosPorLeer.Create;
end;


function TArchiTexto.CreateRamaForWrite(
  const Carpeta, NombreClase, NombreCosa: string): TArchiTexto;
var
  s: string;
  i: integer;
  a: TArchiTexto;
begin
  if ramas = nil then
    ramas := TList.Create;
  i := pos('.sse', nombreArchivo);
  s := copy(nombreArchivo, 1, i - 1);
  if carpeta <> '' then
    s := s + '/' + carpeta + '/';

  s := s + '$' + NombreClase + '$' + NombreCosa + '.sse';

  a := TArchiTexto.CreateForWrite(s, False, 0);
  a.unArchivoPorCosa := True;
  ramas.Add(a);
  Result := a;
end;


function TArchiTexto.CreateRamaForRead(const carpeta, NombreClase,
  NombreCosa: string): TArchiTexto;
var
  s: string;
  i: integer;
  a: TArchiTexto;
begin
  if ramas = nil then
    ramas := TList.Create;
  i := pos('.sse', nombreArchivo);
  s := copy(nombreArchivo, 1, i - 1);
  if carpeta <> '' then
    s := s + '/' + carpeta + '/';

  s := s + '$' + NombreClase + '$' + NombreCosa + '.sse';
  a := TArchiTexto.CreateForRead(s, self.abortarEnError);
  a.unArchivoPorCosa := True;
  ramas.Add(a);
  Result := a;
end;


procedure TArchiTexto.Free;
var
  i: integer;
  //  a: TArchiTexto;
begin
  setSeparadoresLocales;

  if listaObjetosPorLeer <> nil then
    listaObjetosPorLeer.Free;

  if ramas <> nil then
  begin
    if ramas.Count > 0 then
      raise Exception.Create(
        'Toy tratando de FREE un ArchiTexto pero tiene RAMAS, revisar esto')
    else
      ramas.Free;
(* creo que no es necesario esto, si lo de arriba salta, talvez tenga que
   usar lo de abajo
    for i:= 0 to ramas.Count - 1 do
    begin
      a:= ramas.Items[i];
      a.padre:= self.padre;
      padre.ramas.Add( a );
    end;
*)
  end;

  if padre <> nil then
  begin
    // quitamos este archivo de la lista del padre
    i := padre.ramas.IndexOf(self);
    padre.ramas.Delete(i);
  end;

  if (f_abierto) then
  begin
    f_abierto := False;
    closefile(f);
  end;
  FileMode := oldFileMode;


  inherited Free;
end;

function TArchiTexto.NextLine: string;
var
  r: string;
begin
  if not system.EOF(f) then
  begin
    readln(f, r);
    Inc(klinea);
    Result := trim(r);
  end
  else
    raise Exception.Create('Llegué al fin del archivo en forma inesperada.');
end;

procedure TArchiTexto.writeline(const s: string);
begin
  writeln(f, indentstr + s);
end;

function TArchiTexto.EOF: boolean;
begin
  Result := system.EOF(f);
end;

function TArchiTexto.quedanCosasDelObjetoActualPorLeer(claseObjeto: string): boolean;
var
  s: string;
begin
  s := NextLine;
  //Solo puede haber lineas vacias, sino devuelvo true
  while (s <> '<-' + claseObjeto + '>;') and (trim(s) = '') do
    s := NextLine;
  if s = '<-' + claseObjeto + '>;' then
    Result := False
  else
    Result := True;
end;

function TArchiTexto.ListaPorLeer: TListCamposParaLeer;
begin
  Result := listaObjetosPorLeer.ListaActiva;
end;

procedure TArchiTexto.wr(const nombre: string; const valor: NReal;
  precision: integer; decimales: integer);
begin
  writeline(nombre + '= ' + FloatToStrF(valor, ffGeneral, precision, decimales) + ';');
end;

procedure TArchiTexto.wr(const nombre: string; const valor: NReal);
begin
  wr(nombre, valor, CF_PRECISION, CF_DECIMALES);
end;

procedure TArchiTexto.wr(const nombre: string; const valor: string);
begin
  writeline(nombre + '= ' + valor + ';');
end;

procedure TArchiTexto.wrArchRef(const nombre: string; const valor: TArchiRef);
begin
  if valor <> nil then
    writeline(nombre + '= ' + valor.archiRefStr + ';')
  else
    writeline(nombre + '= ;');
end;

procedure TArchiTexto.wr(const nombre: string; const valor: NInt);
begin
  writeline(nombre + '= ' + IntToStr(valor) + ';');
end;

procedure TArchiTexto.wr(const nombre: string; const valor: cardinal);
begin
  writeline(nombre + '= ' + IntToStr(valor) + ';');
end;

procedure TArchiTexto.wr(const nombre: string; const valor: TFecha);
begin
  if valor <> nil then
    writeline(nombre + '= ' + valor.AsStr + ';')
  else
    writeline(nombre + '= 0;');
end;

procedure TArchiTexto.wr(const nombre: string; const valor: TCosa);
var
  CosaStrId: string;
  clase: TClaseDeCosa;
  b: String;
begin
  if valor <> nil then
  begin
    b:=valor.ClassName;
    clase := TClaseDeCosa(valor.ClassType);
    CosaStrId := getIdStrOf(clase);
    writeline(nombre + '= <+' + CosaStrId + '>');
    indentstr := indentstr + '  ';
    valor.WriteToText(Self);
    Delete(indentstr, length(indentstr) - 1, 2);
    writeline('<-' + CosaStrId + '>;');
  end
  else
    writeline(nombre + '= NIL;');
end;

procedure TArchiTexto.wr(const nombre: string; const valor: boolean);
begin
  if valor then
    writeline(nombre + '= 1;')
  else
    writeline(nombre + '= 0;');
end;

procedure TArchiTexto.wr(const nombre: string; const valor: TDAOfNReal;
  precision: integer; decimales: integer);
var
  k: integer;
begin
  Write(f, indentstr + nombre, '= [', length(valor), '| ');
  if length(valor) > 0 then
  begin
    Write(f, FloatToStrF(valor[0], ffFixed, precision, decimales));
    for k := 1 to high(valor) do
    begin
      if ((k mod 12) = 0) then
        writeline('  ');
      Write(f, ', ', FloatToStrF(valor[k], ffFixed, precision, decimales));
    end;
  end;
  writeln(f, '];');
end;

procedure TArchiTexto.wr(const nombre: string; const valor: TDAOfNReal);
begin
  wr(nombre, valor, 12, 3);
end;

procedure TArchiTexto.wr(const nombre: string; const valor: TDAOfNInt);
var
  k: integer;
begin
  Write(f, indentstr + nombre, '= [', length(valor), '| ');
  if length(valor) > 0 then
  begin
    Write(f, valor[0]);
    for k := 1 to high(valor) do
    begin
      if ((k mod 12) = 0) then
        writeline('  ');
      Write(f, ', ', valor[k]);
    end;
  end;
  writeln(f, '];');
end;

procedure TArchiTexto.wr(const nombre: string; const valor: TDAOfBoolean); overload;
var
  k: integer;
begin
  Write(f, indentstr + nombre, '= [', length(valor), '| ');
  if length(valor) > 0 then
  begin
    Write(f, valor[0]);
    for k := 1 to high(valor) do
    begin
      if ((k mod 12) = 0) then
        writeline('  ');
      if Valor[k] then
        Write(f, ', ', valor[k])
      else
        Write(f, ', ', valor[k]);
    end;
  end;
  writeln(f, '];');
end;


procedure TArchiTexto.wr(const nombre: string; const valor: TStringList);
var
  k: integer;
begin
  Write(f, indentstr + nombre, '= [', valor.Count, '| ');
  if valor.Count > 0 then
  begin
    Write(f, valor[0]);
    for k := 1 to valor.Count - 1 do
    begin
      if ((k mod 12) = 0) then
        writeline('  ');
      Write(f, ', ', valor[k]);
    end;
  end;
  writeln(f, '];');
end;


procedure TArchiTexto.wr(const nombre: string; const valor: TDAOfString);
var
  k: integer;
begin
  Write(f, indentstr + nombre, '= [', length(valor), '| ');
  if length(valor) > 0 then
  begin
    Write(f, valor[0]);
    for k := 1 to high(valor) do
    begin
      if ((k mod 12) = 0) then
        writeline('  ');
      Write(f, ', ', valor[k]);
    end;
  end;
  writeln(f, '];');
end;



procedure TArchiTexto.wrReferencia(const nombre: string; const valor: TCosa);
begin
  writeline(nombre + '= ' + referenciaACosa(valor) + ';');
{  if assigned(valor) and (valor is TCosaConNombre) then
    writeline(nombre + '= <' + valor.ClassName + '.' +
      TCosaConNombre(valor).nombre + '>;')
(* rch0707121651 comento esto para que si está mal escriba referencia vacía y listo
    else
      raise Exception.Create('TArchiTexto.wrReferencia: intenta escribir una referencia a algo que no es TCosaConNombre');
  end
*)
  else
    writeline(nombre + '= <?.?>;');}
end;

procedure TArchiTexto.rd(const nombre: string; var valor: NReal);
begin
  ListaPorLeer.Esperar_NReal(nombre, valor, nil);
  if not ListaPorLeer.lecturaRetrasada then
    EjecutarLectura;
end;

procedure TArchiTexto.rd(const nombre: string; var valor: string);
var
  a: integer;
begin
  ListaPorLeer.Esperar_String(nombre, valor, nil);
  if not ListaPorLeer.lecturaRetrasada then
    EjecutarLectura;
end;

procedure TArchiTexto.rdArchRef(const nombre: string; var valor: TArchiRef);
begin
  ListaPorLeer.Esperar_ArchRef(nombre, valor, nil);
  if not ListaPorLeer.lecturaRetrasada then
    EjecutarLectura;
end;

procedure TArchiTexto.rd(const nombre: string; var valor: NInt);
begin
  ListaPorLeer.Esperar_NInt(nombre, valor, nil);
  if not ListaPorLeer.lecturaRetrasada then
    EjecutarLectura;
end;

procedure TArchiTexto.rd(const nombre: string; var valor: cardinal);
begin
  ListaPorLeer.Esperar_NCardinal(nombre, valor, nil);
  if not ListaPorLeer.lecturaRetrasada then
    EjecutarLectura;
end;

procedure TArchiTexto.rd(const nombre: string; var valor: TFecha);
begin
  ListaPorLeer.Esperar_Fecha(nombre, valor, nil);
  if not ListaPorLeer.lecturaRetrasada then
    EjecutarLectura;
end;

// flgError= 0 lo mismo que sin flg. flgError= 1 no avisa del error
procedure TArchiTexto.rd(const nombre: string; var valor: TFecha; flgError: integer);
begin
  ListaPorLeer.Esperar_Fecha(nombre, valor, nil, flgError);
  if not ListaPorLeer.lecturaRetrasada then
    EjecutarLectura;
end;


procedure TArchiTexto.rd(const nombre: string; var valor: boolean);
begin
  ListaPorLeer.Esperar_Boolean(nombre, valor, nil);
  if not ListaPorLeer.lecturaRetrasada then
    EjecutarLectura;
end;

procedure TArchiTexto.rd(const nombre: string; var valor: TDAOfNReal);
begin
  ListaPorLeer.Esperar_DAOfNReal(nombre, valor, nil);
  if not ListaPorLeer.lecturaRetrasada then
    EjecutarLectura;
end;

procedure TArchiTexto.rd(const nombre: string; var valor: TDAOfBoolean); overload;
begin
  ListaPorLeer.Esperar_DAOfBoolean(nombre, valor, nil);
  if not ListaPorLeer.lecturaRetrasada then
    EjecutarLectura;
end;

procedure TArchiTexto.rd(const nombre: string; var valor: TDataRowOfCosa);
begin
  ListaPorLeer.Esperar_TDataRowOfCosa(nombre, valor, nil);
  if not ListaPorLeer.lecturaRetrasada then
    EjecutarLectura;
end;

procedure TArchiTexto.rd(const nombre: string; var valor: TCosa);
begin
  ListaPorLeer.Esperar_Cosa(nombre, valor, nil);
  if not ListaPorLeer.lecturaRetrasada then
    EjecutarLectura;
end;

procedure TArchiTexto.rd(const nombre: string; var valor: TDAOfNInt);
begin
  ListaPorLeer.Esperar_DAOfNInt(nombre, valor, nil);
  if not ListaPorLeer.lecturaRetrasada then
    EjecutarLectura;
end;

procedure TArchiTexto.rd(const nombre: string; var valor: TDAOfString);
begin
  ListaPorLeer.Esperar_DAOfString(nombre, valor, nil);
  if not ListaPorLeer.lecturaRetrasada then
    EjecutarLectura;
end;

procedure TArchiTexto.rd(const nombre: string; var valor: TStringList);
begin
  ListaPorLeer.Esperar_StringList(nombre, valor, nil);
  if not ListaPorLeer.lecturaRetrasada then
    EjecutarLectura;
end;



procedure TArchiTexto.rdReferencia(const nombre: string; var valor: TCosa;
  referente: TCosa);
begin
  ListaPorLeer.Esperar_Referencia(nombre, valor, nil, referente);
  if not ListaPorLeer.lecturaRetrasada then
    EjecutarLectura;
end;

procedure TArchiTexto.IniciarLecturaRetrasada;
begin
  if ListaPorLeer <> nil then
    ListaPorLeer.lecturaRetrasada := True;
end;

procedure TArchiTexto.EjecutarLectura;
var
  s, NombreClaseSiguienteObjeto: string;
  resObjetoHijo, resObjetoPadre: TResultadoProcesoLinea;
  lpl: TListCamposParaLeer;
{$IFDEF DBG}
  dbgAux: TDAofString;
{$ENDIF}
  indiceDeLinea: integer;
  posIgual: integer;
  ColumnName: string;
  i: integer;
  a: integer;
begin
  ColumnName:='';
  NombreClaseSiguienteObjeto:='';
  try
    lpl := ListaPorLeer;
    resObjetoPadre := CTPLR_CONTINUAR;
    while (resObjetoPadre <> CTPLR_FIN_OBJETO) and (lpl <> nil) and
      (lpl.Count > 0) and not EOF do
    begin
      {$IFDEF DBG}
      dbgAux := lpl.nombresCamposPorLeer;
      {$ENDIF}

      indiceDeLinea := klinea;
      s := nextline;
      system.writeln(indiceDeLinea: 6, ': ', s);
      try
        resObjetoPadre := lpl.ProcesarLinea(klinea, s, NombreClaseSiguienteObjeto,
          ColumnName);
      except
        on E: ExcepcionNoSePidioElCampo do
        begin
          lpl.Ejecutar_vpds(klinea);
          if quedanCosasDelObjetoActualPorLeer(lpl.IdentificadorDeLaClaseDelObjeto) then
          begin
            resObjetoPadre := CTPLR_CONTINUAR;
            klinea := indiceDeLinea;
          end
          else
            resObjetoPadre := CTPLR_FIN_OBJETO;
        end;
        on E: ExcepcionResolverCampoValor do
        begin
          posIgual := pos('=', s);
          lpl.Ejecutar_vpd(klinea, copy(s, 0, posIgual - 1));
          if quedanCosasDelObjetoActualPorLeer(lpl.IdentificadorDeLaClaseDelObjeto) then
          begin
            resObjetoPadre := CTPLR_CONTINUAR;
            klinea := indiceDeLinea + 1;
            //El +1 debería ser el número de lineas que leyo y no le sirvieron
          end
          else
            resObjetoPadre := CTPLR_FIN_OBJETO;
        end
      end;
      if resObjetoPadre = CTPLR_ABRIR_NUEVO_OBJETO then
      begin
        listaObjetosPorLeer.InicioObjeto(NombreClaseSiguienteObjeto);
        lpl := ListaPorLeer;
        try
          if Version >= 10113 then
            ReadDataRowFromText(lpl.DataRow, ColumnName, NombreClaseSiguienteObjeto)
          else
            ReadCosaFromText(lpl.cosa, NombreClaseSiguienteObjeto);

        except
          on E: Exception do
          begin
            lpl.DataRow := nil;
            lpl.cosa := nil;
            if Assigned(procMsgErrorLectura) then
              procMsgErrorLectura(E.Message + '. Leyendo el actor en la linea ' +
                IntToStr(klinea));
            if abortarEnError then
              raise;

            if not EOF then
              repeat
                s := NextLine;
              until (s = '<-' + NombreClaseSiguienteObjeto + '>;') or EOF;
            lpl.leiFinDeObjeto := True;
          end
        end;
        if not lpl.leiFinDeObjeto then
          repeat
            s := nextline;
            resObjetoHijo := lpl.ProcesarLinea(klinea, s,
              NombreClaseSiguienteObjeto, ColumnName);
          until resObjetoHijo = CTPLR_FIN_OBJETO;

        listaObjetosPorLeer.FinalObjeto;
        lpl := ListaPorLeer;
      end;
    end;
    if resObjetoPadre = CTPLR_FIN_OBJETO then
      ListaPorLeer.leiFinDeObjeto := True;
  except
    on E: Exception do
      if pos('kLinea:', E.Message) = 0 then
        raise Exception.Create('kLinea:(' + IntToStr(klinea) + ') ' + E.Message)
      else
        raise;
  end; // try
  //ListaPorLeer.lecturaRetrasada:= false;
end;

function TipoToStr(tipo: TTipoCampo): string;
begin
  case Tipo of
    CTC_NReal: Result := 'NReal';
    CTC_String: Result := 'String';
    CTC_ArchRef: Result := 'Ref. Archivo Externo';
    CTC_NInt: Result := 'Entero';
    CTC_NCardinal: Result := 'Entero Positivo';
    CTC_Boolean: Result := 'Booleano';
    CTC_DAOfNReal: Result := 'Array dinámico de Reales';
    CTC_DAOfNInt: Result := 'Array dinámico de Enteros';
    CTC_DAOfBoolean: Result := 'Array dinámico de Booleanas';
    CTC_Fecha: Result := 'Fecha';
    CTC_TDataRowOfCosa: Result := 'TDataRowOfCosa';
    CTC_Cosa: Result := 'Cosa';
  end; // case
end;

procedure ValorPorDefectoNulo(const nombre: string; pVal: Pointer;
  tipo: TTipoCampo; const klinea: integer);
var
  Msg: string;
begin
  case Tipo of
    CTC_NReal: NReal(pVal^) := 0;
    CTC_String: string(pVal^) := '';
    CTC_ArchRef: TArchiRef(pVal^) := TArchiRef.Create('');
    CTC_NInt: NEntero(pVal^) := 0;
    CTC_NCardinal: cardinal(pVal^) := 0;
    CTC_Boolean: boolean(pVal^) := False;
    CTC_DAOfNReal: setlength(TDAOfNReal(pVal^), 0);
    CTC_DAOfNInt: setlength(TDAOfNInt(pVal^), 0);
    CTC_DAOfBoolean: setlength(TDAOfBoolean(pVal^), 0);
    CTC_Fecha: TFecha(pVal^) := TFecha.Create_dt(0);
    CTC_TDataRowOfCosa: pointer(pVal^) := nil;
    CTC_Cosa: pointer(pVal^) := nil;
    CTC_Referencia: Pointer(pVal^) := nil;
  end; // case


  if global_flgError = 0 then
    if assigned(procMsgValorPorDefecto) then
    begin
      msg := 'Asignando valor por defecto al campo ' + nombre;
      if klinea <> -1 then
        msg := msg + ' leyendo el final del objeto en la linea ' + IntToStr(klinea)
      else
        msg := msg + ' en el final del archivo.';
      procMsgValorPorDefecto(msg);
    end;
end;

procedure ValorPorDefecto_ERROR(const nombre: string; pVal: Pointer;
  tipo: TTipoCampo; const klinea: integer);
var
  msg: string;
begin
  msg := 'No se encontró una definición para el campo: ' + nombre +
    ' tipo: ' + TipoToStr(tipo);
  if klinea <> -1 then
    msg := msg + ' leyendo el final del objeto en la linea ' + IntToStr(klinea)
  else
    msg := msg + ' en el final del archivo.';

  //  raise Exception.Create(msg);

{$IFDEF FPC}
  writeln(msg);
  Write('... presione ENTER para continuar.');
  system.readln;
{$ELSE}
  ShowMessage(msg);
{$ENDIF}
end;

constructor TCampoParaLeer.Create(const Nombre: string; pVal: Pointer;
  Tipo: TTIpoCampo; proc_vpd: TProcFijarValorPorDefecto);
begin
  Create(Nombre, pVal, Tipo, proc_vpd, nil);
end;

constructor TCampoParaLeer.Create(const Nombre: string; pVal: Pointer;
  Tipo: TTIpoCampo; proc_vpd: TProcFijarValorPorDefecto; flgError: integer);
begin
  Create(Nombre, pVal, Tipo, proc_vpd, nil);
  self.flgError := flgError;
end;


constructor TCampoParaLeer.Create(const Nombre: string; pVal: Pointer;
  Tipo: TTipoCampo; proc_vpd: TProcFijarValorPorDefecto; referente: TCosa);
begin
  inherited Create;
  self.flgError := 0;
  self.nombre := nombre;
  self.tipo := tipo;
  self.pVal := pVal;
  self.referente := referente;

{$IFDEF PORDEFECTO_NULO}
  if not assigned(proc_vpd) then
    self.proc_vpd := ValorPorDefectoNulo
  else
    self.proc_vpd := proc_vpd;
{$ELSE}
  self.proc_vpd := proc_vpd;
{$ENDIF}
end;

procedure TCampoParaLeer.Ejecutar_vpd(const klinea: integer);
begin
  if assigned(proc_vpd) then
  begin
    global_flgError := Self.flgError;
    proc_vpd(nombre, pVal, tipo, klinea);
  end
  else
  if flgError = 0 then
    ValorPorDefecto_ERROR(nombre, pVal, tipo, klinea);
end;

constructor TListCamposParaLeer.Create(IdentificadorDeLaClaseDelObjeto: string);
begin
  inherited Create;
  self.IdentificadorDeLaClaseDelObjeto := IdentificadorDeLaClaseDelObjeto;
  lineaAbierta := False;
  xNombre := '';
  xval := '';
  lecturaRetrasada := False;
  leiFinDeObjeto := False;

  self.DataRow := nil;
  self.cosa := nil;
end;

procedure TListCamposParaLeer.Esperar_NReal(const Nombre: string;
  var Variable: NReal; proc_vpd: TProcFijarValorPorDefecto);
var
  a: TCampoParaLeer;
begin
  a := TCampoParaLeer.Create(Nombre, @Variable, CTC_NReal, proc_vpd);
  add(a);
end;

procedure TListCamposParaLeer.Esperar_String(const Nombre: string;
  var Variable: string; proc_vpd: TProcFijarValorPorDefecto);
var
  a: TCampoParaLeer;
begin
  a := TCampoParaLeer.Create(Nombre, @Variable, CTC_String, proc_vpd);
  add(a);
end;

procedure TListCamposParaLeer.Esperar_ArchRef(const Nombre: string;
  var Variable: TArchiRef; proc_vpd: TProcFijarValorPorDefecto);
var
  a: TCampoParaLeer;
begin
  a := TCampoParaLeer.Create(Nombre, @Variable, CTC_ArchRef, proc_vpd);
  add(a);
end;

procedure TListCamposParaLeer.Esperar_NInt(const Nombre: string;
  var Variable: integer; proc_vpd: TProcFijarValorPorDefecto);
var
  a: TCampoParaLeer;
begin
  a := TCampoParaLeer.Create(Nombre, @Variable, CTC_NInt, proc_vpd);
  add(a);
end;

procedure TListCamposParaLeer.Esperar_NCardinal(const Nombre: string;
  var Variable: cardinal; proc_vpd: TProcFijarValorPorDefecto);
var
  a: TCampoParaLeer;
begin
  a := TCampoParaLeer.Create(Nombre, @Variable, CTC_NCardinal, proc_vpd);
  add(a);
end;

procedure TListCamposParaLeer.Esperar_Boolean(const Nombre: string;
  var Variable: boolean; proc_vpd: TProcFijarValorPorDefecto);
var
  a: TCampoParaLeer;
begin
  a := TCampoParaLeer.Create(Nombre, @Variable, CTC_Boolean, proc_vpd);
  add(a);
end;

procedure TListCamposParaLeer.Esperar_DAOfNReal(const Nombre: string;
  var Variable: TDAOfNReal; proc_vpd: TProcFijarValorPorDefecto);
var
  a: TCampoParaLeer;
begin
  a := TCampoParaLeer.Create(Nombre, @Variable, CTC_DAOfNReal, proc_vpd);
  add(a);
end;

procedure TListCamposParaLeer.Esperar_DAOfNInt(const Nombre: string;
  var Variable: TDAOfNInt; proc_vpd: TProcFijarValorPorDefecto);
var
  a: TCampoParaLeer;
begin
  a := TCampoParaLeer.Create(Nombre, @Variable, CTC_DAOfNInt, proc_vpd);
  add(a);
end;


procedure TListCamposParaLeer.Esperar_DAOfBoolean(const Nombre: string;
  var Variable: TDAOfBoolean; proc_vpd: TProcFijarValorPorDefecto);
var
  a: TCampoParaLeer;
begin
  a := TCampoParaLeer.Create(Nombre, @Variable, CTC_DAOfBoolean, proc_vpd);
  add(a);
end;


procedure TListCamposParaLeer.Esperar_DAOfString(const Nombre: string;
  var Variable: TDAOfString; proc_vpd: TProcFijarValorPorDefecto);
var
  a: TCampoParaLeer;
begin
  a := TCampoParaLeer.Create(Nombre, @Variable, CTC_DAOfString, proc_vpd);
  add(a);
end;



procedure TListCamposParaLeer.Esperar_StringList(const Nombre: string;
  var Variable: TStringList; proc_vpd: TProcFijarValorPorDefecto);
var
  a: TCampoParaLeer;
begin
  a := TCampoParaLeer.Create(Nombre, @Variable, CTC_StringList, proc_vpd);
  add(a);
end;

procedure TListCamposParaLeer.Esperar_TDataRowOfCosa(const nombre: string;
  var variable: TDataRowOfCosa; proc_vpd: TProcFijarValorPorDefecto);
var
  a: TCampoParaLeer;
begin
  a := TCampoParaLeer.Create(Nombre, @Variable, CTC_TDataRowOfCosa, proc_vpd);
  add(a);
end;

procedure TListCamposParaLeer.Esperar_Cosa(const Nombre: string;
  var Variable: TCosa; proc_vpd: TProcFijarValorPorDefecto);
var
  a: TCampoParaLeer;
begin
  a := TCampoParaLeer.Create(Nombre, @Variable, CTC_Cosa, proc_vpd);
  add(a);
end;

procedure TListCamposParaLeer.Esperar_Referencia(const Nombre: string;
  var Variable: TCosa; proc_vpd: TProcFijarValorPorDefecto; referente: TCosa);
var
  a: TCampoParaLeer;
begin
  a := TCampoParaLeer.Create(Nombre, @Variable, CTC_Referencia, proc_vpd, referente);
  add(a);
end;

procedure TListCamposParaLeer.Esperar_Fecha(const Nombre: string;
  var Variable: TFecha; proc_vpd: TProcFijarValorPorDefecto);
var
  a: TCampoParaLeer;
begin
  a := TCampoParaLeer.Create(Nombre, @Variable, CTC_Fecha, proc_vpd);
  add(a);
end;

procedure TListCamposParaLeer.Esperar_Fecha(const Nombre: string;
  var Variable: TFecha; proc_vpd: TProcFijarValorPorDefecto; flgError: integer);
var
  a: TCampoParaLeer;
begin
  a := TCampoParaLeer.Create(Nombre, @Variable, CTC_Fecha, proc_vpd, flgError);
  add(a);
end;

(*
    CTPLR_CONTINUAR,
    CTPLR_FIN_OBJETO,
    CTPLR_ABRIR_NUEVO_OBJETO );
*)
function TListCamposParaLeer.ProcesarLinea(klinea: integer; linea: string;
  var extres: string; var ColumnName: string): TResultadoProcesoLinea;
var
  NombreClase: string;
begin
  Result := CTPLR_CONTINUAR;
  if pos('<-', linea) = 1 then
  begin
    if IdentificadorDeLaClaseDelObjeto <> '' then
      if pos(IdentificadorDeLaClaseDelObjeto, linea) <> 3 then
        raise Exception.Create('Error, cierre de objeto : ' + linea +
          ' Se esperaba: ' + '<-' + IdentificadorDeLaClaseDelObjeto +
          '>, en línea: ' + IntToStr(klinea));

    Ejecutar_vpds(klinea);      // procesamos los no resueltos
    Result := CTPLR_FIN_OBJETO; // cierro esta lista FIN DEL OBJETO
  end
  else
  begin
    if LineaAbierta then
    begin
      xval := xval + ' ' + linea;
      if xval[length(xval)] = ';' then
      begin
        xval := copy(xval, 0, Length(xval) - 1);//Saco el ;
        ResolverCampoValor(xNombre, xVal);
        LineaAbierta := False;
      end;
    end
    else
    begin
      if getPalHastaSep(xnombre, linea, '=') then
      begin
        xval := trim(linea);
        if length(xval) > 0 then
        begin
          if pos('<+', xval) = 1 then
          begin
            system.Delete(xval, 1, 2);
            if xval[length(xval)] <> '>' then
              raise Exception.Create('Error, falta el > de cierre en fin de clase : '
                + xval);
            NombreClase := copy(xval, 1, length(xval) - 1);
            ColumnName := xNombre;
            extres := NombreClase;
            Result := CTPLR_ABRIR_NUEVO_OBJETO;
          end
          else
          if xval[length(xval)] = ';' then
            //if getPalHastaSep(xval, linea, ';') then
          begin
            xval := copy(xval, 0, Length(xval) - 1);//Saco el ;
            try
              ResolverCampoValor(xNombre, xVal)
            except
              on E: ExcepcionNoSePidioElCampo do
              begin
                if Assigned(procMsgAdvertenciaLectura) then
                  procMsgAdvertenciaLectura(E.Message);
              end;
              on Exception do
              begin
                raise;
              end;
            end;
          end
          else
            LineaAbierta := True;
        end
        else
          LineaAbierta := True;
      end
      else
        Exception.Create('Error, no encontré el (=) en la línea: [' +
          IntToStr(klinea) + '] :' + linea);
    end;
  end;
end;

procedure TListCamposParaLeer.Free;
var
  k: integer;
begin
  Ejecutar_vpds(-1);
  //PAlfaro@101104 0307
  //Ojo, estaba comentado, pero me parece que tiene que ir. No se tranca y sin esto
  //quedarían colgados todos los campos para leer de cada TCosa
  for k := 0 to Count - 1 do
    TCampoParaLeer(items[k]).Free;
  inherited Free;
end;

procedure TListCamposParaLeer.ResolverCampoValor(const xNombre, xVal: string);
var
  buscando: boolean;
  k: integer;
  clase, nombreRef: string;
  lc_xNombre, lct: string;
  ts: string;
begin
  buscando := True;
  k := 0;
  lc_xNombre := lowercase(xNombre);
  while buscando and (k < Count) do
  begin
    lct := lowercase(TCampoParaLeer(items[k]).nombre);
    if lct = lc_xNombre then
      buscando := False
    else
      Inc(k);
  end;

  if buscando then
    raise ExcepcionNoSePidioElCampo.Create('Se leyó el campo: ' +
      xnombre + ' pero no se esperaba en el objeto: ' +
      IdentificadorDeLaClaseDelObjeto +
      '. Pruebe guardar y cargar nuevamente la sala. Intentaré solucionar el problema.')
  else
  begin
    try
      with  TCampoParaLeer(items[k]) do
        case Tipo of
          CTC_NReal: NReal(pVal^) := StrToFloat(xval);
          CTC_String: string(pVal^) := xval;
          CTC_ArchRef:
          begin
            ts := xval;
             {$IFDEF LINUX}
            toLinuxDir(ts);
             {$ENDIF}
            TArchiRef(pVal^) := TArchiRef.Create(ts);
          end;
          CTC_NInt: NEntero(pVal^) := StrToInt(xval);
          CTC_NCardinal: cardinal(pVal^) := StrToInt(xval);
          CTC_Boolean: boolean(pVal^) := (xval <> '') and (xval[1] = '1');
          CTC_DAOfNReal: parseDAOfNreal(TDAOfNReal(pVal^), xval);
          CTC_DAOfNInt: parseDAOfNInt(TDAOfNInt(pVal^), xval);
          CTC_DAOfBoolean: parseDAOfBoolean(TDAOfBoolean(pVal^), xval);
          CTC_DAOfString: parseDAOfString(TDAOfString(pVal^), xval);
          CTC_StringList: parseStringList(TStringList(pVal^), xval);
          CTC_Fecha: TFecha(pVal^) := TFecha.Create_str(xval);
          CTC_Referencia:
          begin
            nombreRef:='';
            clase:='';
            parsearReferencia(xval, clase, nombreRef);
            if (clase <> '?') then
              uCosaConNombre.registrar_referencia(referente,
                clase, nombreRef, pVal^)
            else
              TCosa(pVal^) := nil;
          end;
          CTC_TDataRowOfCosa:
          begin
            TDataRowOfCosa(pval^) := nil;
          end;
          CTC_Cosa:
          begin
            TCosa(pval^) := nil;
          end;
        end; // case
    except
      on E: Exception do
      begin
        raise ExcepcionResolverCampoValor.Create('ResolverCampoValor(' +
          xnombre + ',' + xval + ') ' + E.Message);
      end;
    end; // try
    // eliminamos la ficha de lectura pendiente pues fué resuelta
    TCampoParaLeer(items[k]).Free;
    Delete(k);
  end;
end;

// recorremos las fichas que hallan quedado sin resolver y les asignamos los
// valores por defecto (o tiramos excepciones si no se definió una función
// alternativa.
procedure TListCamposParaLeer.Ejecutar_vpds(klinea: integer);
var
  k: integer;
begin
  for k := 0 to Count - 1 do
  begin
    TCampoParaLeer(items[k]).Ejecutar_vpd(klinea);
    TCampoParaLeer(items[k]).Free;
  end;
  self.Clear;
end;

procedure TListCamposParaLeer.Ejecutar_vpd(klinea: integer; nombreCampo: string);
var
  k: integer;
begin
  for k := 0 to Count - 1 do
  begin
    if TCampoParaLeer(items[k]).nombre = nombreCampo then
    begin
      TCampoParaLeer(items[k]).Ejecutar_vpd(klinea);
      TCampoParaLeer(items[k]).Free;
      Delete(k);
      Pack;
      break;
    end;
  end;
end;

{$IFDEF DBG}
function TListCamposParaLeer.nombresCamposPorLeer: TDAofString;
var
  i: integer;
  res: TDAofString;
begin
  SetLength(res, self.Count);
  for i := 0 to Self.Count - 1 do
    res[i] := TCampoParaLeer(items[i]).nombre;
  Result := res;
end;

{$ENDIF}


//////////////////////////////////////////////////////////////////////////////
//TDataBaseConnection
//////////////////////////////////////////////////////////////////////////////
constructor TSalaDeJuegoConnection.Create(xFileName: WideString);
begin
  xFileName := Utf8ToAnsi(xFileName);
  chdir(extractFilePath(xFileName));
  self.FileName:=xFileName;
end;

constructor TSalaDeJuegoConnection.Create(xFileHandler: TArchiTexto);
begin
  self.FileHandler := xFileHandler;
end;

procedure TSalaDeJuegoConnection.ReadDataStoreFromTextFile();
var
  row: TDataRowOfCosa=nil;
  cosa: TCosa=nil;
  aux: TListaDeCosasConNombre=nil;
  a: Integer=0;
  FPHashList: TCosaHashList;
  sSourceList: TStringList;
  i: Integer;
begin
  row:=nil;

  self.FileHandler := TArchiTexto.CreateForRead(self.FileName, False);

  if self.FileHandler.Version >= 10113 then
  begin
    FPHashList:=TCosaHashList.Create('');
    sSourceList:=TStringList.Create();
    sSourceList.LoadFromFile(FileName);
    i:=-1;
    ProcessFile(sSourceList, i, FPHashList);
    //FPHashList.print(true);
    DATA_STORE.LoadFromCosaHashList(FPHashList);
    ucosa.PrintDataSet(DATA_STORE);
    sSourceList.Free;
    FPHashList.Free;
  end
  else
  begin
    // SE LEE DE LA FORMA VIEJA
    self.FileHandler.rd('sala', cosa);

    if uCosaConNombre.referenciasSinResolver > 0 then
    begin
      aux := TListaDeCosasConNombre.Create(0, 'aux');
      aux.Add(TSalaDeJuego(cosa).globs);
      uCosaConNombre.resolver_referencias(aux);
      uCosaConNombre.resolver_referencias(TSalaDeJuego(cosa).listaActores);
      uCosaConNombre.resolver_referencias(TSalaDeJuego(cosa).listaFuentes);
      uCosaConNombre.resolver_referencias(TSalaDeJuego(cosa).listaCombustibles);
      aux.FreeSinElemenentos;
      aux := nil;
    end;
    if uCosaConNombre.referenciasSinResolver > 0 then
    begin
      uCosaConNombre.DumpReferencias('errRefs.txt');
      raise Exception.Create(
        'TSalaDeJuego.cargarSala: Quedaron Referencias Sin Resolver Cargando la Sala. Puede Ver Que Referencias No Se Resolvieron En: '
        + 'errRefs.txt');
    end;
    //CREO EL DATA STORE A PARTIR DE LA INSTANCIA
    DATA_STORE.CreateDataRowFromInstance(cosa, 'dt_TSalaDeJuego', 'dr_Sala');
    TSalaDeJuego(cosa).Free;
  end;

  self.FileHandler.Free();


end;

function TSalaDeJuegoConnection.WriteDataStoreToTextFile(filename: string
  ): boolean;
var
  oldFileMode: Byte;
begin
  try
    if filename='' then
      filename:=self.FileName;

    self.FileHandler := TArchiTexto.CreateForWrite(filename, False, 0);
    self.wr('dt_TSalaDeJuego', 1001, '');
    self.FileHandler.Free;
    Result:=true;
  except
    On E: Exception do
    begin
      Result:=false;
      system.writeln('ERROR!!! '+
                 'Ha ocurrido un error mientras se intentaba guardar la sala.' + '\n\r' + 'TSalaDeJuegoConnection.WriteDataStoreToTextFile: ' + E.Message      );
      system.writeln('Presione ENTER para continuar ... ' );
      system.readln;
    end;
  end;
end;

procedure TSalaDeJuegoConnection.ProcessFile(SourceList: TStringList;
  var i: integer; var FPHashList: TCosaHashList);

var
Line  : String;
split: TStringList;
key: String;
value: String;
cfphl: TCosaHashList;
a: Integer;
b: String;
j: Integer;
item: TCosaHashListItem;

begin

  while i<SourceList.Count-1 do
  begin
    i:=i+1;

    Line:=Trim(SourceList[i]);

    if Copy(Line, 1, 2)='<-' then
    begin
      a:=1;
      Exit();
    end;

    split := TStringList.Create;
    ExtractStrings(['=',';'], [], PChar(Line), split);
    key   := LowerCase( Trim(split[0]) );
    value := Trim(split[1]);
    split.Free;

    if Copy(value, 1, 2)='<+' then
    begin
      value:=Copy(value, 3, Length(value)-3);
      cfphl:=TCosaHashList.Create(value);
      ProcessFile(SourceList, i, cfphl);
      item:=TCosaHashListItem.CreateHashListItem(cfphl);
      FPHashList.Add(key, item);

      if FPHashList.CosaStrId='TList' then
        Exit();

    end

    else
    begin

      item:=TCosaHashListItem.CreateValueItem(value);
      FPHashList.Add(key, item);

      if key='n' then
      begin
        cfphl:=TCosaHashList.Create('TList');
        for j:=0 to StrToInt(value)-1 do
        begin
          ProcessFile(SourceList, i, cfphl);
        end;
        item:=TCosaHashListItem.CreateHashListItem(cfphl);
        FPHashList.Add('lst', item);
      end;

    end;
  end;
end;

procedure TSalaDeJuegoConnection.writeline(const s: string);
begin
  self.FileHandler.writeline(s);
end;

procedure TSalaDeJuegoConnection.wr(DataTableName: string; DataRowId: integer;
  tabulation: string);
var
  j, z: integer;
  DataTable: TDataTableOfCosa;
  DataRow: TDataRowOfCosa;
  DataCell: TDataCellOfCosa;
  DataColumn: TDataColumnOfCosa;
  sval: string='';
  a: integer;
  split: TStrings;
  DataTableChildName: string;
  DataRowChildId: integer;
  DataColumnChildName: string;
  DataCellChildSVal: string;
  g: integer;
  daos: TDAofString;
  adt: TDateTime;
  ad: NReal;
begin

  DataTable := DATA_STORE.GetDataTableByName(DataTableName);
  DataRow := DataTable.GetDataRowById(DataRowId);

  {dfusco@20150420
   Si la fila fue marcada para ser eliminada, no se persiste en el archivo.}
  if DataRow.SIUD=TSIUD.TDELETE then
    Exit();

  if (DataRow.RowName[4] = ':') then
    writeline(tabulation+':'+'= '+'<+'+Copy(DataTable.TableName, 4, Length(
      DataTable.TableName) - 3)+'>')
  else
    writeline(tabulation+Copy(DataRow.RowName, 4, Length(DataRow.RowName) - 3)+
      '= '+ '<+'+ Copy(DataTable.TableName, 4, Length(DataTable.TableName) - 3)+ '>');

  for j := 0 to DataTable.DataColumnListCount - 1 do
  begin
    if (j < DataRow.DataCellListCount) then
    begin
      DataColumn := DataTable.DataColumnList[j];
      if (DataColumn.ColumnName='iteracion_flucar_modificar_rendimiento') then
        a:=1;
      DATA_STORE.ReadDataCellTextValue(DataTable.TableName, DataRow._RowId, DataColumn.ColumnName, sval);
      if (DataColumn.DataType is TDataTypeDataRowOfCosa) then
      begin
        if (sval <> '') then
        begin
          split := TStringList.Create;
          try
            ExtractStrings(['.'], [], PChar(sval), split);
            DataTableChildName := split[0];
            DataRowChildId := StrToInt(split[1]);
            self.wr(DataTableChildName, DataRowChildId, tabulation+'  ');
          finally
            split.Free;
          end;
        end
        else
        begin
          writeline(tabulation + '  '+ DataColumn.ColumnName+ '= '+ sval+';');
        end;
      end
      else if (DataColumn.DataType is TDataTypeListOfCosa) then
      begin
        try
          split := TStringList.Create;
          if (sval <> '') then
          begin
            ExtractStrings(['.'], [], PChar(sval), split);
            DataTableChildName := split[0];
            DataCellChildSVal := split[2];
            daos := DATA_STORE.ReadDataCellTextValues(DataTableChildName, 'idParent', DataCellChildSVal, 'idchild');
            a:=length(daos);
            for z := 0 to length(daos) - 1 do
            begin
              split := TStringList.Create;
              ExtractStrings(['.'], [], PChar(daos[z]), split);
              if (split.Count>0) then
              begin
                DataTableChildName := split[0];
                DataRowChildId := StrToInt(split[1]);
                self.wr(DataTableChildName, DataRowChildId, tabulation+'  ');
              end;
            end;
          end;
        finally
          split.Free;
        end;
      end
      else
      begin
        if DataColumn.DataType is TDataTypeDoubleOfCosa then
        begin
          //SE CONVIERTE EL REAL AL FORMATO LOCAL
          uauxiliares.setSeparadoresLocales();
          ad:=StrToFloat(sval);
          uauxiliares.setSeparadoresGlobales();
          sval:=FloatToStr(ad);
        end;
        if DataColumn.DataType is TDataTypeDateOfCosa then
        begin
          //SE CONVIERTE LA FECHA AL FORMATO LOCAL
          uauxiliares.setSeparadoresLocales();
          adt:=StrToDate(sval);
          uauxiliares.setSeparadoresGlobales();
          sval:=DateToStr(adt);
        end;
        writeline(tabulation + '  '+ DataColumn.ColumnName+ '= '+ sval+';');
      end;
    end;
  end;
  writeline(tabulation+ '<-'+ Copy(DataTable.TableName, 4, Length(DataTable.TableName) - 3)+ '>;');

end;


//////////////////////////////////////////////////////////////////////////////
//TDataStore
//////////////////////////////////////////////////////////////////////////////
constructor TDataStore.Create(xDataSetParent: TDataStore);
begin

  self._DataTableList := TList.Create;
  cosasConNombre := TListaDeCosas.Create(-1, '');
  self.DataSetParent:= xDataSetParent;

end;

function TDataStore.GetIndexOfDataTableByName(Name: string): integer;
var
  Index: integer;
  foundIt: boolean;
  Count: integer;
  a: string;
begin
  Result := -1;
  for Index := 0 to self._DataTableList.Count - 1 do
  begin
    if (LowerCase(self.DataTableList[Index].TableName) = LowerCase(Name)) then
    begin
      Result := Index;
      break;
    end;
  end;
end;

function TDataStore.GetDataTableList: TList;
begin
  Result := self._DataTableList;
end;

function TDataStore.Query(Select: string; FromName: string;
  var ds: TDataStore): boolean;
begin
  Result := self.Query(Select, FromName, '', '', ds);
end;

function TDataStore.Query(Select: string; FromName: string;
  ColumnNameToEval, Value: string; var ds: TDataStore): boolean;
var
  Alfa_DataTable: TDataTableOfCosa;
  Beta_DataTable: TDataTableOfCosa;
  Gamma_DataTable: TDataTableOfCosa;

begin

  if not Assigned(ds) then
  begin
    Exit;
    Result:=False;
  end;

  Alfa_DataTable := self.GetDataTableByName(FromName);
  if (Alfa_DataTable = nil) then
  begin
    Result := self.DataSetParent.Query(Select, FromName, ColumnNameToEval,
      Value, ds);
    exit;
  end;

  Beta_DataTable:=Alfa_DataTable.Query(Select, ColumnNameToEval, Value);
  //Verifico si ya existe la tabla FromName en el ds
  //Si no la obtiene la crea clonando la tabla alfa
  Gamma_DataTable := ds.GetDataTableByName(FromName);
  if Assigned(Gamma_DataTable) then
    Gamma_DataTable.AppendDataTable(Beta_DataTable,false)
  else
    ds.AddDataTable(Beta_DataTable);

  if Beta_DataTable.DataRowListCount > 0 then
    Result:=True
  else
  begin
    if Assigned(self.DataSetParent) then
      Result := self.DataSetParent.Query(Select, FromName, ColumnNameToEval,
      Value, ds)
    else
      Result:=False;
  end;
end;

function TDataStore.Query(Select: String; FromName: string;
  RowId: integer; var ds: TDataStore): boolean;
var
  Alfa_DataTable: TDataTableOfCosa;
  Beta_DataTable: TDataTableOfCosa;
  Gamma_DataTable: TDataTableOfCosa;

begin

  if not Assigned(ds) then
  begin
    Exit;
    Result:=False;
  end;

  Alfa_DataTable := self.GetDataTableByName(FromName);
  if (Alfa_DataTable = nil) then
  begin
    Result := self.DataSetParent.Query(Select, FromName, RowId, ds);
    exit;
  end;

  Beta_DataTable:=Alfa_DataTable.Query(Select, RowId);
  //Verifico si ya existe la tabla FromName en el ds
  //Si no la obtiene la crea clonando la tabla alfa
  Gamma_DataTable := ds.GetDataTableByName(FromName);
  if Assigned(Gamma_DataTable) then
    Gamma_DataTable.AppendDataTable(Beta_DataTable,false)
  else
    ds.AddDataTable(Beta_DataTable);

  if Beta_DataTable.DataRowListCount > 0 then
    Result:=True
  else
  begin
    if Assigned(self.DataSetParent) then
    Result := self.DataSetParent.Query(Select, FromName, RowId, ds)
    else
      Result:=False;
  end;

end;

function TDataStore.GetEmptyTable(TableName: string; CloneIdCounter: Boolean
  ): TDataTableOfCosa;
var
  dt: TDataTableOfCosa;
begin
  dt:=GetDataTableByName(TableName);
  if Assigned(dt) then
    Result:=dt.Clone(CloneIdCounter, False)
  else
    if Assigned(DataSetParent) then
      Result:=DataSetParent.GetEmptyTable(TableName, CloneIdCounter)
    else
      Result:=nil;
end;

procedure TDataStore.DeleteDataRows(FromName: string; ColumnNameToEval,
  Value: string);
var
  Alfa_DataTable: TDataTableOfCosa;
  Alfa_DataRow: TDataRowOfCosa;
  Alfa_DataColumn: TDataColumnOfCosa;
  Alfa_DataCell: TDataCellOfCosa;

  Beta_DataTable: TDataTableOfCosa;
  Beta_DataRow: TDataRowOfCosa;
  Beta_DataColumn: TDataColumnOfCosa;
  Beta_DataCell: TDataCellOfCosa;

  dcSelectIndex:TDAofNInt;

  i,ii,j,k:Integer;

  ColumnNames: TStringList;
  ColumnIndex: Integer;
begin

  Alfa_DataTable:=self.GetDataTableByName(FromName);
  ii:=Alfa_DataTable.GetIndexOfDataColumnByName(ColumnNameToEval);

  if ((ii>-1) or (ColumnNameToEval='')) then
  begin
    for k:=0 to Alfa_DataTable.DataRowListCount-1 do
      begin
        Alfa_DataRow:= Alfa_DataTable.GetDataRowByIndex(k);
        Alfa_DataCell:=Alfa_DataRow.ConditionalGetDataCell(0,ii,Value);
        if Alfa_DataCell<> nil then
          Alfa_DataRow.SIUD:=TSIUD.TDELETE;
      end;
  end;
end;

function TDataStore.PrepareInsert(Into, ColumnName: string; RowId: integer;
  Value: string; isItemOfList: boolean): TDataStore;
var
  ds: TDataStore;
  dr1, dr2: TDataRowOfCosa;
  dt: TDataTableOfCosa;
  dt_TList: TDataTableOfListOfCosa;

  sval: string='';
  IdParent: string;
  split: TStringList;
  n: integer;
  nsval: string='';

begin
  ds := TDataStore.Create(self);
  Query('*', Into, RowId, ds);
  dt := ds.DataTableList[0];
  dt.Visible := False;
  ds.AddDataTable(GetEmptyTable(Value, True));
  ds.DataTableList[1].Visible := True;
  dr1 := ds.DataTableList[1].GetDataRowWithDefaulValues();
  dr1.SIUD := TSIUD.TINSERT;
  ds.DataTableList[1].AddDataRow(dr1);
  if isItemOfList then
  begin
    ds.ReadDataCellTextValue(Into, RowId, 'n', nsval);
    n := StrToInt(nsval);
    ds.WriteDataCellTextValue(Into, RowId, 'n', IntToStr(n + 1));
    ds.AddDataTable(GetEmptyTable('dt_TList', True));
    dt_TList := TDataTableOfListOfCosa(ds.DataTableList[2]);
    dt_TList.Visible := False;
    dr2 := TDataRowOfCosa.Create;
    dt_TList.AddDataRow(dr2);
    dr2.SIUD := TSIUD.TINSERT;

    ds.ReadDataCellTextValue(Into, RowId, ColumnName, sval);
    if sval = '' then
    begin
      IdParent := IntToStr(dt_TList.GetNextIdParent());
      ds.WriteDataCellTextValue(Into, RowId, ColumnName,
        dt_TList.TableName + '.IdParent.' + IdParent);
    end
    else
    begin
      split := TStringList.Create;
      try
        ExtractStrings(['.'], [], PChar(sval), split);
        IdParent := split[2];
      finally
        split.Free;
      end;
    end;

    dr2.AddDataCell(TDataCellOfCosa.Create(IdParent));
    dr2.AddDataCell(TDataCellOfCosa.Create(Value + '.' + IntToStr(dr1._RowId)));

  end
  else
  begin
    ds.WriteDataCellTextValue(Into, RowId, ColumnName, Value + '.' + IntToStr(dr1._RowId));
  end;
  Result := ds;

end;

function TDataStore.PrepareUpdate(From, ColumnName: string; RowId: integer;
  Value: string; lst: boolean): TDataStore;
var
  ds: TDataStore;
  dr1, dr2: TDataRowOfCosa;
  dt: TDataTableOfCosa;

  sval: string;
  split: TStringList;
  DataTableChildName: string;
  DataRowChildId: integer;
  lst_sval: string='';
  DataTableListName: string;
  DataColumnName: string;
  IdParent: string;
  aDataTableItemName: string;
  aDataRowItemId: integer;
  j: integer;
  ADataRow: TDataRowOfCosa;
  ADataCell: TDataCellOfCosa;

begin
  ds := TDataStore.Create(self);
  split := TStringList.Create;
  try
    ExtractStrings(['.'], [], PChar(Value), split);
    DataTableChildName := split[0];
    DataRowChildId := StrToInt(split[1]);
  finally
    split.Free;
  end;

  Query('*', DataTableChildName, DataRowChildId, ds);
  ds.DataTableList[0].Visible := True;
  if lst then
  begin
    //Como es una lista busco los items
    ds.ReadDataCellTextValue(DataTableChildName, DataRowChildId, 'lst', lst_sval);

    if (lst_sval <> '') then
    begin
      split := TStringList.Create;
      try
        ExtractStrings(['.'], [], PChar(lst_sval), split);
        DataTableListName := split[0];
        DataColumnName := split[1];
        IdParent := split[2];
      finally
        split.Free;
      end;
      Query('*', DataTableListName, DataColumnName, IdParent, ds);
      dt := ds.DataTableList[1];

      for j := 0 to dt.DataRowListCount - 1 do
      begin
        ADataRow := dt.DataRowList[j];
        if ADataRow.SIUD <> TSIUD.TDELETE then
        begin
          ADataCell := aDataRow.DataCellList[1];
          split := TStringList.Create;
          try
            ExtractStrings(['.'], [], PChar(aDataCell.sval), split);
            aDataTableItemName := split[0];
            aDataRowItemId := StrToInt(split[1]);
          finally
            split.Free;
          end;
          Query('*', aDataTableItemName, aDataRowItemId, ds);
          ds.GetDataTableByName(aDataTableItemName).Visible := False;
        end;
      end;
    end;
  end;

  ds.DataTableList[0].DataRowList[0].SIUD := TSIUD.TUPDATE;

  Result := ds;
end;

function TDataStore.PrepareDelete(Parent_DataTableName: string;
                                  Parent_DataRowId: integer;
                                  Child_DataTableName: string;
                                  Child_DataRowId: integer): TDataStore;
var
  DataSet: TDataStore;


  n: integer;
  split: TStringList;

  lst_sval: string='';
  IdParent: string;
  nsval: string='';


begin
  DataSet := TDataStore.Create(self);

  Query('*', Parent_DataTableName, Parent_DataRowId, DataSet);
  DataSet.ReadDataCellTextValue(Parent_DataTableName, Parent_DataRowId, 'n', nsval);
  n:=StrToInt(nsval);
  nsval:=IntToStr(n-1);
  DataSet.WriteDataCellTextValue(Parent_DataTableName, Parent_DataRowId, 'n', nsval);
  if (n=0) then
  begin

  end;

  Query('*', Child_DataTableName, Child_DataRowId, DataSet);
  DataSet.DataTableList[1].DataRowList[0].SIUD:=TSIUD.TDELETE;

  {dfusco@20150417
   Nunca va a pasar que haya más de un registro con el mismo hijo...}
  Query('*', 'dt_TList', 'idChild', Child_DataTableName+'.'+IntToStr(Child_DataRowId), DataSet);
  DataSet.DataTableList[2].DataRowList[0].SIUD:=TSIUD.TDELETE;

  //DataSet.DeleteDataRows('dt_TList', 'IdChild', Value);

  Result:=DataSet;
end;

function TDataStore.Clone(CloneRowIdCounter: boolean;
  CloneRowList: boolean): TDataStore;
var
  Beta_DataSet: TDataStore;
  z: Integer;
  Alfa_DataTable: TDataTableOfCosa;
  Beta_DataTable: TDataTableOfCosa;
begin
  Beta_DataSet := TDataStore.Create(self.DataSetParent);

  for z := 0 to self.DataTableListCount - 1 do
  begin
    Alfa_DataTable := self.DataTableList[z];
    Beta_DataTable := Alfa_DataTable.Clone(CloneRowIdCounter, CloneRowList);
    Beta_DataSet.AddDataTable(Beta_DataTable);
  end;
  Result := Beta_DataSet;
end;

//function TDataStore.View(ColNames: string): TDataTableOfCosa;
//var
//  i,j: integer;
//  dt, dt_res: TDataTableOfCosa;
//  ColumnNames: TStringList;
//
//begin
//  dt_res:=TDataTableOfCosa.Create('dt_DataSetView');
//  ColumnNames := TStringList.Create;
//  ColumnNames.DelimitedText := ColNames;
//
//  for i:=0 to ColumnNames.Count-1 do
//    dt_res.AddDataColumn(TDataColumnOfCosa.Create(ColumnNames[i],nil));
//
//  for j := 0 to DataTableListCount - 1 do
//  begin
//    dt:=self.DataTableList[j];
//    if not (dt is TDataTableOfListOfCosa) then
//      dt_res.AppendDataTable(dt.View(ColNames), true);
//  end;
//
//  Result := dt_res;
//end;

procedure TDataStore.Merge(ds: TDataStore);
var
  TableListCount: Integer;
  dt1, dt2: TDataTableOfCosa;
  i: Integer;
begin
  if Assigned(ds) then
  begin
    TableListCount:=ds.DataTableListCount;
    for i:=0 to TableListCount-1 do
    begin
      dt2:=ds.DataTableList[i];
      dt1:=self.GetDataTableByName(dt2.TableName);
      if Assigned(dt1) then
        dt1.Merge(dt2)
      else
        AddDataTable(dt2.Clone());
    end;
  end;
end;

procedure TDataStore.Free;
var
  z: Integer;
begin
  for z:=0 to self._DataTableList.Count-1 do
    TDataTableOfCosa(self._DataTableList[z]).Free;
  self._DataTableList.Free;
end;

function TDataStore.Equals(ADataStore: TDataStore): boolean;
var
  Alfa_DataTable, Beta_DataTable: TDataTableOfCosa;
  z: Integer;
begin
  Result:=true;
  for z:= 0 to self.DataTableListCount-1 do
  begin
    Alfa_DataTable:=self.DataTableList[z];
    Beta_DataTable:=ADataStore.DataTableList[z];
    if (not(Alfa_DataTable.Equals(Beta_DataTable))) then
    begin
      Result:=false;
      Exit();
    end;
  end;
end;

function TDataStore.CompareTo(Beta_DataStore: TDataStore;
  DataTableName: string; DataRowId: integer; var Gamma_DataStore: TDataStore;
  BetaDataRowId: integer): boolean;
var
Alfa_DataTable: TDataTableOfCosa;
Beta_DataTable: TDataTableOfCosa;
Alfa_DataRow: TDataRowOfCosa;
z: Integer;
Beta_DataRow: TDataRowOfCosa;
i: Integer;
dri, adri, bdri: Integer;
dcv: string;
split: TStringList;
dtn, adtn, bdtn: String;
dcn: String;
ds: TDataStore;
dt: TDataTableOfCosa;
dco: TDataColumnOfCosa;
j: Integer;
dr: TDataRowOfCosa;
dce: TDataCellOfCosa;
dt_TList: TDataTableOfCosa;
Gamma_DataCell: TDataCellOfCosa;
Gamma_DataRow: TDataRowOfCosa;
Gamma_DataTable: TDataTableOfCosa;
Alfa_sval: string;
Beta_sval: string;
bb: String;
ba: String;
a: Integer;
puto: Boolean;
FoundEqualDataRow: Boolean;
ok:boolean;
begin
  if not Assigned(Gamma_DataStore) then
    Gamma_DataStore:=TDataStore.Create(nil);

  if DataTableName = 'dt_TFichaUnidades' then
    a:=1;

  Alfa_DataTable:=self.GetDataTableByName(DataTableName);
  Beta_DataTable:=Beta_DataStore.GetDataTableByName(DataTableName);

  Gamma_DataTable:=Gamma_DataStore.GetDataTableByName(DataTableName);
  if not Assigned(Gamma_DataTable) then
  begin
  Gamma_DataTable:=Alfa_DataTable.Clone(false, false);
    Gamma_DataStore.AddDataTable(Gamma_DataTable);
  end;

  Alfa_DataRow:=Alfa_DataTable.GetDataRowById(DataRowId);

  FoundEqualDataRow:=false;

  if Assigned(Alfa_DataTable) and Assigned(Alfa_DataRow) and Assigned(Beta_DataTable) then
begin
    Gamma_DataRow:=TDataRowOfCosa.Create(Alfa_DataRow._RowId);
    Gamma_DataRow.RowName:=Alfa_DataRow.RowName;
    for z:=0 to Beta_DataTable.DataRowListCount-1 do
    begin
      Beta_DataRow:=Beta_DataTable.DataRowList[z];

      if (Beta_DataRow._RowId=Alfa_DataRow._RowId) then
      (*
      if Alfa_DataTable.GetIndexOfDataColumnByName('nombre')>=0 then
      begin
        ba:=Alfa_DataRow.GetDataCellByIndex(Alfa_DataRow.DataTableOwner.GetIndexOfDataColumnByName('nombre')).sval;
        bb:=Beta_DataRow.GetDataCellByIndex(Beta_DataRow.DataTableOwner.GetIndexOfDataColumnByName('nombre')).sval;
      end
      else
      begin
        ba:='ba';
        bb:='bb';
      end;
      if ((BetaDataRowId<>-1) and (Beta_DataRow._RowId=BetaDataRowId)) or
         (CompareText(ba,bb)=0) then
      *)
        begin
          for i:=0 to Alfa_DataTable.DataColumnListCount-1 do
          begin
          ok:=true;
            Alfa_sval:=Alfa_DataRow.DataCellList[i].sval;
            Beta_sval:=Beta_DataRow.DataCellList[i].sval;
          if (Alfa_DataTable.DataColumnList[i].DataType is TDataTypeDataRowOfCosa) and (Alfa_sval<>'') then
            begin
              split:=TStringList.Create();
              ExtractStrings(['.'], [], PChar(Alfa_sval), split);
            adtn:=split[0];
            adri:=StrToInt(split[1]);
            split.Free();

            split:=TStringList.Create();
            ExtractStrings(['.'], [], PChar(Beta_sval), split);
            bdtn:=split[0];
            bdri:=StrToInt(split[1]);
            split.Free();

            if (CompareText(adtn, bdtn)=0) then
            begin
              if not self.CompareTo(Beta_DataStore, adtn, adri, Gamma_DataStore, bdri) then
                ok:=false;
            end
            else
              a:=1; //ACA SE TIENE QUE PUDRIR TODO
          end
          else if (Alfa_DataTable.DataColumnList[i].DataType is TDataTypeListOfCosa) and (Alfa_sval<>'') then
            begin
              split:=TStringList.Create();
              ExtractStrings(['.'], [], PChar(Alfa_sval), split);
              dtn:=split[0];
              dcn:=split[1];
              dcv:=split[2];

              ds:=TDataStore.Create(nil);
              self.Query('*', dtn, dcn, dcv, ds);
              if (ds<>nil) and (ds.DataTableListCount=1) then
              begin
                dt:=ds.DataTableList[0];
              a:=dt.DataRowListCount;
                dco:=dt.DataColumnList[1];
                for j:=0 to dt.DataRowListCount-1 do
                begin
                  dr:=dt.DataRowList[j];
                  dce:=dr.DataCellList[1];
                  split:=TStringList.Create();
                  ExtractStrings(['.'], [], PChar(dce.sval), split);
                  dtn:=split[0];
                  dri:=StrToInt(split[1]);

                    dt_TList:=Gamma_DataStore.GetDataTableByName('dt_TList');
                    if not Assigned(dt_TList) then
                    begin
                  dt_TList:=self.GetDataTableByName('dt_TList').Clone(False,False);
                      Gamma_DataStore.AddDataTable(dt_TList);
                    end;
                    dr:=TDataRowOfCosa.Create();
                    dr.AddDataCell(TDataCellOfCosa.Create(dcv));
                    dr.AddDataCell(TDataCellOfCosa.Create(dtn+'.'+IntToStr(dri)));
                    dt_TList.AddDataRow(dr);

                if not self.CompareTo(Beta_DataStore, dtn, dri, Gamma_DataStore) then
                    ok:=false;

                end;
              end
            end
            else
              if CompareText(Alfa_sval, Beta_sval)<>0 then
                ok:=false;

          Gamma_DataCell:=TDataCellOfCosa.Create(Alfa_sval);
          Gamma_DataCell.Changed:=not ok;
            Gamma_DataRow.AddDataCell(Gamma_DataCell);
          end;
        FoundEqualDataRow:=true;
        break;
      end
      (*
      else
      begin
        if Alfa_DataRow.Equals(Beta_DataRow) then
        begin
          for j:=0 to Alfa_DataRow.DataCellListCount-1 do
        begin
            Gamma_DataCell:=Alfa_DataRow.DataCellList[j].Clone();
            Gamma_DataCell.Changed:=false;
            Gamma_DataRow.AddDataCell(Gamma_DataCell);
        end;
          FoundEqualDataRow:=true;
          break;
        end
      end;
      *)
    end;
    if not FoundEqualDataRow then
    begin
      for j:=0 to Alfa_DataRow.DataCellListCount-1 do
      begin
        Gamma_DataCell:=Alfa_DataRow.DataCellList[j].Clone();
        Gamma_DataCell.Changed:=true;
        Gamma_DataRow.AddDataCell(Gamma_DataCell);
end;
end;
    Gamma_DataTable.AddDataRow(Gamma_DataRow, false);
  end;
  BetaDataRowId:=-1;
  Result:=FoundEqualDataRow;

end;

function TDataStore.Fill(xDataTableName: string; xDataRowId: integer;
  var ADataSet: TDataStore): Boolean;
var
  ds:TDataStore;
  DataTable: TDataTableOfCosa;
  i: Integer;
  DataColumn: TDataColumnOfCosa;
  DataCell: TDataCellOfCosa;
  split: TStringList;
  DataTableChildName: String;
  DataRowChildId: Integer;
  DataColumnName: String;
  DataCellSVal: String;
  ds_TList:TDataStore;
  j: Integer;
  a: Integer;
  DataRow: TDataRowOfCosa;
  b: String;
begin

  Result:=True;
  // DataSet auxiliar para realizar las consultas.
  ds:=TDataStore.Create(nil);
  if self.Query('*', xDataTableName, xDataRowId, ds) then
  begin

    //if assigned(ADataSet.GetDataTableByName('dt_TFuenteConstante')) then
    //  b:=ADataSet.GetDataTableByName('dt_TFuenteConstante').DataRowList[0].DataCellList[
    //    ADataSet.GetDataTableByName('dt_TFuenteConstante').GetIndexOfDataColumnByName('nombresdebornes_publicados')
    //  ].sval;
    //
    //WriteLn();
    //WriteLn('>>> DEBUG: TDataStore.Fill('+xDataTableName+', '+IntToStr(xDataRowId)+')');

    ADataSet.Merge(ds);

    //if assigned(self.GetDataTableByName('dt_TFuenteConstante')) then
    //  b:=self.GetDataTableByName('dt_TFuenteConstante').DataRowList[0].DataCellList[
    //    self.GetDataTableByName('dt_TFuenteConstante').GetIndexOfDataColumnByName('nombresdebornes_publicados')
    //  ].sval;
    //
    DataTable:=ADataSet.GetDataTableByName(xDataTableName);
    for i:=0 to DataTable.DataColumnListCount-1 do
    begin
      DataColumn:=DataTable.DataColumnList[i];
      DataRow:=DataTable.GetDataRowById(xDataRowId);
      DataCell:=DataRow.DataCellList[i];
      //Writeln(DataColumn.ColumnName + ' = ' + DataCell.sval);
      if DataCell.sval<>'' then
      begin
        if DataColumn.DataType is TDataTypeDataRowOfCosa then
        begin
          parseRowReference(DataCell.sval, DataTableChildName, DataRowChildId);
          self.Fill(DataTableChildName, DataRowChildId, ADataSet);
        end
        else if DataColumn.DataType is TDataTypeListOfCosa then
        begin
          //Writeln('DataColumn.ColumnName = '+DataColumn.ColumnName + ' / ' + 'DataCell.sval = ' + DataCell.sval);
          parseListReference(DataCell.sval, DataTableChildName, DataColumnName, DataCellSVal);
          ds_TList:=TDataStore.Create(nil);
          self.Query('*', DataTableChildName, DataColumnName, DataCellSVal, ds_TList);
          ADataSet.Merge(ds_TList);
          for j:=0 to ds_TList.DataTableList[0].DataRowListCount-1 do
          begin
            parseRowReference(ds_TList.DataTableList[0].DataRowList[j].DataCellList[1].sval,
              DataTableChildName, DataRowChildId);
            self.Fill(DataTableChildName, DataRowChildId, ADataSet);
          end;
          ds_TList.Free;
        end;
      end;
    end;
    //WriteLn('<<<');
  end
  else
    Result:=False;

  ds.Free;

end;

procedure TDataStore.GetDataTableOnLastVersion(DataTableName: String;
  var DataTable: TDataTableOfCosa);
var
  dt1, dt2: TDataTableOfCosa;
begin

  dt1:=GetDataTableByName(DataTableName);
  if Assigned(dt1) then
  begin
    dt2:=dt1.Clone();
    if Assigned(DataTable) then
    begin
      dt2.Merge(DataTable);
    end;
    DataTable:=dt2;
  end;

  if Assigned(self.DataSetParent) then
    self.DataSetParent.GetDataTableOnLastVersion(DataTableName, DataTable);

end;

procedure parseRowReference(sval: string; var DataTableName: string;
  var RowId: Integer);
var
  split: TStringList;
begin
  split:=TStringList.Create();
  ExtractStrings(['.'], [], PChar(sval), split);
  DataTableName:=split[0];
  RowId:=StrToInt(split[1]);
  split.Free();
end;

function parseListReference(sval: string; var DataTableName: string;
  var DataColumnName: string; var IdParent:string): Boolean;
var
  split: TStringList;
begin
  split:=TStringList.Create();
  ExtractStrings(['.'], [], PChar(sval), split);
  DataTableName:=split[0];
  DataColumnName:=split[1];
  IdParent:=split[2];
  split.Free();

end;

function TDataStore.View(VirtualTable: TDataVTableOfCosa): TDataTableOfCosa;
var
  i: Integer;

  vdrl: TList;
  nRows:Integer;

  dt_res: TDataTableOfCosa;
begin

  vdrl:= VirtualTable.GetDataRowList(self);
  nRows:=vdrl.Count;
  dt_res:=TDataTableOfCosa.Create('view', VirtualTable.GetDataColumnList().Clone());

  for i:=0 to nRows-1 do
    dt_res.AddDataRow(TDataVRowOfCosa(vdrl.Items[i]));

  Result:= dt_res;

end;

procedure TDataStore.Clear;
var
  z: Integer;
begin
  for z:=0 to self.DataTableListCount-1 do
  begin
    self.DataTableList[z].Clear();
  end;
end;

function TDataStore.GetDataTableByIndex(Index: integer): TDataTableOfCosa;
begin
  Result := self._DataTableList[Index];
end;

function TDataStore.GetDataTableByName(DataTableName: string; climb:boolean): TDataTableOfCosa;
var
  Index: integer;
begin
  Result:=nil;
  Index:=GetIndexOfDataTableByName(DataTableName);
  if Index>-1 then
    Result:=DataTableList[Index]
  else
  begin
    if Assigned(self.DataSetParent) and climb then
      Result:=self.DataSetParent.GetDataTableByName(DataTableName, climb);
  end;
end;

function TDataStore.GetDataTableListCount: integer;
var
  count: LongInt;
begin
  Result := self._DataTableList.Count;
end;

procedure TDataStore.LoadFromCosaHashList(cosaHashList: TCosaHashList);
var
  item: TCosaHashListItem;
  version: Integer;

  function HashToDataRow(CosaHashList: TCosaHashList; version:integer): TDataRowOfCosa;
  var
    DataTableName: String;

    ClaseDeCosa: TClaseDeCosa;

    DataTable_NewestVersion, DataTable_FileVersion: TDataTableOfCosa;
    DataRow_NewestVersion, DataRow_FileVersion, dr_TList, ADataRow: TDataRowOfCosa;

    i, j: Integer;
    item: TCosaHashListItem;

    dt_TList: TDataTableOfListOfCosa;
    listItem: TCosaHashList;
    LastIdParent: Integer;
    sval: String;
    DataColumn: TDataColumnOfCosa;
    ad: NReal;
    adt: TDateTime;
    a: Integer;
  begin

    DataTableName := 'dt_'+CosaHashList.CosaStrId;

    DataTable_NewestVersion:=DATA_STORE.GetDataTableByName(DataTableName);
    if not Assigned(DataTable_NewestVersion) then
      raise Exception.Create('TDataStore.AddTablesFromHashList: No se encuenta la tabla '+DataTableName+' en el DATA_STORE');

    ClaseDeCosa:=TClaseDeCosa(getClaseOf(CosaHashList.CosaStrId));
    DataTable_FileVersion:=TDataTableOfCosa.Create(DataTableName, ClaseDeCosa, version);
    DataRow_FileVersion:=TDataRowOfCosa.Create();

    for i:=0 to DataTable_FileVersion.GetDataColumnList().Count-1 do
    begin

      DataColumn:=DataTable_FileVersion.GetDataColumnList()[i];

      if DataColumn.ColumnName='fuentecota' then
        a:=1;

      if not CosaHashList.Find(DataColumn.ColumnName, item) then
        raise Exception.Create('No se encontro el campo '+DataTable_FileVersion.GetDataColumnList()[i].ColumnName+' en '+CosaHashList.CosaStrId);

      if item.isString then
      begin

        sval:=item.sval;

        if DataColumn.DataType is TDataTypeDoubleOfCosa then
        begin
          ad:=StrToFloat(sval);
          uauxiliares.setSeparadoresLocales();
          sval:=FloatToStr(ad);
          uauxiliares.setSeparadoresGlobales();
        end;

        if DataColumn.DataType is TDataTypeDateOfCosa then
        begin
          adt:=StrToDate(sval);
          uauxiliares.setSeparadoresLocales();
          sval:=DateToStr(adt);
          uauxiliares.setSeparadoresGlobales();
        end;

      end

      else
      begin
        if item.CosaHashList.CosaStrId='TList' then
        begin
          dt_TList:=TDataTableOfListOfCosa(DATA_STORE.GetDataTableByName('dt_TList'));
          LastIdParent:=dt_TList.GetNextIdParent();
          for j:=0 to item.CosaHashList.Count-1 do
          begin
            ADataRow:=HashToDataRow(item.CosaHashList[j].CosaHashList, version);
            dr_TList:=TDataRowOfCosa.Create();
            dr_TList.AddDataCell(TDataCellOfCosa.Create(IntToStr(LastIdParent)));
            dr_TList.AddDataCell(TDataCellOfCosa.Create('dt_'+item.CosaHashList[j].CosaHashList.CosaStrId+'.'+IntToStr(ADataRow.RowId)));
            dt_TList.AddDataRow(dr_TList);
          end;
          sval:=dt_TList.TableName+'.IdParent.'+IntToStr(LastIdParent);
        end

        else
        begin
          ADataRow:=HashToDataRow(item.CosaHashList, version);
          sval:='dt_'+item.CosaHashList.CosaStrId+'.'+IntToStr(ADataRow.RowId);
        end;
      end;

      DataRow_FileVersion.AddDataCell(TDataCellOfCosa.Create(sval));

    end;

    DataRow_FileVersion.SIUD:=TSELECT;
    DataTable_FileVersion.AddDataRow(DataRow_FileVersion, true);

    DataRow_NewestVersion:=TDataRowOfCosa.Create();
    DataRow_NewestVersion.SIUD:=DataRow_FileVersion.SIUD;
    DataTable_NewestVersion.AddDataRow(DataRow_NewestVersion);

    DataTable_NewestVersion.ConvertDataRowToNewestVersion(DataRow_NewestVersion, DataTable_FileVersion);

    Result:=DataRow_NewestVersion;

  end;

begin

  if not cosaHashList.Find('version', item) then
    raise Exception.Create('TDataStore.LoadFromCosaHashList: No se encontro el campo version en el archivo.');
  version:=StrToInt(item.sval);

  if not cosaHashList.Find('sala', item) then
    raise Exception.Create('TDataStore.LoadFromCosaHashList: No se encontro el campo sala en el archivo.');
  HashToDataRow(item.cosaHashList, version);

end;


function TDataStore.ReadDataCellTextValue(TableName: string; RowId: integer;
  ColumnName: string; var sval: string; climb: boolean): boolean;
var
  DataTable: TDataTableOfCosa;
  DataRow: TDataRowOfCosa;
  DataCell: TDataCellOfCosa;
begin
  Result:=false;
  DataTable := self.GetDataTableByName(TableName);
  if (not assigned(DataTable)) and climb then
  begin
    Result:=self.DataSetParent.ReadDataCellTextValue(TableName,
      RowId, ColumnName, sval, climb);
    exit;
  end;
  if Assigned(DataTable) then
    Result:=DataTable.ReadDataCellTextValue(RowId, ColumnName, sval);
end;


function TDataStore.ReadDataCellTextValues(TableName, ColumnNameToQuery,
  CellSValToQuery, ColumnNameToGetSVal: string; climb:boolean): TDAofString;
var
  h, i, j, j1, j2, count: integer;
  DataTable: TDataTableOfCosa;
  DataRow: TDataRowOfCosa;
  DataCell1, DataCell2: TDataCellOfCosa;
  ssvals: string='';
  daossval: array of string;
begin
  SetLength(daossval,0);
  Count := 0;
  DataTable := self.GetDataTableByName(TableName);
  if not Assigned(DataTable) then
  begin
    if Assigned(self.DataSetParent) and climb then
    begin
      Result:=self.DataSetParent.ReadDataCellTextValues(TableName, ColumnNameToQuery, CellSValToQuery, ColumnNameToGetSVal, climb);
      Exit();
    end
    else
      raise Exception.Create('TDataStore.ReadDataCellTextValues: No se encontro la tabla "'+TableName+'" en el DATA SET.');
  end;

  j1 := DataTable.GetIndexOfDataColumnByName(ColumnNameToQuery);
  j2 := DataTable.GetIndexOfDataColumnByName(ColumnNameToGetSVal);
  for i := 0 to DataTable.DataRowListCount - 1 do
  begin
    DataRow := DataTable.DataRowList[i];
    DataCell1 := DataRow.DataCellList[j1];
    if (DataCell1.sval = CellSValToQuery) then
    begin
      DataCell2 := DataRow.DataCellList[j2];
      ssvals := ssvals + ',' + DataCell2.sval;
      Count := Count + 1;
    end;
  end;

  if count=0 then
    Exit();

  ssvals := Copy(ssvals, 2, Length(ssvals) - 1);
  ssvals := '[' + IntToStr(Count) + '| ' + ssvals + ']';
  parseDAOfString(daossval, ssvals);
  Result := daossval;
end;

function TDataStore.ReadRowIdsTextValues(TableName, ColumnNameToQuery,
  CellSValToQuery: string): TDAofNInt;
var
  i, j: integer;
  DataTable: TDataTableOfCosa;
  DataRow: TDataRowOfCosa;
  DataCell: TDataCellOfCosa;
  ssvals: string;
  daossval: array of string;
  rowIds: TDAofNInt;
  count: integer;
begin
  Count := 0;
  DataTable := self.GetDataTableByName(TableName);
  SetLength(rowIds, DataTable.GetDataRowListCount());
  j := DataTable.GetIndexOfDataColumnByName(ColumnNameToQuery);
  for i := 0 to DataTable.DataRowListCount - 1 do
  begin
    DataRow := DataTable.DataRowList[i];
    DataCell := DataRow.DataCellList[j];
    if (DataCell.sval = CellSValToQuery) then
    begin
      rowIds[Count] := DataRow._RowId;
      Count := Count + 1;
    end;
  end;
  SetLength(rowIds, count);
  Result := rowIds;
end;

function TDataStore.WriteDataCellTextValue(TableName:string; RowId:Integer;
  AttributeName: string; Value: string): boolean;
var
  dt: TDataTableOfCosa;
  dr: TDataRowOfCosa;
  i: Integer;
  dcll: TDataCellOfCosa;
begin
  Result := True;
  dt:=GetDataTableByName(TableName);
  dr:=dt.GetDataRowById(RowId);
  dr.SIUD:=TSIUD.TUPDATE;
  i:= dt.GetIndexOfDataColumnByName(AttributeName);
  if Assigned(dr) and (i>-1) then
  begin
    dcll:=dr.GetDataCellByIndex(i);
    dcll.sval:=Value;
  end
  else
    Result:=False;
end;

procedure TDataStore.CreateClassInstance(TableName: string; RowId: integer;
  var cosa: TCosa);
var
  i: integer;
  DataTable: TDataTableOfCosa;
  DataColumn: TDataColumnOfCosa;
  sval: string='';
  pVariable: pointer;
  ColumnList: TDataColumnListOfCosa;
  ClassOwner: TClaseDeCosa;
  a: Integer;
begin

  if TableName='dt_TFichaHidroConEmbalse' then
    a:=1;

  DataTable := self.GetDataTableByName(TableName);
  ColumnList := DataTable.GetDataColumnList;
  ClassOwner := ColumnList.ClassOwner;
  cosa:=ClassOwner.Create;
  for i := 0 to DataTable.DataColumnListCount - 1 do
  begin
    DataColumn := DataTable.DataColumnList[i];

    if LowerCase(DataColumn.ColumnName)=LowerCase('flg_controlCotaObjetivoInferior_opt') then
      a:=1;

    pVariable := Pbyte(cosa) + DataColumn.Offset;

    self.ReadDataCellTextValue(DataTable.TableName, RowId, DataColumn.ColumnName, sval);
    writeln(DataTable.TableName,' ',DataColumn.ColumnName,' ',sval);

    if (sval <> '') then
      DataColumn.DataType.Eval(sval, pVariable)
    else
      DataColumn.DataType.Eval(DataColumn.ValorPorDefecto, pVariable);
  end;

  cosa.id:=RowId;
  cosa.AfterInstantiation();
end;

function TDataStore.CreateDataRowFromInstance(cosa: TCosa;
  DataTableName: string; RowName: string; climb: boolean): integer;
var
  DataTable: TDataTableOfCosa;
  h: Integer;
  DataRow: TDataRowOfCosa;
  DataColumn: TDataColumnOfCosa;
  sval: string;
  i, j: integer;
  alfa_dt_TList, beta_dt_TList: TDataTableOfListOfCosa;
  LastIdParent: Integer;

  pCosa: PLongWord;
  DataRowOfList: TDataRowOfCosa;
  Item: TCosa;
  DataTableItem: TDataTableOfCosa;
  DataTableItemName: String;
  rowIds: TDAofNInt;
  rowIdChild: integer;
  cosaRef: PCosaConNombre;
  DataTableNameChild: String;
  a: Integer;

  Beta_ds_TList: TDataStore;

  Alfa_DataRow: TDataRowOfCosa;
  split: TStringList;
  idparent: String;
  found: Boolean;
  l: Integer;
  dr: TDataRowOfCosa;

begin

  if cosa.apodo='Cero' then
     a:=TFuenteConstante(cosa).lpd.items[0].id;

  DataTable:=self.GetDataTableByName(DataTableName, true);
  if not Assigned(DataTable) then
    raise Exception.Create('TDataStore.CreateDataRowFromInstance: No se encontro la tabla "'+DataTableName+'" en el DATA SET.');

  // SE CREA LA FILA DE LA TABLA
  DataRow:=TDataRowOfCosa.Create(DataTable, RowName);

  for i := 0 to DataTable.DataColumnListCount - 1 do
  begin
    sval := '';
    rowIdChild := -1;
    DataColumn := DataTable.DataColumnList[i];

    if (DataColumn.ColumnName='nombresdebornes_publicados') then
      a:=1;

    //Funca para OS de 64 bits ??
    pCosa := PLongWord(cosa);
    pCosa := PLongWord(PByte(cosa) + DataColumn.Offset);
    pCosa := PLongWord(pCosa^);

    if DataColumn.DataType is TDataTypeDataRowOfCosa then
    begin
      if Assigned(pCosa) then
      begin
        //20150223: PARCHE PORQUE TODAVIA NO FUERON MIGRADAS TODAS LAS COSAS
        if Assigned(DataColumn.DataType.ClaseDeCosa) then
          DataTableNameChild:='dt_' + DataColumn.DataType.ClaseDeCosa.ClassName
        else
          DataTableNameChild:='dt_' + TCosa(pCosa).ClassName;

        rowIdChild := CreateDataRowFromInstance(TCosa(pCosa), DataTableNameChild,
          'dr_' + DataColumn.ColumnName, climb);

        sval:=DataTableNameChild + '.' + IntToStr(rowIdChild)
      end
      else
        sval:='';
    end

    else if DataColumn.DataType is TDataTypeReferenciaOfCosa then
    begin
      if Assigned(pCosa) then
        sval:=referenciaACosa(TCosa(pCosa))
      else
        sval:='<?.?>';
    end

    else if DataColumn.DataType is TDataTypeListOfCosa then
    begin
      if Assigned(pCosa) and (TList(pCosa).Count>0) then
      begin

        sval:=DataTable.GetDataRowById(cosa.id).DataCellList[DataColumn.Index].sval;

        Beta_ds_TList:=TDataStore.Create(self);

        split:=TStringList.Create;
        ExtractStrings(['.'], [], PChar(sval), split);
        idparent:=split[2];
        self.Query('*', 'dt_TList', 'idParent', idparent, Beta_ds_TList);
        beta_dt_TList:=TDataTableOfListOfCosa(Beta_ds_TList.DataTableList[0]);

        //VOY A PONER TODAS LAS FILAS EN SIUD DELETE, LAS QUE A FINAL CONTINUEN
        //EN ESTE ESTADO ES PORQUE FUERON ELIMINADAS (EN EL OBJETO).
        for j:=0 to beta_dt_TList.DataRowListCount-1 do
          beta_dt_TList.DataRowList[j].SIUD:=TSIUD.TDELETE;

        for j:=0 to TList(pcosa).Count-1 do
        begin
          Item:=TCosa(TList(pcosa).items[j]);

          //CREO EL REGISTRO EN LA TABLA CORRESPONDIENTE:
          RowIdChild:=CreateDataRowFromInstance(Item, 'dt_' + Item.ClassName, 'dr_:' + IntToStr(j+1), climb);

          found:=false;
          for l:=0 to beta_dt_TList.DataRowListCount-1 do
          begin
            if LowerCase(beta_dt_TList.DataRowList[l].DataCellList[1].sval)=LowerCase('dt_'+Item.ClassName+'.'+IntToStr(Item.id)) then
            begin
              beta_dt_TList.DataRowList[l].SIUD:=TSIUD.TUPDATE;
              found:=true;
              break;
            end;
          end;

          if not found then
          begin
            //CREO EL REGISTRO EN LA TABLA DE LISTAS:
            dr:=TDataRowOfCosa.Create(beta_dt_TList, '');
            dr.SIUD:=TSIUD.TINSERT;
            dr.AddDataCell(TDataCellOfCosa.Create(idparent));
            dr.AddDataCell(TDataCellOfCosa.Create('dt_'+Item.ClassName+'.'+IntToStr(RowIdChild)));
            beta_dt_TList.AddDataRow(dr);
          end;
        end;

        //Ya que la QUERY devuelve una clonación hago el MARGE con la tabla POSTA.
        alfa_dt_TList:=TDataTableOfListOfCosa(self.GetDataTableByName('dt_TList', true));
        alfa_dt_TList.Merge(beta_dt_TList);
      end
      else
      begin
        sval:='';
      end;
    end
    else if DataColumn.DataType is TDataTypeArchiRefOfCosa then
    begin
      if Assigned(pCosa) then
        sval := TArchiRef(pCosa).get_archi
      else
        sval:='';
    end
    else
      sval := DataColumn.DataType.ToString(Pbyte(cosa)+DataColumn.Offset);

    DataRow.AddDataCell(TDataCellOfCosa.Create(sval));
  end;

  //SE HACE MARGE ENTRE LA FILA DE LA BASE DEDATOS Y LA FILA CONVERTIDA...
  if cosa.id>0 then
  begin
    DataRow._RowId:=cosa.id;
    DataRow.SIUD:=TSIUD.TUPDATE;
    DataTable.GetDataRowById(cosa.id).Merge(DataRow);
  end
  else
  begin
    DataRow.SIUD:=TSIUD.TINSERT;
    DataTable.AddDataRow(DataRow);
  end;

  Result:=DataRow._RowId;

end;

function TDataStore.AddDataTable(xDataTable: TDataTableOfCosa): integer;
begin
  self._DataTableList.Add(xDataTable);
  xDataTable.DataStoreOwner:=self;
end;


//////////////////////////////////////////////////////////////////////////////
//TDataTableOfCosa
//////////////////////////////////////////////////////////////////////////////
constructor TDataTableOfCosa.Create(xTableName:string; claseDeCosa:TClaseDeCosa; xVersion: Integer = -2);
var
  DataRow: TDataRowOfCosa;
begin
  Visible:=True;
  self._DATAROWID_COUNTER := 1000;
  self.TableName := xTableName;
  if claseDeCosa <> nil then
  begin
    self._DataColumnList := claseDeCosa.CreateDataColumnList(claseDeCosa, xVersion);
    self._DataColumnList.DataTableOwner:=self;
    self._DataConversionList := claseDeCosa.CreateDataConversionList();
    self._DataRowList := TList.Create();
  end;
end;

constructor TDataTableOfCosa.Create(xTableName:string; xDataColumnList:TDataColumnListOfCosa);
var
  DataRow: TDataRowOfCosa;
  i: Integer;
begin
  Visible:=True;
  self._DATAROWID_COUNTER := 1000;
  self.TableName := xTableName;
  self._DataColumnList := xDataColumnList;
  self._DataConversionList := TListDataConversionOfCosa.Create();
  self._DataColumnList.DataTableOwner:=self;
  self._DataRowList := TList.Create();
end;

constructor TDataTableOfCosa.Create(xTableName: string;
  xDataColumnList: TDataColumnListOfCosa; xDataRowList: TList);
begin
  self.Visible:=True;
  self.TableName := xTableName;
  self._DataColumnList := xDataColumnList;
  self._DataRowList:=xDataRowList;
end;

constructor TDataTableOfCosa.Create(xTableName: string);
var
  DataRow: TDataRowOfCosa;
begin
  Visible:=True;
  self._DATAROWID_COUNTER := 1000;
  self.TableName := xTableName;
  self._DataColumnList := TDataColumnListOfCosa.Create();
  self._DataColumnList.DataTableOwner:=self;
  self._DataConversionList := TListDataConversionOfCosa.Create();
  self._DataRowList := TList.Create();
end;

function TDataTableOfCosa.ReadDataCellTextValue(ColumnName: string;
  var sval: string): boolean;
begin
  Result:=self.ReadDataCellTextValue(1001,ColumnName,sval);
end;

function TDataTableOfCosa.GetDataRowById(Id: integer): TDataRowOfCosa;
var
  i: Integer;
begin
  Result:= nil;
  for i:=0 to self._DataRowList.Count-1 do
  if DataRowList[i]._RowId=Id then
  begin
    Result:=DataRowList[i];
    break;
  end;
end;

function TDataTableOfCosa.GetIndexOfDataRowById(Id: integer): integer;
var
  Index: integer;
begin
  Result:=-1;
  for Index:=0 to self._DataRowList.Count-1 do
    if (self.GetDataRowById(Index)._RowId = Id) then
    begin
      Result:=index;
      Break;
    end;
end;


function TDataTableOfCosa.GetDataRowList: TList;
begin
  Result := self._DataRowList;
end;

procedure TDataTableOfCosa.SetDataRowList(drl: TList);
begin
  self._DataRowList := drl;
end;

function TDataTableOfCosa.GetDataRowListCount: integer;
begin
  Result := self._DataRowList.Count;
end;

function TDataTableOfCosa.GetDataConversionByIndex(Index: integer):
TDataConversionOfCosa;
begin

end;

function TDataTableOfCosa.GetDataRowByIndex(Index: integer): TDataRowOfCosa;
begin
  Result := self._DataRowList[Index];
end;

function TDataTableOfCosa.GetDataRowIdCounter: integer;
begin
  Result := self._DATAROWID_COUNTER;
end;

procedure TDataTableOfCosa.SetDataRowIdCounter(AValue: integer);
begin
  self._DATAROWID_COUNTER := AValue;
end;

function TDataTableOfCosa.GetDataColumnByIndex(Index: integer): TDataColumnOfCosa;
begin
  Result := self._DataColumnList.Items[Index];
end;

function TDataTableOfCosa.AddDataColumn(xDataColumn: TDataColumnOfCosa
  ): integer;
begin
  if xDataColumn <> nil then
  begin
    Result:=self._DataColumnList.Add(xDataColumn);
    xDataColumn.DataColumnListOwner:=self._DataColumnList;
    xDataColumn.Index:=Result;
  end
  else
    Result := -1;
end;

procedure TDataTableOfCosa.SetDataColumnList(dcl: TDataColumnListOfCosa);
var
  i: Integer;
begin
  self._DataColumnList := dcl;
  dcl.DataTableOwner:=self;
end;

function TDataTableOfCosa.GetDataColumnByName(Name: string): TDataColumnOfCosa;
var
  Index: integer;
  DataColumn: TDataColumnOfCosa;
begin
  Result := nil;
  for Index := 0 to self._DataColumnList.Count - 1 do
  begin
    DataColumn := self.GetDataColumnByIndex(Index);
    if (LowerCase(DataColumn.ColumnName) = LowerCase(Name)) then
    begin
      Result := DataColumn;
      Break;
    end;
  end;
end;

function TDataTableOfCosa.GetIndexOfDataColumnByName(Name: string): integer;
var
  Index: integer;
  b: String;
begin
  Result := -1;
  for Index := 0 to self._DataColumnList.Count - 1 do
  begin
    b:=LowerCase(self.GetDataColumnByIndex(Index).ColumnName);
    if (LowerCase(self.GetDataColumnByIndex(Index).ColumnName) = LowerCase(Name)) then
    begin
      Result := index;
      Break;
    end;
  end;
end;

function TDataTableOfCosa.GetDataColumnList: TDataColumnListOfCosa;

var
  a: String;
begin
  Result := self._DataColumnList;
end;

function TDataTableOfCosa.GetDataColumnListCount: integer;
begin
  Result:=self._DataColumnList.Count
end;

function TDataTableOfCosa.GetDataConversionList: TListDataConversionOfCosa;
begin
  Result := self._DataConversionList;
end;

function TDataTableOfCosa.GetDataConversionListCount: integer;
begin
  Result := self._DataConversionList.Count;
end;

function TDataTableOfCosa.Clone(CloneRowIdCounter: boolean;
  CloneRowList: boolean): TDataTableOfCosa;
var
  Beta_DataTable: TDataTableOfCosa;
  Alfa_DataRow: TDataRowOfCosa;
  Beta_DataRow: TDataRowOfCosa;
  i,j: Integer;
  Alfa_DataColumn: TDataColumnOfCosa;
  Beta_DataColumn: TDataColumnOfCosa;
begin
  Beta_DataTable := TDataTableOfCosa.Create;
  if (CloneRowIdCounter) then
    Beta_DataTable._DATAROWID_COUNTER := self._DATAROWID_COUNTER;
  Beta_DataTable.TableName := self.TableName;

  Beta_DataTable.SetDataColumnList(self.GetDataColumnList().Clone());

  Beta_DataTable._DataConversionList:=nil;//self._DataConversionList;      *** nil o Clonar??

  //Beta_Datatable.SetDataRowList(self.GetDataRowList().Clone());
  Beta_DataTable._DataRowList:=TList.Create();
  if (CloneRowList) then
  begin
    for j := 0 to self.DataRowListCount - 1 do
    begin
      Alfa_DataRow := self.DataRowList[j];
      Beta_DataRow := Alfa_DataRow.Clone();
      Beta_DataTable.AddDataRow(Beta_DataRow, false);
    end;
  end;
  Beta_DataTable.Visible:=self.Visible;
  Result := Beta_DataTable;
end;

function TDataTableOfCosa.ReadDataCellTextValue(RowId:integer; ColumnName:string;
  var sval:string):boolean;
var
  Index, j: integer;
  DataRow: TDataRowOfCosa;
  DataCell: TDataCellOfCosa;
begin
  sval:='';
  Result:=false;
  DataRow:=self.GetDataRowById(RowId);
  if Assigned(DataRow) then
  begin
    j := self.GetIndexOfDataColumnByName(ColumnName);
    if (j>=0) then
    begin
      DataCell := DataRow.DataCellList[j];
      sval:=DataCell.sval;
      Result:=true;
    end;
  end;
end;

function TDataTableOfCosa.ReadDataCellTextValueByRowIndex(RowIndex: integer;
  ColumnName: string; var sval: string): boolean;
var
  j: integer;
  DataRow: TDataRowOfCosa;
  DataCell: TDataCellOfCosa;
begin
  Result:=false;
  DataRow:=self.GetDataRowByIndex(RowIndex);
  if Assigned(DataRow) then
  begin
    j := self.GetIndexOfDataColumnByName(ColumnName);
    if (j>=0) then
    begin
      DataCell := DataRow.DataCellList[j];
      sval:=DataCell.sval;
      Result:=true;
    end;
  end;
end;

{procedure TDataTableOfCosa.WriteDataCellTextValue(RowName, ColumnName, value:string);
var
  i, j: integer;
  DataRow: TDataRowOfCosa;
  DataCell: TDataCellOfCosa;
begin
  DataRow:=self.GetDataRowByName(RowName);
  j:=self.GetIndexOfDataColumnByName(ColumnName);
  DataCell := DataRow.DataCellList[j];
  DataCell.sval:=value;
end;}

function TDataTableOfCosa.AddDataRow(xDataRow: TDataRowOfCosa;
  SetRowId: boolean): integer;
begin
  Result := -1;
  if xDataRow <> nil then
  begin
    if (SetRowId) then
    begin
      _DATAROWID_COUNTER := _DATAROWID_COUNTER + 1;
      xDataRow._RowId := _DATAROWID_COUNTER;
    end
    else
    begin
      if (not (self is TDataVTableOfCosa)) and Assigned(self.GetDataRowById(xDataRow._RowId)) then
        raise Exception.Create('TDataTableOfCosa.AddDataRow: RowId ' +
          IntToStr(xDataRow._RowId)+' ya existe en la tabla '+TableName );
    end;
    xDataRow.DataTableOwner:=self;
    self._DataRowList.Add(xDataRow);
    Result := _DATAROWID_COUNTER;
  end;
end;

function TDataTableOfCosa.ConvertDataRowToNewestVersion(var DataRow: TDataRowOfCosa;
  DataTableAux: TDataTableOfCosa): Boolean;
var
  i, j: integer;
  DataColumn: TDataColumnOfCosa;
  DataCell: TDataCellOfCosa;
  vVariable1: string='';
  vVariable2: string='';
  vResult: string='';
  a: Integer;
  converter: TDataConversionOfCosa;
  sval: string='';
  b: String;
  Index: Integer;
begin
  for i := 0 to self.DataColumnListCount - 1 do
  begin

    DataColumn:=self.DataColumnList[i];

    {dfusco@20150505
     El índice en la tabla vieja no siempre es el mismo en la tabla nueva. }
    Index:=DataTableAux.GetIndexOfDataColumnByName(DataColumn.ColumnName);

    converter:=nil;
    if self.GetDataConversionList().FindConverterByName(DataColumn.ColumnName, converter) then
    begin

      //DataTableAux tiene una única fila (RowId=1001)
      DataTableAux.ReadDataCellTextValue(1001,converter.nVariable1,vVariable1);
      DataTableAux.ReadDataCellTextValue(1001,converter.nVariable2,vVariable2);

      converter.Convert(vResult, vVariable1, vVariable2);

      DataCell:=TDataCellOfCosa.Create(vResult);
    end

    else if Index=-1 then
    begin
      DataCell:=TDataCellOfCosa.Create(DataColumn.ValorPorDefecto);
    end

    else
    begin
      DataCell:=DataTableAux.DataRowList[0].DataCellList[Index].Clone();
    end;

    DataRow.AddDataCell(DataCell);
  end;
end;


function TDataTableOfCosa.GetDataRowWithDefaulValues: TDataRowOfCosa;
var
  dr: TDataRowOfCosa;
  i: Integer;
begin

  dr := TDataRowOfCosa.Create;
  for i := 0 to DataColumnListCount - 1 do
    dr.AddDataCell(TDataCellOfCosa.Create(DataColumnList[i].ValorPorDefecto));

  Result := dr;
end;

function TDataTableOfCosa.Equals(dt: TDataTableOfCosa): boolean;
begin
  if Assigned(dt) and self.GetDataColumnList.Equals(dt.GetDataColumnList) then
    Result := True
  else
    Result := False;
end;

procedure TDataTableOfCosa.Merge(Beta_DataTable: TDataTableOfCosa);
var
  RowListCount: Integer;
  Alfa_DataRow, Beta_DataRow: TDataRowOfCosa;
  i,j: Integer;
begin
  if Assigned(Beta_DataTable) and self.Equals(Beta_DataTable) and (Beta_DataTable.DataRowListCount>0) then
  begin
    RowListCount:=Beta_DataTable.DataRowListCount;
    for i:=0 to Beta_DataTable.DataRowListCount-1 do
    begin
      Beta_DataRow := Beta_DataTable.DataRowList[i];
      case Beta_DataRow.SIUD of
        TSIUD.TSELECT:
        begin
          Alfa_DataRow:=self.GetDataRowById(Beta_DataRow._RowId);
          if Assigned(Alfa_DataRow) then
          begin
            if not Alfa_DataRow.Equals(Beta_DataRow) then
             raise Exception.Create('TDataTableOfCosa.Merge: DataRows en modo select son distintos');
          end
          else
              self.AddDataRow(Beta_DataRow.Clone(true), false);
        end;
        TSIUD.TINSERT:
        begin
          self.AddDataRow(Beta_DataRow.Clone(true),false);
        end;
        TSIUD.TUPDATE:
        begin
          Alfa_DataRow:=GetDataRowById(Beta_DataRow._RowId);
          if Assigned(Alfa_DataRow) then
          begin
            Alfa_DataRow.SIUD:=TSIUD.TUPDATE;
            Alfa_DataRow.Merge(Beta_DataRow);
          end
          else
          begin
            self.AddDataRow(Beta_DataRow.Clone(true), false);
            //raise Exception.Create(' TDataTableOfCosa.Merge TSIUD.UPDATE NO DEBERIA PASAR POR ACA');
          end;
        end;
        TSIUD.TDELETE:
        begin
          Alfa_DataRow:=GetDataRowById(Beta_DataRow._RowId);
          if Assigned(Alfa_DataRow) then
          begin
            Alfa_DataRow.SIUD:=TSIUD.TDELETE;
          end
        end;
      end;
    end;
  end;
end;

procedure TDataTableOfCosa.Free;
var
  i,j: Integer;
begin
  if Assigned(self._DataColumnList) then
  begin
   for i:=0 to self._DataColumnList.Count-1 do
      TDataColumnOfCosa(self._DataColumnList.Items[i]).Free;
   self._DataColumnList.Free;
  end;

  if Assigned(self._DataConversionList) then self._DataConversionList.Free;

  if Assigned(self._DataRowList) then
  begin
    for j:=0 to self._DataRowList.Count-1 do
      TDataRowOfCosa(self._DataRowList.Items[j]).Free;
    self._DataRowList.Free();
  end;
end;

procedure TDataTableOfCosa.Clear;
var
  j: Integer;
begin
  self._DATAROWID_COUNTER:=1000;
  if Assigned(self._DataRowList) then
  begin
    for j:=0 to self._DataRowList.Count-1 do
      self.DataRowList[j].Free();
    self._DataRowList.Clear();
  end;

end;

function TDataTableOfCosa.GetColValues(kCol: Integer): TStringList;
var
  res: TStringList;
  dcll: TDataCellOfCosa;
  i: Integer;
begin

  if (kCol<0) or (kCol>self.DataColumnListCount) then
    raise Exception.Create(
     'TDataTableOfCosa.GetColValues : La columna a evaluar no pertenece a tabla '+self.TableName);

  res:=TStringList.Create;

  for i:=0 to self.DataRowListCount-1 do
  begin
    try
      dcll:=self.DataRowList[i].DataCellList[kCol];
    except
      raise Exception.Create(
       'TDataTableOfCosa.GetColValues : DimError: Length(fila)<kCol');
    end;
    res.Add(dcll.sval);
  end;

  Result:=res;
end;

function TDataTableOfCosa.GetColValues(ColName: string): TStringList;
var
  kCol: Integer;
begin
  kCol := self.GetIndexOfDataColumnByName(ColName);
  Result:=self.GetColValues(kCol);
end;

procedure TDataTableOfCosa.AppendDataTable(dt: TDataTableOfCosa;
  reAssignIds: Boolean);
var
  i: Integer;
begin
  if not Assigned(dt) then
    Exit;

  if not self.Equals(dt) then
    raise Exception.Create('TDataTableOfCosa.AppendDataTable: las tablas no tienen igual definicion de columnas');

  for i:=0 to dt.DataRowListCount-1 do
    self.AddDataRow(dt.DataRowList[i].Clone(not reAssignIds), reAssignIds);

end;

function TDataTableOfCosa.Query(Select: string; ColumnNameToEval: string;
  Value: string): TDataTableOfCosa;
var
  dt: TDataTableOfCosa;
  ColumnNames: TStringList;
  SelectAster:Boolean;
  kCol:Integer;
  ColValues: TStringList;
  evalValue: Boolean;
  kEvalCol:Integer;
  EvalColValues: TStringList;
  i, j: Integer;
  kRow: Integer;

begin

  ColumnNames := TStringList.Create;
  ColumnNames.DelimitedText := Select;

  kEvalCol:= self.GetIndexOfDataColumnByName(ColumnNameToEval);
  if (kEvalCol=-1) and (ColumnNameToEval<>'')  then
    raise Exception.Create(
     'TDataTableOfCosa.Query : La columna a evaluar no pertenece a tabla '+self.TableName);

  // Si ColumnNames tiene un '*' entre sus items devuelte una tabla con todas
  // las columnas
  SelectAster:=ColumnNames.IndexOf('*') > -1;
  //Si ColumnNameToEval es vacio, se devuelven todas las filas de la tabla
  evalValue:=ColumnNameToEval<>'';

  if SelectAster then
  begin
    dt:=self.Clone(true,false);
    //reocrrido de las filas
    for i := 0 to self.DataRowListCount - 1 do
    begin
      if evalValue then
        dt.AddDataRow(self.DataRowList[i].ConditionalClone(kEvalCol, Value), false)
      else
        dt.AddDataRow(self.DataRowList[i].Clone(true), false);
    end;
  end
  else
  begin
    dt:= TDataTableOfCosa.Create(self.TableName);
    for i := 0 to ColumnNames.Count - 1 do
    begin
      kCol:=self.GetIndexOfDataColumnByName(ColumnNames[i]);;
      if kCol=-1 then
        raise Exception.Create(
         'TDataTableOfCosa.Query : La columna "'+ColumnNames[i]+
         '" no pertenece a tabla '+self.TableName);

      dt.AddDataColumn(self.GetDataColumnByIndex(kCol).Clone());
      ColValues:=self.GetColValues(kCol);
      EvalColValues:=self.GetColValues(kEvalCol);
      kRow:=0;
      for j := 0 to ColValues.Count-1 do
      begin
        if EvalColValues[j]=Value then
        begin
          if i=0 then
            dt.AddDataRow(TDataRowOfCosa.Create(dt));

          dt.DataRowList[kRow].AddDataCell(TDataCellOfCosa.Create(ColValues[j]));
          kRow:=kRow+1;
        end;
      end;
    end;
  end;

  ColumnNames.Free;
  Result:=dt;
end;

function TDataTableOfCosa.Query(Select: string; rowId: Integer
  ): TDataTableOfCosa;
var
  dt: TDataTableOfCosa;
  ColumnNames: TStringList;
  SelectAster:Boolean;
  kCol:Integer;
  i, j: Integer;
  kRow: Integer;
  dr1, dr2: TDataRowOfCosa;

begin

  ColumnNames := TStringList.Create;
  ColumnNames.DelimitedText := Select;

  // Si ColumnNames tiene un '*' entre sus items devuelte una tabla con todas
  // las columnas
  SelectAster:=ColumnNames.IndexOf('*') > -1;
  dr1:=self.GetDataRowById(rowId);

  if SelectAster then
  begin
    dt:=self.Clone(true,false);
    if Assigned(dr1) then
      dt.AddDataRow(dr1.Clone(), false);
  end
  else
  begin
    dt:= TDataTableOfCosa.Create(self.TableName);
    dr2 := TDataRowOfCosa.Create(dt);
    for i := 0 to ColumnNames.Count - 1 do
    begin
      kCol:=self.GetIndexOfDataColumnByName(ColumnNames[i]);;
      if kCol=-1 then
        raise Exception.Create(
         'TDataTableOfCosa.Query : La columna "'+ColumnNames[i]+
         '" no pertenece a tabla '+self.TableName);
        dt.AddDataColumn(self.GetDataColumnByIndex(kCol).Clone());
        if Assigned(dr1) then
          dr2.AddDataCell(TDataCellOfCosa.Create(dr1.GetDataCellByIndex(kCol).sval));
    end;
    dt.AddDataRow(dr2, false)
  end;
  ColumnNames.Free;
  Result:=dt;
end;

function TDataTableOfCosa.View(ColNames: string): TDataTableOfCosa;
var
  dt: TDataTableOfCosa;
  ColumnNames: TStringList;
  i, j, kCol: Integer;
  vdr: TDataVRowOfCosa;
begin
  dt:= TDataTableOfCosa.Create(self.TableName+'_view');
  ColumnNames := TStringList.Create;
  ColumnNames.DelimitedText := ColNames;

  for i:=0 to ColumnNames.Count-1 do
  begin
    kCol:=Self.GetIndexOfDataColumnByName(ColumnNames[i]);
    dt.AddDataColumn(TDataColumnOfCosa.Create(ColumnNames[i],nil));
    for j:=0 to self.DataRowListCount-1 do
    begin
      if i=0 then
      begin
        vdr:=TDataVRowOfCosa.Create(dt);
        vdr.OriginalDataTableName:=self.TableName;
        vdr.OriginalDataRowId:=self.DataRowList[j]._RowId;
        dt.AddDataRow(vdr, true);
      end;

      if kCol>-1 then
        dt.DataRowList[j].AddDataCell(TDataCellOfCosa.Create(
          self.DataRowList[j].GetDataCellByIndex(kCol).sval))
      else
        dt.DataRowList[j].AddDataCell(TDataCellOfCosa.Create('-'));
    end;
  end;
  Result:=dt;
end;


{ TDataVTableOfCosa }

function TDataVTableOfCosa.GetChildClassNameList: TStringList;
begin
  Result := self._ChildClassNameList;
end;

constructor TDataVTableOfCosa.Create(xDataTableName: string);
begin
  inherited Create(xDataTableName);
  self._ChildClassNameList := TStringList.Create;
end;

constructor TDataVTableOfCosa.Create(xDataTableName: string;
  xDataColumnList: TDataColumnListOfCosa);
begin
  inherited Create(xDataTableName);
  self._ChildClassNameList := TStringList.Create;
  self._DataColumnList:=xDataColumnList;
end;

constructor TDataVTableOfCosa.Create(xDataTableName: string;
  xClassNameList: TStringList; xDataColumnList: TDataColumnListOfCosa);
begin
  inherited Create(xDataTableName);
  self._ChildClassNameList := xClassNameList;
  self._DataColumnList := xDataColumnList;
end;

function TDataVTableOfCosa.AddChildClassName(AClassName: string): integer;
var
  index: Integer;
begin
  if self._ChildClassNameList.Find(AClassName, index) then
    Result:=index
  else
    Result := self._ChildClassNameList.Add(AClassName);
end;

function TDataVTableOfCosa.GetDataRowList(ds: TDataStore): TList;
var
  z, i, j: integer;
  AClassName: string;
  DataTable: TDataTableOfCosa;
  DataRow: TDataRowOfCosa;

  TheDataRowList: TList;
  ADataRow:TDataVRowOfCosa;
  ADataColumn: TDataColumnOfCosa;
  DataColumnIndex: integer;
  ADataCell: TDataCellOfCosa;
  a: Integer;
begin

  TheDataRowList := TList.Create;
  for z := 0 to self._ChildClassNameList.Count - 1 do
  begin
    AClassName := self._ChildClassNameList[z];
    DataTable:=nil;
    ds.GetDataTableOnLastVersion('dt_'+AClassName, DataTable);
    if (DataTable <> nil) then
    begin
      for i := 0 to DataTable.DataRowListCount - 1 do
      begin
        DataRow := DataTable.DataRowList[i];
        if DataRow.SIUD <> TSIUD.TDELETE then
        begin
          ADataRow:=TDataVRowOfCosa.Create(self, 'dr_'+AClassName);
          for j := 0 to self._DataColumnList.Count - 1 do
          begin
            ADataColumn := self._DataColumnList[j];
            DataColumnIndex:=DataTable.GetDataColumnList().GetIndexOfDataColumnByName(ADataColumn.ColumnName);
            ADataCell:=DataRow.GetDataCellByIndex(DataColumnIndex).Clone();
            ADataRow.AddDataCell(ADataCell);
          end;
          ADataRow.OriginalDataTableName:=DataTable.TableName;
          ADataRow.OriginalDataRowId:=DataRow._RowId;
          TheDataRowList.Add(ADataRow);
        end
        else
          a:=1;
      end;
    end;
  end;
  Result := TheDataRowList;
end;

constructor TDataTableOfListOfCosa.Create;
begin
  self.Visible:=False;
  self.TableName := 'dt_TList';
  self._LastIdParent := 0;
  self._DATAROWID_COUNTER := 1000;

  self._DataColumnList := TDataColumnListOfCosa.Create(nil);
  self._DataColumnList.AddDataColumn('IdParent', TDataTypeStringOfCosa);
  self._DataColumnList.AddDataColumn('IdChild', TDataTypeStringOfCosa);

  self._DataRowList := TList.Create;
end;

function TDataTableOfListOfCosa.GetNextIdParent: integer;
begin
  self._LastIdParent := self._LastIdParent + 1;
  Result := self._LastIdParent;
end;

function TDataTableOfListOfCosa.Clone(CloneRowIdCounter: boolean;
  CloneRowList: boolean): TDataTableOfListOfCosa;
var
  dt: TDataTableOfListOfCosa;
begin
  dt:= TDataTableOfListOfCosa( inherited Clone(CloneRowIdCounter, CloneRowList) );
  dt._LastIdParent:=self._LastIdParent;

  Result:=dt;
end;

procedure TDataTableOfListOfCosa.Free;
begin
  inherited;
end;

procedure TDataTableOfListOfCosa.Clear;
begin
  inherited;
  _LastIdParent:=0;
end;

//////////////////////////////////////////////////////////////////////////////
//TDataRowOfCosa
//////////////////////////////////////////////////////////////////////////////
constructor TDataRowOfCosa.Create(xRowName: string);
begin
  inherited Create;
  self.RowName := xRowName;
  self._DataCellList := TList.Create;
  self.DataTableOwner := nil;
  self.SIUD := TSIUD.TINSERT;
end;

constructor TDataRowOfCosa.Create(xRowId: integer);
begin
  inherited Create;
  self._DataCellList := TList.Create();
  self._RowId:=xRowId;
end;

constructor TDataRowOfCosa.Create(xDataTableOwner: TDataTableOfCosa;
  xRowName: string);
begin
  inherited Create;
  self.RowName := xRowName;
  self._DataCellList := TList.Create;
  self.DataTableOwner := xDataTableOwner;
  self.SIUD := TSIUD.TINSERT;
end;

function TDataRowOfCosa.AddDataCell(xDataCell: TDataCellOfCosa; SetOwner:boolean): Integer;
begin
  if Assigned(xDataCell) and SetOwner then
    xDataCell.DataRowOwner := self;
  Result := self._DataCellList.Add(xDataCell);
end;

procedure TDataRowOfCosa.InsertDataCell(ADataCell: TDataCellOfCosa;
  AIndex: integer);
begin
  self._DataCellList.Insert(AIndex, ADataCell);
end;

function TDataRowOfCosa.Clone(CloneRowId: boolean): TDataRowOfCosa;
var
  dr: TDataRowOfCosa;
  i: Integer;
begin
  if self is TDataVRowOfCosa then
  begin
    dr:=TDataVRowOfCosa.Create(self.DataTableOwner, self.RowName);
    TDataVRowOfCosa(dr).OriginalDataTableName:=TDataVRowOfCosa(self).OriginalDataTableName;
    TDataVRowOfCosa(dr).OriginalDataRowId:=TDataVRowOfCosa(self).OriginalDataRowId;
  end
  else
    dr := TDataRowOfCosa.Create(self.DataTableOwner, self.RowName);

  for i := 0 to self.DataCellListCount - 1 do
    dr.AddDataCell(self.DataCellList[i].Clone());
  if CloneRowId then
    dr._RowId := self._RowId;
  dr.SIUD := self.SIUD;
  Result := dr;
end;

function TDataRowOfCosa.ConditionalClone(i: integer; Value: string): TDataRowOfCosa;
begin
  Result := nil;
  if (i <> -1) and (Value <> '') and (self.DataCellList[i].sval = Value) then
    Result := self.Clone;
end;

function TDataRowOfCosa.ConditionalGetDataCell(kCell, kCellToEval: Integer;
  Value: String): TDataCellOfCosa;
begin
  // kCellToEval=-1 Parche TODO: Corregir!!
  if (kCellToEval=-1) or (DataCellList[kCellToEval].sval = Value) then
    Result := DataCellList[kCell]
  else
    Result := nil;
end;

function TDataRowOfCosa.Equals(dr: TDataRowOfCosa): boolean;
var
  i: Integer;
  c1: Integer;
begin
  Result := True;
  c1 := self.DataCellListCount;

  if not (Assigned(dr) and (c1 <> 0) and (c1 = dr.DataCellListCount)) then
    Result := False
  else
    for i := 0 to c1 - 1 do
      if not self.DataCellList[i].Equals(dr.DataCellList[i]) then
      begin
        Result := False;
        Break;
      end;
end;

procedure TDataRowOfCosa.Merge(ADataRow: TDataRowOfCosa);
var
  i: Integer;
begin
  for i:=0 to ADataRow.DataCellListCount-1 do
    self.DataCellList[i].sval:=ADataRow.DataCellList[i].sval;

end;

function TDataRowOfCosa.CompareTo(Beta_DataRow: TDataRowOfCosa;
  Gamma_DataTable: TDataTableOfCosa): boolean;
var
  ok:boolean;
  Gamma_DataRow: TDataRowOfCosa;
  split: TStringList;
  dtn: String;
  dri: Integer;
  dcn: String;
  dcv: String;
  ds: TDataStore;
  dt: TDataTableOfCosa;
  dc: TDataColumnOfCosa;
  j: Integer;
  dco: TDataColumnOfCosa;
  dr: TDataRowOfCosa;
  dce: TDataCellOfCosa;
  dt_TList: TDataTableOfCosa;
  sval: string;
  i: Integer;
  Gamma_DataCell: TDataCellOfCosa;
begin


end;

procedure TDataRowOfCosa.Free;
var
  i: Integer;
begin
  DataTableOwner:=nil;
  for i:=0 to DataCellListCount-1 do
    if Assigned(DataCellList[i]) then
      DataCellList[i].Free;

  _DataCellList.Free;
end;

function TDataRowOfCosa.ToString: TStrings;
var
  strs: TStringList;
  i: Integer;
begin
  strs:=TStringList.Create;
  for i:=0 to DataCellListCount-1 do
    strs.Add(DataCellList[i].sval);

  Result:=strs;
end;

function TDataRowOfCosa.GetRowId: integer;
begin
  Result:=self.RowId;
end;

procedure TDataRowOfCosa.SetRowId(AValue: integer);
begin
  self.RowId:=AValue;
end;

function TDataRowOfCosa.GetDataCellByIndex(Index: integer): TDataCellOfCosa;
begin
  Result := self._DataCellList.Items[Index];
end;

function TDataRowOfCosa.GetDataCellListCount: integer;
begin
  Result := self._DataCellList.Count;
end;

function TDataRowOfCosa.GetLastDataCell: TDataCellOfCosa;
begin
  Result := self._DataCellList.Items[self.DataCellListCount - 1];
end;

function TDataCellOfCosa.getVal: string;
begin
  Result:=_sval;
end;

procedure TDataCellOfCosa.setVal(AValue: string);
begin
  self._sval:=AValue;
  if Assigned(self.DataRowOwner) and (self.DataRowOwner.SIUD = TSELECT) then
    self.DataRowOwner.SIUD:=TUPDATE;
end;

//////////////////////////////////////////////////////////////////////////////
//TDataCellOfCosa
//////////////////////////////////////////////////////////////////////////////
constructor TDataCellOfCosa.Create(xsval: string);
begin
  SIUD:=TSIUD.TSELECT;
  self.sval := xsVal;
end;

function TDataCellOfCosa.Clone: TDataCellOfCosa;
var
  dce: TDataCellOfCosa;
  asval: String;
begin
  asval:=self.sval;
  Result := TDataCellOfCosa.Create(self.sval);
end;

function TDataCellOfCosa.Equals(dcll: TDataCellOfCosa): boolean;
begin
  if Assigned(dcll) then
    Result := self.sval = dcll.sval
  else
    Result := False;
end;

procedure TDataCellOfCosa.Free;
begin
  sval:='';
  if Assigned( pval ) then
    Dispose( pval );
  DataRowOwner:=nil;
end;

procedure TDataColumnListOfCosa.SetClassOwner(AValue: TClaseDeCosa);
begin
  self._ClassOwner := AValue;
end;

procedure TDataColumnListOfCosa.SetItemListClaseDeCosa(AValue: TClaseDeCosa);
begin
  self._ItemListClaseDeCosa := AValue;
end;

{ TDataColumnListOfCosa }
function TDataColumnListOfCosa.GetItem(Index: Integer): TDataColumnOfCosa;
begin
  Result := TDataColumnOfCosa(inherited Items[Index]);
end;

constructor TDataColumnListOfCosa.Create(xClassOwner: TClaseDeCosa; xVersion: integer);
var
  b: String;
begin
  self.Create(xClassOwner,nil,xVersion);
end;

constructor TDataColumnListOfCosa.Create(xClassOwner: TClaseDeCosa;
  xItemListClaseDeCosa: TClaseDeCosa; xVersion: Integer);
begin
  inherited Create;
  self._ItemListClaseDeCosa := xItemListClaseDeCosa;
  self._ClassOwner := xClassOwner;
  self._Version := xVersion;
  //self.RegularExpressionValidatorList:=TListRegularExpressionValidator.Create();
  self.ValidationList:=TListValidation.Create();
end;

function TDataColumnListOfCosa.AddDataColumn(xDataColumn: TDataColumnOfCosa;
  SetOwner: boolean): integer;
begin
  if SetOwner then
    XDataColumn.DataColumnListOwner:=self;
  Result := inherited Add(XDataColumn);
  XDataColumn.Index:=Result;
end;

function TDataColumnListOfCosa.AddDataColumn(xColumnName: string; xDataType: DataTypeOfCosa): integer;
var
  DataColumn: TDataColumnOfCosa;
begin
  DataColumn:=TDataColumnOfCosa.Create(xColumnName, xDataType);
  Result:=self.AddDataColumn(DataColumn);
end;

function TDataColumnListOfCosa.AddDataColumn(xColumnName:  string;
                                             xDataType:    DataTypeOfCosa;
                                             xOptions:     array of string;
                                             xVersionAlta: integer;
                                             xVersionBaja: integer;
                                             xClaseDeCosa: TClaseDeCosa): integer;
begin
  Result:=self.AddDataColumn(xColumnName,
                             TBlackHole(nil).TheHole,
                             xDataType,
                             xOptions,
                             xVersionAlta,
                             xVersionBaja,
                             xClaseDeCosa);
end;

function TDataColumnListOfCosa.AddDataColumn(xColumnName:  string;
                                             xDataType:    DataTypeOfCosa;
                                             xVersionAlta: integer;
                                             xVersionBaja: integer;
                                             xClaseDeCosa: TClaseDeCosa): integer;
begin
  Result:=self.AddDataColumn(xColumnName,
                             TBlackHole(nil).TheHole,
                             xDataType,
                             [],
                             xVersionAlta,
                             xVersionBaja,
                             xClaseDeCosa);
end;

function TDataColumnListOfCosa.AddDataColumn(xColumnName:  string;
                                             var xField;
                                             xDataType:    DataTypeOfCosa;
                                             xVersionAlta: integer;
                                             xVersionBaja: integer;
                                             xClaseDeCosa: TClaseDeCosa): integer;
begin
  Result:=self.AddDataColumn(xColumnName,
                             xField,
                             xDataType,
                             [],
                             xVersionAlta,
                             xVersionBaja,
                             xClaseDeCosa);
end;

function TDataColumnListOfCosa.AddDataColumn(xColumnName:  string;
                                             var xField;
                                             xDataType:    DataTypeOfCosa;
                                             xOptions:     array of string;
                                             xVersionAlta: integer;
                                             xVersionBaja: integer;
                                             xClaseDeCosa: TClaseDeCosa): integer;
var
  DataColumn: TDataColumnOfCosa;
  offset: longint;
  a: Integer;
begin
  Result:=-1;

  a:=self.Version;
  a:=xVersionBaja;

  if ((self.Version = -2) and (xVersionBaja = -1)) or

     ((self.Version >= xVersionAlta) and ((xVersionBaja = -1) or (self.Version < xVersionBaja))) then

  begin
    offset := longint(@xField);
    DataColumn := TDataColumnOfCosa.Create(xColumnName,
                                           xDataType,
                                           offset,
                                           xOptions,
                                           xVersionAlta,
                                           xVersionBaja,
                                           xClaseDeCosa);
    DataColumn.DataColumnListOwner:=self;
    Result := inherited Add(DataColumn);
    DataColumn.Index:=Result;
  end;
end;

function TDataColumnListOfCosa.GetIndexOfDataColumnByName(AName: string): integer;
var
  Index: integer;
  foundIt: boolean;
  Count: integer;
  a: string;
begin
  Result := -1;
  for Index := 0 to self.Count - 1 do
  begin
    if (LowerCase(self.Items[Index].ColumnName) = LowerCase(AName)) then
    begin
      Result := Index;
      break;
    end;
  end;
end;

procedure TDataColumnListOfCosa.Move(CurIndex, NewIndex: Integer);
var
  i: Integer;
begin
  inherited Move(CurIndex, NewIndex);
  for i:=0 to self.Count-1 do
    Items[i].Index:=i;
end;



// Lee el campo iCampo de aCosa y lo retorna como String
function TDataColumnListOfCosa.GetValStr( aCosa: TCosa; iCampo: integer ): string;
var
  aColDef: TDataColumnOfCosa;
begin
  aColDef:= GetItem( iCampo );
  result:= aColDef.DataType.ToString( pointer(aCosa)+ aColDef.Offset );
end;

// Fija el valor del campo iCampo de aCosa de acuerdo al NuevoValor
// El resultado es TRUE si logró setear el campo (no falló la conversión de tipo)
function TDataColumnListOfCosa.SetValStr( aCosa: TCosa; iCampo: integer; NuevoValor: string ): boolean;
var
  aColDef: TDataColumnOfCosa;
begin
  aColDef:= GetItem( iCampo );
  result:= aColDef.DataType.Eval( NuevoValor, pointer( aCosa )+ aColDef.Offset );
end;

function TDataColumnListOfCosa.Clone: TDataColumnListOfCosa;
var
  dcl: TDataColumnListOfCosa;
  i: Integer;
begin
  dcl := TDataColumnListOfCosa.Create(self._ClassOwner);
  for i := 0 to self.Count - 1 do
  begin
    dcl.Add(self.Items[i].Clone);
    dcl.Items[i].DataColumnListOwner:=dcl;
  end;
  dcl.ClassOwner := self.ClassOwner;
  dcl.ItemListClaseDeCosa := self.ItemListClaseDeCosa;

  if Assigned(self.ValidationList) then
    dcl.ValidationList:=self.ValidationList.Clone();
  Result := dcl;
end;

function TDataColumnListOfCosa.Equals(dcl: TDataColumnListOfCosa): boolean;
var
  i: Integer;
  c1: Integer;
begin
  Result := True;
  c1 := self.Count;

  if not (Assigned(dcl) and (c1 <> 0) and (c1 = dcl.Count)) then
    Result := False
  else
    for i := 0 to c1 - 1 do
      if not self.Items[i].Equals(dcl.Items[i]) then
      begin
        Result := False;
        Break;
      end;
end;

procedure TDataColumnListOfCosa.Free;
var
  i: Integer;
begin
  _ClassOwner:=nil;
  _ItemListClaseDeCosa:=nil;
  for i:=0 to Count-1 do
    Items[i].Free;

  Inherited Free;
end;

function TDataColumnListOfCosa.GetClassOwner():TClaseDeCosa;
begin
  Result := self._ClassOwner;
end;

function TDataColumnListOfCosa.GetItemListClaseDeCosa: TClaseDeCosa;
begin
  Result := self._ItemListClaseDeCosa;
end;

function TDataColumnListOfCosa.GetVersion: Integer;
begin
  Result := self._Version;
end;

constructor TDataColumnListOfCosa.Create;
begin
  inherited Create;
end;

//////////////////////////////////////////////////////////////////////////////
//TDataColumnOfCosa
//////////////////////////////////////////////////////////////////////////////
constructor TDataColumnOfCosa.Create(xColumnName:  string;
                                     xDataType:    DataTypeOfCosa;
                                     xOffset:      longint;
                                     xVersionAlta: integer;
                                     xVersionBaja: integer;
                                     xClaseDeCosa: TClaseDeCosa);
begin
  self.Create(xColumnName, xDataType, xOffset, [], xVersionAlta, xVersionBaja, xClaseDeCosa);
end;

constructor TDataColumnOfCosa.Create(xColumnName:  string;
                                     xDataType:    DataTypeOfCosa;
                                     xOffset:      longint;
                                     xOptions:     array of string;
                                     xVersionAlta: integer;
                                     xVersionBaja: integer;
                                     xClaseDeCosa: TClaseDeCosa);
begin
  self.ColumnName := LowerCase(xColumnName);

  self.DataType := xDataType.Create(Self);

  self.Offset := xOffset;

  if Length(xoptions)=2 then
  begin
    self.LabelCaption    := xoptions[0];
    self.ValorPorDefecto := xoptions[1];
  end
  else if Length(xoptions)=1 then
  begin
    self.LabelCaption    := xoptions[0];
    self.ValorPorDefecto := '';
  end;

  self.VersionAlta := xVersionAlta;
  self.VersionBaja := xVersionBaja;

  if Assigned(self.DataType) then
    self.DataType.ClaseDeCosa := xClaseDeCosa;

end;

constructor TDataColumnOfCosa.Create(xColumnName:  string;
                                     xDataType:    DataTypeOfCosa;
                                     xClaseDeCosa: TClaseDeCosa);
begin
  self.Create(xColumnName, xDataType, -1, -1, -1, xClaseDeCosa);
end;

function TDataColumnOfCosa.Clone: TDataColumnOfCosa;
var
  DataColumn: TDataColumnOfCosa;
begin

  DataColumn:=TDataColumnOfCosa.Create();
  DataColumn.ColumnName := self.ColumnName;
  DataColumn.Offset := self.Offset;
  DataColumn.ValorPorDefecto := self.ValorPorDefecto;
  DataColumn.VersionAlta := self.VersionAlta;
  DataColumn.VersionBaja := self.VersionBaja;
  DataColumn.LabelCaption:=self.LabelCaption;

  if Assigned(self.DataType) then
  begin
    DataColumn.DataType:= self.DataType.Clone();
    DataColumn.DataType.DataColumnOwner:=DataColumn;
  end
  else
    DataColumn.DataType:= nil;

  Result := DataColumn;
end;

function TDataColumnOfCosa.Equals(dc: TDataColumnOfCosa): boolean;
begin
  Result:= LowerCase(self.ColumnName)=LowerCase(dc.ColumnName);
end;

procedure TDataColumnOfCosa.Free;
begin
  DataType.Free;
end;

constructor TDataTypeOfCosa.Create(DataColumnOwner: TDataColumnOfCosa);
begin
  inherited Create;
  self.DataColumnOwner:= DataColumnOwner;
end;

function TDataTypeOfCosa.Validate(sval: string; var ErrorMessage: string
  ): boolean;
begin
  Result:=true;
end;

function TDataTypeOfCosa.CompareTo(TheOperator: TOperator; asval: string;
  bsval: string): boolean;
begin
  Result:=true;
end;

//////////////////////////////////////////////////////////////////////////////
//TDataTypeOfCosa
//////////////////////////////////////////////////////////////////////////////
function TDataTypeOfCosa.IsAMacroVariable(sval: string): boolean;
var
  r: boolean;
  RegexObj: TRegExpr;
begin
  RegexObj := TRegExpr.Create;
  RegexObj.Expression := '\{\$.*\}';
  r := RegexObj.Exec(sval);
  RegexObj.Free;
  Result := r;
end;

function TDataTypeOfCosa.Clone: TDataTypeOfCosa;
var
  DataType: TDataTypeOfCosa;
begin
  DataType:=TDataTypeOfCosa(self.ClassType.Create());
  DataType.ClaseDeCosa:=self.ClaseDeCosa;
  Result := DataType;
end;

function TDataTypeOfCosa.Equals(dtc: TDataTypeOfCosa): boolean;
begin
  Result := self.ClaseDeCosa = dtc.ClaseDeCosa;
end;

function TDataTypeOfCosa.Format(sval: string): string;
begin
  Result:=sval;
end;

procedure TDataTypeOfCosa.Free;
begin
  ClaseDeCosa:=nil;
end;

function TDataTypeIntegerOfCosa.Validate(sval: string; var ErrorMessage: string
  ): boolean;
var
  RegexObj: TRegExpr;
begin
  RegexObj := TRegExpr.Create;
  RegexObj.Expression := '^[0-9]+$';
  Result := RegexObj.Exec(sval);
  RegexObj.Free;

  ErrorMessage:='El valor ingresado no es correcto. Ingrese un número entero.'

  //Try
  //  Writeln (StrToInt('12345678901234567890'));
  //except
  //  On E : EConvertError do
  //    Writeln ('Invalid number encountered');
  //end;
end;

function TDataTypeIntegerOfCosa.CompareTo(TheOperator: TOperator;
  asval: string; bsval: string): boolean;
var
  aival:integer;
  bival:integer;
begin

  aival:=0;
  bival:=0;

  self.Eval(asval,@aival);
  self.Eval(bsval,@bival);

  case TheOperator of

    TOperator.EQUAL:            Result:=aival= bival;
    TOperator.DISTINCT:         Result:=aival<>bival;
    TOperator.LESS:             Result:=aival< bival;
    TOperator.LESS_OR_EQUAL:    Result:=aival<=bival;
    TOperator.GREATER:          Result:=aival> bival;
    TOperator.GREATER_OR_EQUAL: Result:=aival>=bival;

  end;
end;

function TDataTypeIntegerOfCosa.Eval(sval: string; pval: pointer): boolean;
var
  res: Extended;
begin
  try
    if not (self.IsAMacroVariable(sval)) then
    begin
      res := uevaluador.evalStrToFloat(sval);
      integer(pval^):=trunc(res);
      Result := True;
    end;
  except
    Result := False;
  end;
end;

function TDataTypeIntegerOfCosa.ToString(pval: pointer): string;
begin
  Result := IntToStr(integer(pval^));
end;

function TDataTypeStringOfCosa.Eval(sval: string; pval: pointer): boolean;
begin
  try
    if not (self.IsAMacroVariable(sval)) then
    begin
      string(pval^):=sval;
      Result := True;
    end
  except
    Result := False;
  end;
end;

function TDataTypeStringOfCosa.ToString(pval: pointer): string;
begin
  Result := string(pval^);
end;

function TDataTypeDataRowOfCosa.Eval(sval: string; pval: pointer): boolean;
var
  split: TStringList;
  DataTableName: string;
  DataRowId: integer;
  ref: PCosaConNombre;
  cosa:TCosa;
begin
  Result := True;
  //PARCHE PARA PROBAR
  if sval = '' then
    exit;

  try
    cosa:=nil;
    split := TStringList.Create;
    ExtractStrings(['.'], [], PChar(sval), split);
    DataTableName := split[0];
    DataRowId := StrToInt(split[1]);
    self.DataColumnOwner.DataColumnListOwner.DataTableOwner.DataStoreOwner.CreateClassInstance(DataTableName, DataRowId, cosa);

    //ACA SE PUEDE HACER LAS COMPARACIONES ENTRE CAMPOS, ES PARTE DE LA EVALUACIÓN
    cosa.Validate();

    TCosa(pval^):=cosa;
    if cosa is TCosaConNombre then
    begin
      cosasConNombre.Add(TCosaConNombre(cosa));
    end;
  except
    on e: Exception do
    begin
      Result := False;
      split.Free;
    end;
  end;
end;

function TDataTypeDataRowOfCosa.ToString(pval: pointer): string;
begin

end;

function TDataTypeDateOfCosa.Validate(sval: string; var ErrorMessage: string
  ): boolean;
var
  RegexObj: TRegExpr;
begin
  //RegexObj := TRegExpr.Create;
  //RegexObj.Expression := '^([0-9]{4}|[0-9]{2})[./-]([0]?[1-9]|[1][0-2])[./-]([0]?[1-9]|[1|2][0-9]|[3][0|1])$';
  //Result := RegexObj.Exec(sval);
  //RegexObj.Free;
end;

function TDataTypeDateOfCosa.CompareTo(TheOperator: TOperator; asval: string;
  bsval: string): boolean;
var
  adval:TFecha;
  bdval:TFecha;
begin

  adval:=nil;
  bdval:=nil;

  self.Eval(asval,@adval);
  self.Eval(bsval,@bdval);

  case TheOperator of

    TOperator.EQUAL:            Result:=adval.igualQue(bdval);
    TOperator.DISTINCT:         Result:=not adval.igualQue(bdval);
    TOperator.LESS:             Result:=adval.menorQue(bdval);
    TOperator.LESS_OR_EQUAL:    Result:=adval.menorOIgualQue(bdval);
    TOperator.GREATER:          Result:=adval.mayorQue(bdval);
    TOperator.GREATER_OR_EQUAL: Result:=adval.mayorOIgualQue(bdval);

  end;
end;

function TDataTypeDateOfCosa.Eval(sval: string; pval: pointer): boolean;
var
  f:TFecha;
begin

  if not (self.IsAMacroVariable(sval)) then
  begin
    TFecha(pval^):=TFecha.Create_str(sval);;
    Result := True;
  end;
end;

function TDataTypeDateOfCosa.ToString(pval: pointer): string;
begin
  Result := TFecha(pval^).AsStr;
end;

{ TDataTypeListOfCosa }

function TDataTypeListOfCosa.Eval(sval: string; pval: pointer): boolean;
var
  split: TStringList;
  TableName, ColumnNameToQuery, CellSValToQuery, ColumnNameToGetSVal: string;
  Childs: TDAofString;
  DataTableChild: TDataTableOfCosa;
  DataRowChild: TDataRowOfCosa;
  DataRowChildId: integer;
  IdChild: string;
  i, j: Integer;

  cosa: TCosa=nil;
  lst: TList;

begin
  lst := TList.Create;
  if sval <> '' then
  begin

    split := TStringList.Create;
    try
      ExtractStrings(['.'], [], PChar(sval), split);
      TableName := split[0];
      ColumnNameToQuery := split[1];
      CellSValToQuery := split[2];
      ColumnNameToGetSVal := 'IdChild'
    finally
      split.Free;
    end;
    Childs := self.DataColumnOwner.DataColumnListOwner.DataTableOwner.DataStoreOwner.ReadDataCellTextValues(
      TableName, ColumnNameToQuery, CellSValToQuery, ColumnNameToGetSVal, true);

    for i := 0 to Length(Childs) - 1 do
    begin
      split := TStringList.Create;
      try
        ExtractStrings(['.'], [], PChar(Childs[i]), split);
        TableName := split[0];
        IdChild := copy(split[1], Pos(':', split[1]) + 1, Length(split[1]));
      finally
        split.Free;
      end;
      j := self.DataColumnOwner.DataColumnListOwner.DataTableOwner.DataStoreOwner.GetIndexOfDataTableByName(TableName);
      if (j >= 0) then
        DataTableChild := self.DataColumnOwner.DataColumnListOwner.DataTableOwner.DataStoreOwner.DataTableList[j]
      else
        raise Exception.Create('No se encontro la tabla "' + TableName +
          '" en el DATA_STORE');

      if DataTableChild.TableName = 'dt_TNodo' then
        j := 0;

      self.DataColumnOwner.DataColumnListOwner.DataTableOwner.DataStoreOwner.CreateClassInstance(DataTableChild.TableName, StrToInt(IdChild), cosa);
      lst.Add(cosa);
      if cosa is TCosaConNombre then
        cosasConNombre.Add(TCosaConNombre(cosa));

    end;

  end;
  TList(pval^) := lst;
  Result := True;
end;

function TDataTypeListOfCosa.ToString(pval: pointer): string;
begin

end;

function TDataTypeDoubleOfCosa.Validate(sval: string; var ErrorMessage: string
  ): boolean;
var
  RegexObj: TRegExpr;
begin
  RegexObj := TRegExpr.Create;
  RegexObj.Expression := '/^\d*\.?\d*$/';
  Result := RegexObj.Exec(sval);
  RegexObj.Free;
end;

function TDataTypeDoubleOfCosa.Eval(sval: string; pval: pointer): boolean;
var
  res: Extended;

begin
  try
    if not (self.IsAMacroVariable(sval)) then
    begin
      double(pval^):=StrToFloat(sval);
      Result := True;
    end;
  except
    Result := False;
  end;
end;

function TDataTypeDoubleOfCosa.ToString(pval: pointer): string;
var
  val: NReal;
begin
  val := double(pval^);
  Result := FloatToStrF(val, ffGeneral, uconstantesSimSEE.CF_PRECISION,
    uconstantesSimSEE.CF_DECIMALES);
end;

function TDataTypeBooleanOfCosa.Eval(sval: string; pval: pointer): boolean;
begin
  try
    if not (self.IsAMacroVariable(sval)) then
    begin

      {dfusco@20150508
       Valor por defecto si el sval es nulo.}
      if sval='' then sval:='0';

      boolean(pval^):=StrToBool(sval);
      Result := True;
    end;
  except
    Result := False;
  end;
end;

function TDataTypeBooleanOfCosa.ToString(pval: pointer): string;
begin
  if boolean(pval^) then
    Result := '1'
  else
    Result := '0';
end;

function TDataTypeDAOfDoubleOfCosa.Eval(sval: string; pval: pointer): boolean;
begin
  try
    if not (self.IsAMacroVariable(sval)) then
    begin
      parseDAOfNreal(TDAofNReal(pval^), sval);
      Result := True;
    end
    else
    begin

    end;
  except
    Result := False;
  end;
end;

function TDataTypeDAOfDoubleOfCosa.ToString(pval: pointer): string;
begin
  Result := TDAOfNRealToString(TDAOfNReal(pval^), CF_PRECISION, CF_DECIMALES, ', ', '|');
end;

function TDataTypeDAOfDoubleOfCosa.Format(sval: string): string;
var
  split: TStringList;
begin
  split:=TStringList.Create;
  ExtractStrings([','], [], PChar(sval), split);
  Result:='['+IntToStr(split.Count)+'|'+sval+']';
  split.Free;
end;

function TDataTypeDAOfStringOfCosa.Eval(sval: string; pval: pointer): boolean;
begin
  try
    if not (self.IsAMacroVariable(sval)) then
    begin
      parseDAOfString(TDAOfString(pval^), sval);
      Result := True;
    end
    else
    begin

    end;
  except
    Result := False;
  end;
end;

function TDataTypeDAOfStringOfCosa.ToString(pval: pointer): string;
begin
  Result := TDAOfStringToString(TDAofString(pval^), ', ', '|');
end;

function TDataTypeDAOfStringOfCosa.Format(sval: string): string;
var
  split: TStringList;
begin
  split:=TStringList.Create;
  ExtractStrings([','], [], PChar(sval), split);
  Result:='['+IntToStr(split.Count)+'|'+sval+']';
  split.Free;
end;

function TDataTypeStringListOfCosa.Eval(sval: string; pval: pointer): boolean;
begin
  try
    if not (self.IsAMacroVariable(sval)) then
    begin
      parseStringList(TStringList(pval^), sval);
      Result := True;
    end
    else
    begin

    end;
  except
    Result := False;
  end;
end;

function TDataTypeStringListOfCosa.ToString(pval: pointer): string;
begin
  Result := TStringListToString(TStringList(pval^), ',', '|');
end;

function TDataTypeStringListOfCosa.Format(sval: string): string;
var
  split: TStringList;
begin
  split:=TStringList.Create;
  ExtractStrings([','], [], PChar(sval), split);
  Result:='['+IntToStr(split.Count)+'|'+sval+']';
  split.Free;
end;

function TDataTypeDAOfIntegerOfCosa.Eval(sval: string; pval: pointer): boolean;
begin
  try
    if not (self.IsAMacroVariable(sval)) then
    begin
      parseDAOfNInt(TDAofNInt(pval^), sval);
      Result := True;
    end
    else
    begin

    end;
  except
    Result := False;
  end;
end;

function TDataTypeDAOfIntegerOfCosa.ToString(pval: pointer): string;
begin
  Result:=TDAOfNIntToString(TDAofNInt(pval^), ', ', '|');
end;

function TDataTypeDAOfIntegerOfCosa.Format(sval: string): string;
var
  split: TStringList;
begin
  split:=TStringList.Create;
  ExtractStrings([','], [], PChar(sval), split);
  Result:='['+IntToStr(split.Count)+'|'+sval+']';
  split.Free;
end;

function TDataTypeDAOfBooleanOfCosa.Eval(sval: string; pval: pointer): boolean;
begin
  try
    if not (self.IsAMacroVariable(sval)) then
    begin
      parseDAOfBoolean(TDAOfBoolean(pval^), sval);
      Result := True;
    end
    else
    begin

    end;
  except
    Result := False;
  end;
end;

function TDataTypeDAOfBooleanOfCosa.ToString(pval: pointer): string;
begin
  Result := TDAOfBooleanToString(TDAOfBoolean(pval^), ', ', True, '|');
end;

function TDataTypeDAOfBooleanOfCosa.Format(sval: string): string;
var
  split: TStringList;
begin
  split:=TStringList.Create;
  ExtractStrings([','], [], PChar(sval), split);
  Result:='['+IntToStr(split.Count)+'|'+sval+']';
  split.Free;
end;

function TDataTypeArchiRefOfCosa.Eval(sval: string; pval: pointer): boolean;
begin
  try
    if not (self.IsAMacroVariable(sval)) then
    begin
      TArchiRef(pval^):=TArchiRef.Create(sval);
      Result := True;
    end
    else
    begin

    end;
  except
    Result := False;
  end;
end;

function TDataTypeArchiRefOfCosa.ToString(pval: pointer): string;
begin

end;

function TDataTypeCardinalOfCosa.Eval(sval: string; pval: pointer): boolean;
begin
  try
    if not (self.IsAMacroVariable(sval)) then
    begin
      cardinal(pval^):=StrToInt(sval);
      Result := True;
    end;
  except
    Result := False;
  end;
end;

function TDataTypeCardinalOfCosa.ToString(pval: pointer): string;
begin
  Result := IntToStr(cardinal(pval^));
end;

function TDataTypeReferenciaOfCosa.Eval(sval: string; pval: pointer): boolean;
var
  clase: string='';
  nombreRef: string='';
  TableName, ColumnNameToQuery, CellSValToQuery, ColumnNameToGetSVal: string;
  Childs: TDAofNInt;
  rowId: integer;

begin
  try
    if not (self.IsAMacroVariable(sval)) then
    begin
      parsearReferencia(sval, clase, nombreRef);
      if (clase <> '?') then
      begin
        TableName := 'dt_' + clase;

        writeln(TableName);

        ColumnNameToQuery := 'Nombre';
        CellSValToQuery := nombreRef;

        Childs := self.DataColumnOwner.DataColumnListOwner.DataTableOwner.DataStoreOwner.ReadRowIdsTextValues(TableName,
          ColumnNameToQuery, CellSValToQuery);

        rowId := Childs[0];
        uCosaConNombre.registrar_referencia(nil, clase, nombreRef, pval^);
      end;
      Result := True;
    end;
  except
    Result := False;
  end;
end;

function TDataTypeReferenciaOfCosa.ToString(pval: pointer): string;
begin

end;

begin
  ReferenciasAArchivosExternos := nil;
  lista_caminos := nil;
end.
